
          ********************************************
          *               Intloop22                  *
          *                                          *
          *    Special thermodynamic parameters      *
          *         for 2x2 internal loops           *
          ********************************************

> module Intloop22 where

> import Foldingspace
> import Data.Array
> import Intloop

> il22_energy:: RNAInput -> Int -> Int -> Int
> il22_energy seq lb rb = int22_energy seq lb (lb+1) (lb+2) (lb+3) (rb-3) (rb-2) (rb-1) rb 

Data arrangement: internal loop formed by j,k,o,n framed by pairs (i,p) and (l,m)

     5'.... i j k l ......
            |     |      . 
     3'.... p o n m ......

      _____________
     |      _      |
     |     | |     |
     i j k l m n o p  

> int22_energy:: RNAInput -> Int  -> Int  -> Int  -> Int  -> Int  -> Int  -> Int  -> Int -> Int
> int22_energy seq i j k l m n o p = intloop22 (seq!i) (seq!j) (seq!k) (seq!l)
>					       (seq!m) (seq!n) (seq!o) (seq!p)

> intloop22:: Ebase -> Ebase -> Ebase -> Ebase -> Ebase -> Ebase -> Ebase -> Ebase ->Int 
>					       
> intloop22 C A A G C A A G = 130
> intloop22 C A A G C A C G = 160
> intloop22 C A A G C A G G = 30
> intloop22 C A A G C A U G = 200
> intloop22 C A A G C C A G = 120
> intloop22 C A A G C C C G = 150
> intloop22 C A A G C C G G = 20
> intloop22 C A A G C C U G = 200
> intloop22 C A A G C G A G = 30
> intloop22 C A A G C G C G = 60
> intloop22 C A A G C G G G = -70
> intloop22 C A A G C G U G = 200
> intloop22 C A A G C U A G = 200
> intloop22 C A A G C U C G = 200
> intloop22 C A A G C U G G = 200
> intloop22 C A A G C U U G = 200
> intloop22 C A C G C A A G = 160
> intloop22 C A C G C A C G = 200
> intloop22 C A C G C A G G = 60
> intloop22 C A C G C A U G = 200
> intloop22 C A C G C C A G = 210
> intloop22 C A C G C C C G = 180
> intloop22 C A C G C C G G = 150
> intloop22 C A C G C C U G = 200
> intloop22 C A C G C G A G = 200
> intloop22 C A C G C G C G = 200
> intloop22 C A C G C G G G = 200
> intloop22 C A C G C G U G = 200
> intloop22 C A C G C U A G = 190
> intloop22 C A C G C U C G = 170
> intloop22 C A C G C U G G = 130
> intloop22 C A C G C U U G = 200
> intloop22 C A G G C A A G = 30
> intloop22 C A G G C A C G = 60
> intloop22 C A G G C A G G = -70
> intloop22 C A G G C A U G = 200
> intloop22 C A G G C C A G = 200
> intloop22 C A G G C C C G = 200
> intloop22 C A G G C C G G = 200
> intloop22 C A G G C C U G = 200
> intloop22 C A G G C G A G = 100
> intloop22 C A G G C G C G = 140
> intloop22 C A G G C G G G = 0
> intloop22 C A G G C G U G = 200
> intloop22 C A G G C U A G = -40
> intloop22 C A G G C U C G = -110
> intloop22 C A G G C U G G = -60
> intloop22 C A G G C U U G = 200
> intloop22 C A U G C A A G = 200
> intloop22 C A U G C A C G = 200
> intloop22 C A U G C A G G = 200
> intloop22 C A U G C A U G = 200
> intloop22 C A U G C C A G = 190
> intloop22 C A U G C C C G = 170
> intloop22 C A U G C C G G = 130
> intloop22 C A U G C C U G = 200
> intloop22 C A U G C G A G = 110
> intloop22 C A U G C G C G = 40
> intloop22 C A U G C G G G = 90
> intloop22 C A U G C G U G = 200
> intloop22 C A U G C U A G = 140
> intloop22 C A U G C U C G = 80
> intloop22 C A U G C U G G = 130
> intloop22 C A U G C U U G = 200
> intloop22 C C A G C A A G = 120
> intloop22 C C A G C A C G = 210
> intloop22 C C A G C A G G = 200
> intloop22 C C A G C A U G = 190
> intloop22 C C A G C C A G = 110
> intloop22 C C A G C C C G = 140
> intloop22 C C A G C C G G = 200
> intloop22 C C A G C C U G = 120
> intloop22 C C A G C G A G = 20
> intloop22 C C A G C G C G = 150
> intloop22 C C A G C G G G = 200
> intloop22 C C A G C G U G = 130
> intloop22 C C A G C U A G = 200
> intloop22 C C A G C U C G = 200
> intloop22 C C A G C U G G = 200
> intloop22 C C A G C U U G = 200
> intloop22 C C C G C A A G = 150
> intloop22 C C C G C A C G = 180
> intloop22 C C C G C A G G = 200
> intloop22 C C C G C A U G = 170
> intloop22 C C C G C C A G = 140
> intloop22 C C C G C C C G = 170
> intloop22 C C C G C C G G = 200
> intloop22 C C C G C C U G = 150
> intloop22 C C C G C G A G = 200
> intloop22 C C C G C G C G = 200
> intloop22 C C C G C G G G = 200
> intloop22 C C C G C G U G = 200
> intloop22 C C C G C U A G = 120
> intloop22 C C C G C U C G = 150
> intloop22 C C C G C U G G = 200
> intloop22 C C C G C U U G = 140
> intloop22 C C G G C A A G = 20
> intloop22 C C G G C A C G = 150
> intloop22 C C G G C A G G = 200
> intloop22 C C G G C A U G = 130
> intloop22 C C G G C C A G = 200
> intloop22 C C G G C C C G = 200
> intloop22 C C G G C C G G = 200
> intloop22 C C G G C C U G = 200
> intloop22 C C G G C G A G = 90
> intloop22 C C G G C G C G = 180
> intloop22 C C G G C G G G = 200
> intloop22 C C G G C G U G = 170
> intloop22 C C G G C U A G = -150
> intloop22 C C G G C U C G = -20
> intloop22 C C G G C U G G = 200
> intloop22 C C G G C U U G = -40
> intloop22 C C U G C A A G = 200
> intloop22 C C U G C A C G = 200
> intloop22 C C U G C A G G = 200
> intloop22 C C U G C A U G = 200
> intloop22 C C U G C C A G = 120
> intloop22 C C U G C C C G = 150
> intloop22 C C U G C C G G = 200
> intloop22 C C U G C C U G = 140
> intloop22 C C U G C G A G = 0
> intloop22 C C U G C G C G = 130
> intloop22 C C U G C G G G = 200
> intloop22 C C U G C G U G = 110
> intloop22 C C U G C U A G = 30
> intloop22 C C U G C U C G = 60
> intloop22 C C U G C U G G = 200
> intloop22 C C U G C U U G = 50
> intloop22 C G A G C A A G = 30
> intloop22 C G A G C A C G = 200
> intloop22 C G A G C A G G = 100
> intloop22 C G A G C A U G = 110
> intloop22 C G A G C C A G = 20
> intloop22 C G A G C C C G = 200
> intloop22 C G A G C C G G = 90
> intloop22 C G A G C C U G = 0
> intloop22 C G A G C G A G = -70
> intloop22 C G A G C G C G = 200
> intloop22 C G A G C G G G = 0
> intloop22 C G A G C G U G = 90
> intloop22 C G A G C U A G = 200
> intloop22 C G A G C U C G = 200
> intloop22 C G A G C U G G = 200
> intloop22 C G A G C U U G = 200
> intloop22 C G C G C A A G = 60
> intloop22 C G C G C A C G = 200
> intloop22 C G C G C A G G = 140
> intloop22 C G C G C A U G = 40
> intloop22 C G C G C C A G = 150
> intloop22 C G C G C C C G = 200
> intloop22 C G C G C C G G = 180
> intloop22 C G C G C C U G = 130
> intloop22 C G C G C G A G = 200
> intloop22 C G C G C G C G = 200
> intloop22 C G C G C G G G = 200
> intloop22 C G C G C G U G = 200
> intloop22 C G C G C U A G = 130
> intloop22 C G C G C U C G = 200
> intloop22 C G C G C U G G = 170
> intloop22 C G C G C U U G = 110
> intloop22 C G G G C A A G = -70
> intloop22 C G G G C A C G = 200
> intloop22 C G G G C A G G = 0
> intloop22 C G G G C A U G = 90
> intloop22 C G G G C C A G = 200
> intloop22 C G G G C C C G = 200
> intloop22 C G G G C C G G = 200
> intloop22 C G G G C C U G = 200
> intloop22 C G G G C G A G = 0
> intloop22 C G G G C G C G = 200
> intloop22 C G G G C G G G = 80
> intloop22 C G G G C G U G = 90
> intloop22 C G G G C U A G = -60
> intloop22 C G G G C U C G = 200
> intloop22 C G G G C U G G = -70
> intloop22 C G G G C U U G = -260
> intloop22 C G U G C A A G = 200
> intloop22 C G U G C A C G = 200
> intloop22 C G U G C A G G = 200
> intloop22 C G U G C A U G = 200
> intloop22 C G U G C C A G = 130
> intloop22 C G U G C C C G = 200
> intloop22 C G U G C C G G = 170
> intloop22 C G U G C C U G = 110
> intloop22 C G U G C G A G = 90
> intloop22 C G U G C G C G = 200
> intloop22 C G U G C G G G = 90
> intloop22 C G U G C G U G = -110
> intloop22 C G U G C U A G = 130
> intloop22 C G U G C U C G = 200
> intloop22 C G U G C U G G = 120
> intloop22 C G U G C U U G = 110
> intloop22 C U A G C A A G = 200
> intloop22 C U A G C A C G = 190
> intloop22 C U A G C A G G = -40
> intloop22 C U A G C A U G = 140
> intloop22 C U A G C C A G = 200
> intloop22 C U A G C C C G = 120
> intloop22 C U A G C C G G = -150
> intloop22 C U A G C C U G = 30
> intloop22 C U A G C G A G = 200
> intloop22 C U A G C G C G = 130
> intloop22 C U A G C G G G = -60
> intloop22 C U A G C G U G = 130
> intloop22 C U A G C U A G = 200
> intloop22 C U A G C U C G = 200
> intloop22 C U A G C U G G = 200
> intloop22 C U A G C U U G = 200
> intloop22 C U C G C A A G = 200
> intloop22 C U C G C A C G = 170
> intloop22 C U C G C A G G = -110
> intloop22 C U C G C A U G = 80
> intloop22 C U C G C C A G = 200
> intloop22 C U C G C C C G = 150
> intloop22 C U C G C C G G = -20
> intloop22 C U C G C C U G = 60
> intloop22 C U C G C G A G = 200
> intloop22 C U C G C G C G = 200
> intloop22 C U C G C G G G = 200
> intloop22 C U C G C G U G = 200
> intloop22 C U C G C U A G = 200
> intloop22 C U C G C U C G = 140
> intloop22 C U C G C U G G = -40
> intloop22 C U C G C U U G = 50
> intloop22 C U G G C A A G = 200
> intloop22 C U G G C A C G = 130
> intloop22 C U G G C A G G = -60
> intloop22 C U G G C A U G = 130
> intloop22 C U G G C C A G = 200
> intloop22 C U G G C C C G = 200
> intloop22 C U G G C C G G = 200
> intloop22 C U G G C C U G = 200
> intloop22 C U G G C G A G = 200
> intloop22 C U G G C G C G = 170
> intloop22 C U G G C G G G = -70
> intloop22 C U G G C G U G = 120
> intloop22 C U G G C U A G = 200
> intloop22 C U G G C U C G = -40
> intloop22 C U G G C U G G = -420
> intloop22 C U G G C U U G = -50
> intloop22 C U U G C A A G = 200
> intloop22 C U U G C A C G = 200
> intloop22 C U U G C A G G = 200
> intloop22 C U U G C A U G = 200
> intloop22 C U U G C C A G = 200
> intloop22 C U U G C C C G = 140
> intloop22 C U U G C C G G = -40
> intloop22 C U U G C C U G = 50
> intloop22 C U U G C G A G = 200
> intloop22 C U U G C G C G = 110
> intloop22 C U U G C G G G = -260
> intloop22 C U U G C G U G = 110
> intloop22 C U U G C U A G = 200
> intloop22 C U U G C U C G = 50
> intloop22 C U U G C U G G = -50
> intloop22 C U U G C U U G = -40
> intloop22 C A A C G A A G = 50
> intloop22 C A A C G A C G = 60
> intloop22 C A A C G A G G = 0
> intloop22 C A A C G A U G = 200
> intloop22 C A A C G C A G = 110
> intloop22 C A A C G C C G = 150
> intloop22 C A A C G C G G = -70
> intloop22 C A A C G C U G = 200
> intloop22 C A A C G G A G = -30
> intloop22 C A A C G G C G = 10
> intloop22 C A A C G G G G = -160
> intloop22 C A A C G G U G = 200
> intloop22 C A A C G U A G = 200
> intloop22 C A A C G U C G = 200
> intloop22 C A A C G U G G = 200
> intloop22 C A A C G U U G = 200
> intloop22 C A C C G A A G = 110
> intloop22 C A C C G A C G = 110
> intloop22 C A C C G A G G = -100
> intloop22 C A C C G A U G = 200
> intloop22 C A C C G C A G = 170
> intloop22 C A C C G C C G = 150
> intloop22 C A C C G C G G = -60
> intloop22 C A C C G C U G = 200
> intloop22 C A C C G G A G = 200
> intloop22 C A C C G G C G = 200
> intloop22 C A C C G G G G = 200
> intloop22 C A C C G G U G = 200
> intloop22 C A C C G U A G = 70
> intloop22 C A C C G U C G = 50
> intloop22 C A C C G U G G = 20
> intloop22 C A C C G U U G = 200
> intloop22 C A G C G A A G = 40
> intloop22 C A G C G A C G = 50
> intloop22 C A G C G A G G = -70
> intloop22 C A G C G A U G = 200
> intloop22 C A G C G C A G = 200
> intloop22 C A G C G C C G = 200
> intloop22 C A G C G C G G = 200
> intloop22 C A G C G C U G = 200
> intloop22 C A G C G G A G = 100
> intloop22 C A G C G G C G = 140
> intloop22 C A G C G G G G = 0
> intloop22 C A G C G G U G = 200
> intloop22 C A G C G U A G = 10
> intloop22 C A G C G U C G = -70
> intloop22 C A G C G U G G = -80
> intloop22 C A G C G U U G = 200
> intloop22 C A U C G A A G = 200
> intloop22 C A U C G A C G = 200
> intloop22 C A U C G A G G = 200
> intloop22 C A U C G A U G = 200
> intloop22 C A U C G C A G = 180
> intloop22 C A U C G C C G = 150
> intloop22 C A U C G C G G = 120
> intloop22 C A U C G C U G = 200
> intloop22 C A U C G G A G = -50
> intloop22 C A U C G G C G = -60
> intloop22 C A U C G G G G = -60
> intloop22 C A U C G G U G = 200
> intloop22 C A U C G U A G = 150
> intloop22 C A U C G U C G = 0
> intloop22 C A U C G U G G = 90
> intloop22 C A U C G U U G = 200
> intloop22 C C A C G A A G = 130
> intloop22 C C A C G A C G = 220
> intloop22 C C A C G A G G = 200
> intloop22 C C A C G A U G = 200
> intloop22 C C A C G C A G = 100
> intloop22 C C A C G C C G = 130
> intloop22 C C A C G C G G = 200
> intloop22 C C A C G C U G = 120
> intloop22 C C A C G G A G = -70
> intloop22 C C A C G G C G = 70
> intloop22 C C A C G G G G = 200
> intloop22 C C A C G G U G = 40
> intloop22 C C A C G U A G = 200
> intloop22 C C A C G U C G = 200
> intloop22 C C A C G U G G = 200
> intloop22 C C A C G U U G = 200
> intloop22 C C C C G A A G = 100
> intloop22 C C C C G A C G = 190
> intloop22 C C C C G A G G = 200
> intloop22 C C C C G A U G = 110
> intloop22 C C C C G C A G = 100
> intloop22 C C C C G C C G = 130
> intloop22 C C C C G C G G = 200
> intloop22 C C C C G C U G = 120
> intloop22 C C C C G G A G = 200
> intloop22 C C C C G G C G = 200
> intloop22 C C C C G G G G = 200
> intloop22 C C C C G G U G = 200
> intloop22 C C C C G U A G = 0
> intloop22 C C C C G U C G = 30
> intloop22 C C C C G U G G = 200
> intloop22 C C C C G U U G = 170
> intloop22 C C G C G A A G = 70
> intloop22 C C G C G A C G = 70
> intloop22 C C G C G A G G = 200
> intloop22 C C G C G A U G = 100
> intloop22 C C G C G C A G = 200
> intloop22 C C G C G C C G = 200
> intloop22 C C G C G C G G = 200
> intloop22 C C G C G C U G = 200
> intloop22 C C G C G G A G = 90
> intloop22 C C G C G G C G = 180
> intloop22 C C G C G G G G = 200
> intloop22 C C G C G G U G = 170
> intloop22 C C G C G U A G = -190
> intloop22 C C G C G U C G = -30
> intloop22 C C G C G U G G = 200
> intloop22 C C G C G U U G = -70
> intloop22 C C U C G A A G = 200
> intloop22 C C U C G A C G = 200
> intloop22 C C U C G A G G = 200
> intloop22 C C U C G A U G = 200
> intloop22 C C U C G C A G = 110
> intloop22 C C U C G C C G = 140
> intloop22 C C U C G C G G = 200
> intloop22 C C U C G C U G = 120
> intloop22 C C U C G G A G = -150
> intloop22 C C U C G G C G = -20
> intloop22 C C U C G G G G = 200
> intloop22 C C U C G G U G = -30
> intloop22 C C U C G U A G = -20
> intloop22 C C U C G U C G = -10
> intloop22 C C U C G U G G = 200
> intloop22 C C U C G U U G = 20
> intloop22 C G A C G A A G = -20
> intloop22 C G A C G A C G = 200
> intloop22 C G A C G A G G = 110
> intloop22 C G A C G A U G = 90
> intloop22 C G A C G C A G = -40
> intloop22 C G A C G C C G = 200
> intloop22 C G A C G C G G = 90
> intloop22 C G A C G C U G = 0
> intloop22 C G A C G G A G = -170
> intloop22 C G A C G G C G = 200
> intloop22 C G A C G G G G = -90
> intloop22 C G A C G G U G = 30
> intloop22 C G A C G U A G = 200
> intloop22 C G A C G U C G = 200
> intloop22 C G A C G U G G = 200
> intloop22 C G A C G U U G = 200
> intloop22 C G C C G A A G = 70
> intloop22 C G C C G A C G = 200
> intloop22 C G C C G A G G = 80
> intloop22 C G C C G A U G = -10
> intloop22 C G C C G C A G = 110
> intloop22 C G C C G C C G = 200
> intloop22 C G C C G C G G = 150
> intloop22 C G C C G C U G = 100
> intloop22 C G C C G G A G = 200
> intloop22 C G C C G G C G = 200
> intloop22 C G C C G G G G = 200
> intloop22 C G C C G G U G = 200
> intloop22 C G C C G U A G = 20
> intloop22 C G C C G U C G = 200
> intloop22 C G C C G U G G = 50
> intloop22 C G C C G U U G = 0
> intloop22 C G G C G A A G = -50
> intloop22 C G G C G A C G = 200
> intloop22 C G G C G A G G = -20
> intloop22 C G G C G A U G = 60
> intloop22 C G G C G C A G = 200
> intloop22 C G G C G C C G = 200
> intloop22 C G G C G C G G = 200
> intloop22 C G G C G C U G = 200
> intloop22 C G G C G G A G = 0
> intloop22 C G G C G G C G = 200
> intloop22 C G G C G G G G = 80
> intloop22 C G G C G G U G = 90
> intloop22 C G G C G U A G = -90
> intloop22 C G G C G U C G = 200
> intloop22 C G G C G U G G = -100
> intloop22 C G G C G U U G = -300
> intloop22 C G U C G A A G = 200
> intloop22 C G U C G A C G = 200
> intloop22 C G U C G A G G = 200
> intloop22 C G U C G A U G = 200
> intloop22 C G U C G C A G = 120
> intloop22 C G U C G C C G = 200
> intloop22 C G U C G C G G = 150
> intloop22 C G U C G C U G = 100
> intloop22 C G U C G G A G = -130
> intloop22 C G U C G G C G = 200
> intloop22 C G U C G G G G = -60
> intloop22 C G U C G G U G = -240
> intloop22 C G U C G U A G = 90
> intloop22 C G U C G U C G = 200
> intloop22 C G U C G U G G = 110
> intloop22 C G U C G U U G = 60
> intloop22 C U A C G A A G = 200
> intloop22 C U A C G A C G = 200
> intloop22 C U A C G A G G = -10
> intloop22 C U A C G A U G = 140
> intloop22 C U A C G C A G = 200
> intloop22 C U A C G C C G = 120
> intloop22 C U A C G C G G = -160
> intloop22 C U A C G C U G = 30
> intloop22 C U A C G G A G = 200
> intloop22 C U A C G G C G = 40
> intloop22 C U A C G G G G = -160
> intloop22 C U A C G G U G = 50
> intloop22 C U A C G U A G = 200
> intloop22 C U A C G U C G = 200
> intloop22 C U A C G U G G = 200
> intloop22 C U A C G U U G = 200
> intloop22 C U C C G A A G = 200
> intloop22 C U C C G A C G = 110
> intloop22 C U C C G A G G = -160
> intloop22 C U C C G A U G = 30
> intloop22 C U C C G C A G = 200
> intloop22 C U C C G C C G = 120
> intloop22 C U C C G C G G = -60
> intloop22 C U C C G C U G = 30
> intloop22 C U C C G G A G = 200
> intloop22 C U C C G G C G = 200
> intloop22 C U C C G G G G = 200
> intloop22 C U C C G G U G = 200
> intloop22 C U C C G U A G = 200
> intloop22 C U C C G U C G = 20
> intloop22 C U C C G U G G = -160
> intloop22 C U C C G U U G = 10
> intloop22 C U G C G A A G = 200
> intloop22 C U G C G A C G = 50
> intloop22 C U G C G A G G = -60
> intloop22 C U G C G A U G = 140
> intloop22 C U G C G C A G = 200
> intloop22 C U G C G C C G = 200
> intloop22 C U G C G C G G = 200
> intloop22 C U G C G C U G = 200
> intloop22 C U G C G G A G = 200
> intloop22 C U G C G G C G = 170
> intloop22 C U G C G G G G = -70
> intloop22 C U G C G G U G = 120
> intloop22 C U G C G U A G = 200
> intloop22 C U G C G U C G = -70
> intloop22 C U G C G U G G = -440
> intloop22 C U G C G U U G = -100
> intloop22 C U U C G A A G = 200
> intloop22 C U U C G A C G = 200
> intloop22 C U U C G A G G = 200
> intloop22 C U U C G A U G = 200
> intloop22 C U U C G C A G = 200
> intloop22 C U U C G C C G = 120
> intloop22 C U U C G C G G = -50
> intloop22 C U U C G C U G = 30
> intloop22 C U U C G G A G = 200
> intloop22 C U U C G G C G = -10
> intloop22 C U U C G G G G = -410
> intloop22 C U U C G G U G = 10
> intloop22 C U U C G U A G = 200
> intloop22 C U U C G U C G = 40
> intloop22 C U U C G U G G = -100
> intloop22 C U U C G U U G = 60
> intloop22 C A A U G A A G = 200
> intloop22 C A A U G A C G = 240
> intloop22 C A A U G A G G = 100
> intloop22 C A A U G A U G = 200
> intloop22 C A A U G C A G = 180
> intloop22 C A A U G C C G = 210
> intloop22 C A A U G C G G = 80
> intloop22 C A A U G C U G = 200
> intloop22 C A A U G G A G = 80
> intloop22 C A A U G G C G = 110
> intloop22 C A A U G G G G = -20
> intloop22 C A A U G G U G = 200
> intloop22 C A A U G U A G = 200
> intloop22 C A A U G U C G = 200
> intloop22 C A A U G U G G = 200
> intloop22 C A A U G U U G = 200
> intloop22 C A C U G A A G = 190
> intloop22 C A C U G A C G = 220
> intloop22 C A C U G A G G = 90
> intloop22 C A C U G A U G = 200
> intloop22 C A C U G C A G = 230
> intloop22 C A C U G C C G = 210
> intloop22 C A C U G C G G = 170
> intloop22 C A C U G C U G = 200
> intloop22 C A C U G G A G = 200
> intloop22 C A C U G G C G = 200
> intloop22 C A C U G G G G = 200
> intloop22 C A C U G G U G = 200
> intloop22 C A C U G U A G = 230
> intloop22 C A C U G U C G = 210
> intloop22 C A C U G U G G = 170
> intloop22 C A C U G U U G = 200
> intloop22 C A G U G A A G = 80
> intloop22 C A G U G A C G = 110
> intloop22 C A G U G A G G = -20
> intloop22 C A G U G A U G = 200
> intloop22 C A G U G C A G = 200
> intloop22 C A G U G C C G = 200
> intloop22 C A G U G C G G = 200
> intloop22 C A G U G C U G = 200
> intloop22 C A G U G G A G = 130
> intloop22 C A G U G G C G = 170
> intloop22 C A G U G G G G = 30
> intloop22 C A G U G G U G = 200
> intloop22 C A G U G U A G = 60
> intloop22 C A G U G U C G = 0
> intloop22 C A G U G U G G = 40
> intloop22 C A G U G U U G = 200
> intloop22 C A U U G A A G = 200
> intloop22 C A U U G A C G = 200
> intloop22 C A U U G A G G = 200
> intloop22 C A U U G A U G = 200
> intloop22 C A U U G C A G = 230
> intloop22 C A U U G C C G = 210
> intloop22 C A U U G C G G = 170
> intloop22 C A U U G C U G = 200
> intloop22 C A U U G G A G = 160
> intloop22 C A U U G G C G = 90
> intloop22 C A U U G G G G = 140
> intloop22 C A U U G G U G = 200
> intloop22 C A U U G U A G = 190
> intloop22 C A U U G U C G = 130
> intloop22 C A U U G U G G = 180
> intloop22 C A U U G U U G = 200
> intloop22 C C A U G A A G = 190
> intloop22 C C A U G A C G = 280
> intloop22 C C A U G A G G = 200
> intloop22 C C A U G A U G = 270
> intloop22 C C A U G C A G = 170
> intloop22 C C A U G C C G = 200
> intloop22 C C A U G C G G = 200
> intloop22 C C A U G C U G = 180
> intloop22 C C A U G G A G = 70
> intloop22 C C A U G G C G = 200
> intloop22 C C A U G G G G = 200
> intloop22 C C A U G G U G = 180
> intloop22 C C A U G U A G = 200
> intloop22 C C A U G U C G = 200
> intloop22 C C A U G U G G = 200
> intloop22 C C A U G U U G = 200
> intloop22 C C C U G A A G = 180
> intloop22 C C C U G A C G = 210
> intloop22 C C C U G A G G = 200
> intloop22 C C C U G A U G = 190
> intloop22 C C C U G C A G = 160
> intloop22 C C C U G C C G = 190
> intloop22 C C C U G C G G = 200
> intloop22 C C C U G C U G = 180
> intloop22 C C C U G G A G = 200
> intloop22 C C C U G G C G = 200
> intloop22 C C C U G G G G = 200
> intloop22 C C C U G G U G = 200
> intloop22 C C C U G U A G = 160
> intloop22 C C C U G U C G = 190
> intloop22 C C C U G U G G = 200
> intloop22 C C C U G U U G = 180
> intloop22 C C G U G A A G = 70
> intloop22 C C G U G A C G = 200
> intloop22 C C G U G A G G = 200
> intloop22 C C G U G A U G = 180
> intloop22 C C G U G C A G = 200
> intloop22 C C G U G C C G = 200
> intloop22 C C G U G C G G = 200
> intloop22 C C G U G C U G = 200
> intloop22 C C G U G G A G = 120
> intloop22 C C G U G G C G = 210
> intloop22 C C G U G G G G = 200
> intloop22 C C G U G G U G = 200
> intloop22 C C G U G U A G = -50
> intloop22 C C G U G U C G = 80
> intloop22 C C G U G U G G = 200
> intloop22 C C G U G U U G = 70
> intloop22 C C U U G A A G = 200
> intloop22 C C U U G A C G = 200
> intloop22 C C U U G A G G = 200
> intloop22 C C U U G A U G = 200
> intloop22 C C U U G C A G = 160
> intloop22 C C U U G C C G = 190
> intloop22 C C U U G C G G = 200
> intloop22 C C U U G C U G = 180
> intloop22 C C U U G G A G = 50
> intloop22 C C U U G G C G = 180
> intloop22 C C U U G G G G = 200
> intloop22 C C U U G G U G = 160
> intloop22 C C U U G U A G = 80
> intloop22 C C U U G U C G = 110
> intloop22 C C U U G U G G = 200
> intloop22 C C U U G U U G = 100
> intloop22 C G A U G A A G = 100
> intloop22 C G A U G A C G = 200
> intloop22 C G A U G A G G = 180
> intloop22 C G A U G A U G = 180
> intloop22 C G A U G C A G = 80
> intloop22 C G A U G C C G = 200
> intloop22 C G A U G C G G = 150
> intloop22 C G A U G C U G = 60
> intloop22 C G A U G G A G = -20
> intloop22 C G A U G G C G = 200
> intloop22 C G A U G G G G = 50
> intloop22 C G A U G G U G = 140
> intloop22 C G A U G U A G = 200
> intloop22 C G A U G U C G = 200
> intloop22 C G A U G U G G = 200
> intloop22 C G A U G U U G = 200
> intloop22 C G C U G A A G = 90
> intloop22 C G C U G A C G = 200
> intloop22 C G C U G A G G = 160
> intloop22 C G C U G A U G = 70
> intloop22 C G C U G C A G = 170
> intloop22 C G C U G C C G = 200
> intloop22 C G C U G C G G = 210
> intloop22 C G C U G C U G = 150
> intloop22 C G C U G G A G = 200
> intloop22 C G C U G G C G = 200
> intloop22 C G C U G G G G = 200
> intloop22 C G C U G G U G = 200
> intloop22 C G C U G U A G = 170
> intloop22 C G C U G U C G = 200
> intloop22 C G C U G U G G = 210
> intloop22 C G C U G U U G = 150
> intloop22 C G G U G A A G = -20
> intloop22 C G G U G A C G = 200
> intloop22 C G G U G A G G = 50
> intloop22 C G G U G A U G = 140
> intloop22 C G G U G C A G = 200
> intloop22 C G G U G C C G = 200
> intloop22 C G G U G C G G = 200
> intloop22 C G G U G C U G = 200
> intloop22 C G G U G G A G = 30
> intloop22 C G G U G G C G = 200
> intloop22 C G G U G G G G = 110
> intloop22 C G G U G G U G = 110
> intloop22 C G G U G U A G = 40
> intloop22 C G G U G U C G = 200
> intloop22 C G G U G U G G = 40
> intloop22 C G G U G U U G = -160
> intloop22 C G U U G A A G = 200
> intloop22 C G U U G A C G = 200
> intloop22 C G U U G A G G = 200
> intloop22 C G U U G A U G = 200
> intloop22 C G U U G C A G = 170
> intloop22 C G U U G C C G = 200
> intloop22 C G U U G C G G = 210
> intloop22 C G U U G C U G = 150
> intloop22 C G U U G G A G = 140
> intloop22 C G U U G G C G = 200
> intloop22 C G U U G G G G = 130
> intloop22 C G U U G G U G = -60
> intloop22 C G U U G U A G = 180
> intloop22 C G U U G U C G = 200
> intloop22 C G U U G U G G = 170
> intloop22 C G U U G U U G = 160
> intloop22 C U A U G A A G = 200
> intloop22 C U A U G A C G = 270
> intloop22 C U A U G A G G = 30
> intloop22 C U A U G A U G = 220
> intloop22 C U A U G C A G = 200
> intloop22 C U A U G C C G = 180
> intloop22 C U A U G C G G = -90
> intloop22 C U A U G C U G = 90
> intloop22 C U A U G G A G = 200
> intloop22 C U A U G G C G = 180
> intloop22 C U A U G G G G = -10
> intloop22 C U A U G G U G = 180
> intloop22 C U A U G U A G = 200
> intloop22 C U A U G U C G = 200
> intloop22 C U A U G U G G = 200
> intloop22 C U A U G U U G = 200
> intloop22 C U C U G A A G = 200
> intloop22 C U C U G A C G = 190
> intloop22 C U C U G A G G = -80
> intloop22 C U C U G A U G = 100
> intloop22 C U C U G C A G = 200
> intloop22 C U C U G C C G = 180
> intloop22 C U C U G C G G = 0
> intloop22 C U C U G C U G = 90
> intloop22 C U C U G G A G = 200
> intloop22 C U C U G G C G = 200
> intloop22 C U C U G G G G = 200
> intloop22 C U C U G G U G = 200
> intloop22 C U C U G U A G = 200
> intloop22 C U C U G U C G = 180
> intloop22 C U C U G U G G = 0
> intloop22 C U C U G U U G = 90
> intloop22 C U G U G A A G = 200
> intloop22 C U G U G A C G = 180
> intloop22 C U G U G A G G = -10
> intloop22 C U G U G A U G = 180
> intloop22 C U G U G C A G = 200
> intloop22 C U G U G C C G = 200
> intloop22 C U G U G C G G = 200
> intloop22 C U G U G C U G = 200
> intloop22 C U G U G G A G = 200
> intloop22 C U G U G G C G = 200
> intloop22 C U G U G G G G = -40
> intloop22 C U G U G G U G = 150
> intloop22 C U G U G U A G = 200
> intloop22 C U G U G U C G = 70
> intloop22 C U G U G U G G = -310
> intloop22 C U G U G U U G = 60
> intloop22 C U U U G A A G = 200
> intloop22 C U U U G A C G = 200
> intloop22 C U U U G A G G = 200
> intloop22 C U U U G A U G = 200
> intloop22 C U U U G C A G = 200
> intloop22 C U U U G C C G = 180
> intloop22 C U U U G C G G = 0
> intloop22 C U U U G C U G = 90
> intloop22 C U U U G G A G = 200
> intloop22 C U U U G G C G = 160
> intloop22 C U U U G G G G = -210
> intloop22 C U U U G G U G = 160
> intloop22 C U U U G U A G = 200
> intloop22 C U U U G U C G = 100
> intloop22 C U U U G U G G = 0
> intloop22 C U U U G U U G = 10
> intloop22 C A A G U A A G = 200
> intloop22 C A A G U A C G = 240
> intloop22 C A A G U A G G = 100
> intloop22 C A A G U A U G = 200
> intloop22 C A A G U C A G = 160
> intloop22 C A A G U C C G = 190
> intloop22 C A A G U C G G = 60
> intloop22 C A A G U C U G = 200
> intloop22 C A A G U G A G = 100
> intloop22 C A A G U G C G = 130
> intloop22 C A A G U G G G = 0
> intloop22 C A A G U G U G = 200
> intloop22 C A A G U U A G = 200
> intloop22 C A A G U U C G = 200
> intloop22 C A A G U U G G = 200
> intloop22 C A A G U U U G = 200
> intloop22 C A C G U A A G = 200
> intloop22 C A C G U A C G = 240
> intloop22 C A C G U A G G = 100
> intloop22 C A C G U A U G = 200
> intloop22 C A C G U C A G = 260
> intloop22 C A C G U C C G = 240
> intloop22 C A C G U C G G = 200
> intloop22 C A C G U C U G = 200
> intloop22 C A C G U G A G = 200
> intloop22 C A C G U G C G = 200
> intloop22 C A C G U G G G = 200
> intloop22 C A C G U G U G = 200
> intloop22 C A C G U U A G = 260
> intloop22 C A C G U U C G = 240
> intloop22 C A C G U U G G = 200
> intloop22 C A C G U U U G = 200
> intloop22 C A G G U A A G = 100
> intloop22 C A G G U A C G = 130
> intloop22 C A G G U A G G = 0
> intloop22 C A G G U A U G = 200
> intloop22 C A G G U C A G = 200
> intloop22 C A G G U C C G = 200
> intloop22 C A G G U C G G = 200
> intloop22 C A G G U C U G = 200
> intloop22 C A G G U G A G = 140
> intloop22 C A G G U G C G = 170
> intloop22 C A G G U G G G = 40
> intloop22 C A G G U G U G = 200
> intloop22 C A G G U U A G = 20
> intloop22 C A G G U U C G = -40
> intloop22 C A G G U U G G = 0
> intloop22 C A G G U U U G = 200
> intloop22 C A U G U A A G = 200
> intloop22 C A U G U A C G = 200
> intloop22 C A U G U A G G = 200
> intloop22 C A U G U A U G = 200
> intloop22 C A U G U C A G = 230
> intloop22 C A U G U C C G = 210
> intloop22 C A U G U C G G = 170
> intloop22 C A U G U C U G = 200
> intloop22 C A U G U G A G = 150
> intloop22 C A U G U G C G = 80
> intloop22 C A U G U G G G = 130
> intloop22 C A U G U G U G = 200
> intloop22 C A U G U U A G = 220
> intloop22 C A U G U U C G = 150
> intloop22 C A U G U U G G = 200
> intloop22 C A U G U U U G = 200
> intloop22 C C A G U A A G = 190
> intloop22 C C A G U A C G = 280
> intloop22 C C A G U A G G = 200
> intloop22 C C A G U A U G = 270
> intloop22 C C A G U C A G = 150
> intloop22 C C A G U C C G = 180
> intloop22 C C A G U C G G = 200
> intloop22 C C A G U C U G = 160
> intloop22 C C A G U G A G = 90
> intloop22 C C A G U G C G = 220
> intloop22 C C A G U G G G = 200
> intloop22 C C A G U G U G = 200
> intloop22 C C A G U U A G = 200
> intloop22 C C A G U U C G = 200
> intloop22 C C A G U U G G = 200
> intloop22 C C A G U U U G = 200
> intloop22 C C C G U A A G = 190
> intloop22 C C C G U A C G = 220
> intloop22 C C C G U A G G = 200
> intloop22 C C C G U A U G = 210
> intloop22 C C C G U C A G = 190
> intloop22 C C C G U C C G = 220
> intloop22 C C C G U C G G = 200
> intloop22 C C C G U C U G = 210
> intloop22 C C C G U G A G = 200
> intloop22 C C C G U G C G = 200
> intloop22 C C C G U G G G = 200
> intloop22 C C C G U G U G = 200
> intloop22 C C C G U U A G = 190
> intloop22 C C C G U U C G = 220
> intloop22 C C C G U U G G = 200
> intloop22 C C C G U U U G = 210
> intloop22 C C G G U A A G = 90
> intloop22 C C G G U A C G = 220
> intloop22 C C G G U A G G = 200
> intloop22 C C G G U A U G = 200
> intloop22 C C G G U C A G = 200
> intloop22 C C G G U C C G = 200
> intloop22 C C G G U C G G = 200
> intloop22 C C G G U C U G = 200
> intloop22 C C G G U G A G = 130
> intloop22 C C G G U G C G = 220
> intloop22 C C G G U G G G = 200
> intloop22 C C G G U G U G = 200
> intloop22 C C G G U U A G = -90
> intloop22 C C G G U U C G = 40
> intloop22 C C G G U U G G = 200
> intloop22 C C G G U U U G = 30
> intloop22 C C U G U A A G = 200
> intloop22 C C U G U A C G = 200
> intloop22 C C U G U A G G = 200
> intloop22 C C U G U A U G = 200
> intloop22 C C U G U C A G = 160
> intloop22 C C U G U C C G = 190
> intloop22 C C U G U C G G = 200
> intloop22 C C U G U C U G = 180
> intloop22 C C U G U G A G = 40
> intloop22 C C U G U G C G = 170
> intloop22 C C U G U G G G = 200
> intloop22 C C U G U G U G = 150
> intloop22 C C U G U U A G = 110
> intloop22 C C U G U U C G = 140
> intloop22 C C U G U U G G = 200
> intloop22 C C U G U U U G = 120
> intloop22 C G A G U A A G = 100
> intloop22 C G A G U A C G = 200
> intloop22 C G A G U A G G = 180
> intloop22 C G A G U A U G = 180
> intloop22 C G A G U C A G = 60
> intloop22 C G A G U C C G = 200
> intloop22 C G A G U C G G = 130
> intloop22 C G A G U C U G = 40
> intloop22 C G A G U G A G = 0
> intloop22 C G A G U G C G = 200
> intloop22 C G A G U G G G = 70
> intloop22 C G A G U G U G = 160
> intloop22 C G A G U U A G = 200
> intloop22 C G A G U U C G = 200
> intloop22 C G A G U U G G = 200
> intloop22 C G A G U U U G = 200
> intloop22 C G C G U A A G = 100
> intloop22 C G C G U A C G = 200
> intloop22 C G C G U A G G = 180
> intloop22 C G C G U A U G = 80
> intloop22 C G C G U C A G = 200
> intloop22 C G C G U C C G = 200
> intloop22 C G C G U C G G = 240
> intloop22 C G C G U C U G = 180
> intloop22 C G C G U G A G = 200
> intloop22 C G C G U G C G = 200
> intloop22 C G C G U G G G = 200
> intloop22 C G C G U G U G = 200
> intloop22 C G C G U U A G = 200
> intloop22 C G C G U U C G = 200
> intloop22 C G C G U U G G = 240
> intloop22 C G C G U U U G = 180
> intloop22 C G G G U A A G = 0
> intloop22 C G G G U A C G = 200
> intloop22 C G G G U A G G = 70
> intloop22 C G G G U A U G = 160
> intloop22 C G G G U C A G = 200
> intloop22 C G G G U C C G = 200
> intloop22 C G G G U C G G = 200
> intloop22 C G G G U C U G = 200
> intloop22 C G G G U G A G = 40
> intloop22 C G G G U G C G = 200
> intloop22 C G G G U G G G = 110
> intloop22 C G G G U G U G = 120
> intloop22 C G G G U U A G = 0
> intloop22 C G G G U U C G = 200
> intloop22 C G G G U U G G = 0
> intloop22 C G G G U U U G = -200
> intloop22 C G U G U A A G = 200
> intloop22 C G U G U A C G = 200
> intloop22 C G U G U A G G = 200
> intloop22 C G U G U A U G = 200
> intloop22 C G U G U C A G = 170
> intloop22 C G U G U C C G = 200
> intloop22 C G U G U C G G = 210
> intloop22 C G U G U C U G = 150
> intloop22 C G U G U G A G = 130
> intloop22 C G U G U G C G = 200
> intloop22 C G U G U G G G = 120
> intloop22 C G U G U G U G = -70
> intloop22 C G U G U U A G = 200
> intloop22 C G U G U U C G = 200
> intloop22 C G U G U U G G = 190
> intloop22 C G U G U U U G = 180
> intloop22 C U A G U A A G = 200
> intloop22 C U A G U A C G = 270
> intloop22 C U A G U A G G = 30
> intloop22 C U A G U A U G = 220
> intloop22 C U A G U C A G = 200
> intloop22 C U A G U C C G = 160
> intloop22 C U A G U C G G = -110
> intloop22 C U A G U C U G = 70
> intloop22 C U A G U G A G = 200
> intloop22 C U A G U G C G = 200
> intloop22 C U A G U G G G = 10
> intloop22 C U A G U G U G = 190
> intloop22 C U A G U U A G = 200
> intloop22 C U A G U U C G = 200
> intloop22 C U A G U U G G = 200
> intloop22 C U A G U U U G = 200
> intloop22 C U C G U A A G = 200
> intloop22 C U C G U A C G = 210
> intloop22 C U C G U A G G = -70
> intloop22 C U C G U A U G = 120
> intloop22 C U C G U C A G = 200
> intloop22 C U C G U C C G = 210
> intloop22 C U C G U C G G = 30
> intloop22 C U C G U C U G = 120
> intloop22 C U C G U G A G = 200
> intloop22 C U C G U G C G = 200
> intloop22 C U C G U G G G = 200
> intloop22 C U C G U G U G = 200
> intloop22 C U C G U U A G = 200
> intloop22 C U C G U U C G = 210
> intloop22 C U C G U U G G = 30
> intloop22 C U C G U U U G = 120
> intloop22 C U G G U A A G = 200
> intloop22 C U G G U A C G = 200
> intloop22 C U G G U A G G = 10
> intloop22 C U G G U A U G = 190
> intloop22 C U G G U C A G = 200
> intloop22 C U G G U C C G = 200
> intloop22 C U G G U C G G = 200
> intloop22 C U G G U C U G = 200
> intloop22 C U G G U G A G = 200
> intloop22 C U G G U G C G = 200
> intloop22 C U G G U G G G = -30
> intloop22 C U G G U G U G = 150
> intloop22 C U G G U U A G = 200
> intloop22 C U G G U U C G = 30
> intloop22 C U G G U U G G = -350
> intloop22 C U G G U U U G = 20
> intloop22 C U U G U A A G = 200
> intloop22 C U U G U A C G = 200
> intloop22 C U U G U A G G = 200
> intloop22 C U U G U A U G = 200
> intloop22 C U U G U C A G = 200
> intloop22 C U U G U C C G = 180
> intloop22 C U U G U C G G = 0
> intloop22 C U U G U C U G = 90
> intloop22 C U U G U G A G = 200
> intloop22 C U U G U G C G = 150
> intloop22 C U U G U G G G = -220
> intloop22 C U U G U G U G = 150
> intloop22 C U U G U U A G = 200
> intloop22 C U U G U U C G = 120
> intloop22 C U U G U U G G = 30
> intloop22 C U U G U U U G = 30
> intloop22 C A A U A A A G = 200
> intloop22 C A A U A A C G = 240
> intloop22 C A A U A A G G = 100
> intloop22 C A A U A A U G = 200
> intloop22 C A A U A C A G = 180
> intloop22 C A A U A C C G = 210
> intloop22 C A A U A C G G = 80
> intloop22 C A A U A C U G = 200
> intloop22 C A A U A G A G = 80
> intloop22 C A A U A G C G = 110
> intloop22 C A A U A G G G = -20
> intloop22 C A A U A G U G = 200
> intloop22 C A A U A U A G = 200
> intloop22 C A A U A U C G = 200
> intloop22 C A A U A U G G = 200
> intloop22 C A A U A U U G = 200
> intloop22 C A C U A A A G = 190
> intloop22 C A C U A A C G = 220
> intloop22 C A C U A A G G = 90
> intloop22 C A C U A A U G = 200
> intloop22 C A C U A C A G = 230
> intloop22 C A C U A C C G = 210
> intloop22 C A C U A C G G = 170
> intloop22 C A C U A C U G = 200
> intloop22 C A C U A G A G = 200
> intloop22 C A C U A G C G = 200
> intloop22 C A C U A G G G = 200
> intloop22 C A C U A G U G = 200
> intloop22 C A C U A U A G = 230
> intloop22 C A C U A U C G = 210
> intloop22 C A C U A U G G = 170
> intloop22 C A C U A U U G = 200
> intloop22 C A G U A A A G = 80
> intloop22 C A G U A A C G = 110
> intloop22 C A G U A A G G = -20
> intloop22 C A G U A A U G = 200
> intloop22 C A G U A C A G = 200
> intloop22 C A G U A C C G = 200
> intloop22 C A G U A C G G = 200
> intloop22 C A G U A C U G = 200
> intloop22 C A G U A G A G = 130
> intloop22 C A G U A G C G = 170
> intloop22 C A G U A G G G = 30
> intloop22 C A G U A G U G = 200
> intloop22 C A G U A U A G = 60
> intloop22 C A G U A U C G = 0
> intloop22 C A G U A U G G = 40
> intloop22 C A G U A U U G = 200
> intloop22 C A U U A A A G = 200
> intloop22 C A U U A A C G = 200
> intloop22 C A U U A A G G = 200
> intloop22 C A U U A A U G = 200
> intloop22 C A U U A C A G = 230
> intloop22 C A U U A C C G = 210
> intloop22 C A U U A C G G = 170
> intloop22 C A U U A C U G = 200
> intloop22 C A U U A G A G = 160
> intloop22 C A U U A G C G = 90
> intloop22 C A U U A G G G = 140
> intloop22 C A U U A G U G = 200
> intloop22 C A U U A U A G = 190
> intloop22 C A U U A U C G = 130
> intloop22 C A U U A U G G = 180
> intloop22 C A U U A U U G = 200
> intloop22 C C A U A A A G = 190
> intloop22 C C A U A A C G = 280
> intloop22 C C A U A A G G = 200
> intloop22 C C A U A A U G = 270
> intloop22 C C A U A C A G = 170
> intloop22 C C A U A C C G = 200
> intloop22 C C A U A C G G = 200
> intloop22 C C A U A C U G = 180
> intloop22 C C A U A G A G = 70
> intloop22 C C A U A G C G = 200
> intloop22 C C A U A G G G = 200
> intloop22 C C A U A G U G = 180
> intloop22 C C A U A U A G = 200
> intloop22 C C A U A U C G = 200
> intloop22 C C A U A U G G = 200
> intloop22 C C A U A U U G = 200
> intloop22 C C C U A A A G = 180
> intloop22 C C C U A A C G = 210
> intloop22 C C C U A A G G = 200
> intloop22 C C C U A A U G = 190
> intloop22 C C C U A C A G = 160
> intloop22 C C C U A C C G = 190
> intloop22 C C C U A C G G = 200
> intloop22 C C C U A C U G = 180
> intloop22 C C C U A G A G = 200
> intloop22 C C C U A G C G = 200
> intloop22 C C C U A G G G = 200
> intloop22 C C C U A G U G = 200
> intloop22 C C C U A U A G = 160
> intloop22 C C C U A U C G = 190
> intloop22 C C C U A U G G = 200
> intloop22 C C C U A U U G = 180
> intloop22 C C G U A A A G = 70
> intloop22 C C G U A A C G = 200
> intloop22 C C G U A A G G = 200
> intloop22 C C G U A A U G = 180
> intloop22 C C G U A C A G = 200
> intloop22 C C G U A C C G = 200
> intloop22 C C G U A C G G = 200
> intloop22 C C G U A C U G = 200
> intloop22 C C G U A G A G = 120
> intloop22 C C G U A G C G = 210
> intloop22 C C G U A G G G = 200
> intloop22 C C G U A G U G = 200
> intloop22 C C G U A U A G = -50
> intloop22 C C G U A U C G = 80
> intloop22 C C G U A U G G = 200
> intloop22 C C G U A U U G = 70
> intloop22 C C U U A A A G = 200
> intloop22 C C U U A A C G = 200
> intloop22 C C U U A A G G = 200
> intloop22 C C U U A A U G = 200
> intloop22 C C U U A C A G = 160
> intloop22 C C U U A C C G = 190
> intloop22 C C U U A C G G = 200
> intloop22 C C U U A C U G = 180
> intloop22 C C U U A G A G = 50
> intloop22 C C U U A G C G = 180
> intloop22 C C U U A G G G = 200
> intloop22 C C U U A G U G = 160
> intloop22 C C U U A U A G = 80
> intloop22 C C U U A U C G = 110
> intloop22 C C U U A U G G = 200
> intloop22 C C U U A U U G = 100
> intloop22 C G A U A A A G = 100
> intloop22 C G A U A A C G = 200
> intloop22 C G A U A A G G = 180
> intloop22 C G A U A A U G = 180
> intloop22 C G A U A C A G = 80
> intloop22 C G A U A C C G = 200
> intloop22 C G A U A C G G = 150
> intloop22 C G A U A C U G = 60
> intloop22 C G A U A G A G = -20
> intloop22 C G A U A G C G = 200
> intloop22 C G A U A G G G = 50
> intloop22 C G A U A G U G = 140
> intloop22 C G A U A U A G = 200
> intloop22 C G A U A U C G = 200
> intloop22 C G A U A U G G = 200
> intloop22 C G A U A U U G = 200
> intloop22 C G C U A A A G = 90
> intloop22 C G C U A A C G = 200
> intloop22 C G C U A A G G = 160
> intloop22 C G C U A A U G = 70
> intloop22 C G C U A C A G = 170
> intloop22 C G C U A C C G = 200
> intloop22 C G C U A C G G = 210
> intloop22 C G C U A C U G = 150
> intloop22 C G C U A G A G = 200
> intloop22 C G C U A G C G = 200
> intloop22 C G C U A G G G = 200
> intloop22 C G C U A G U G = 200
> intloop22 C G C U A U A G = 170
> intloop22 C G C U A U C G = 200
> intloop22 C G C U A U G G = 210
> intloop22 C G C U A U U G = 150
> intloop22 C G G U A A A G = -20
> intloop22 C G G U A A C G = 200
> intloop22 C G G U A A G G = 50
> intloop22 C G G U A A U G = 140
> intloop22 C G G U A C A G = 200
> intloop22 C G G U A C C G = 200
> intloop22 C G G U A C G G = 200
> intloop22 C G G U A C U G = 200
> intloop22 C G G U A G A G = 30
> intloop22 C G G U A G C G = 200
> intloop22 C G G U A G G G = 110
> intloop22 C G G U A G U G = 110
> intloop22 C G G U A U A G = 40
> intloop22 C G G U A U C G = 200
> intloop22 C G G U A U G G = 40
> intloop22 C G G U A U U G = -160
> intloop22 C G U U A A A G = 200
> intloop22 C G U U A A C G = 200
> intloop22 C G U U A A G G = 200
> intloop22 C G U U A A U G = 200
> intloop22 C G U U A C A G = 170
> intloop22 C G U U A C C G = 200
> intloop22 C G U U A C G G = 210
> intloop22 C G U U A C U G = 150
> intloop22 C G U U A G A G = 140
> intloop22 C G U U A G C G = 200
> intloop22 C G U U A G G G = 130
> intloop22 C G U U A G U G = -60
> intloop22 C G U U A U A G = 180
> intloop22 C G U U A U C G = 200
> intloop22 C G U U A U G G = 170
> intloop22 C G U U A U U G = 160
> intloop22 C U A U A A A G = 200
> intloop22 C U A U A A C G = 270
> intloop22 C U A U A A G G = 30
> intloop22 C U A U A A U G = 220
> intloop22 C U A U A C A G = 200
> intloop22 C U A U A C C G = 180
> intloop22 C U A U A C G G = -90
> intloop22 C U A U A C U G = 90
> intloop22 C U A U A G A G = 200
> intloop22 C U A U A G C G = 180
> intloop22 C U A U A G G G = -10
> intloop22 C U A U A G U G = 180
> intloop22 C U A U A U A G = 200
> intloop22 C U A U A U C G = 200
> intloop22 C U A U A U G G = 200
> intloop22 C U A U A U U G = 200
> intloop22 C U C U A A A G = 200
> intloop22 C U C U A A C G = 190
> intloop22 C U C U A A G G = -80
> intloop22 C U C U A A U G = 100
> intloop22 C U C U A C A G = 200
> intloop22 C U C U A C C G = 180
> intloop22 C U C U A C G G = 0
> intloop22 C U C U A C U G = 90
> intloop22 C U C U A G A G = 200
> intloop22 C U C U A G C G = 200
> intloop22 C U C U A G G G = 200
> intloop22 C U C U A G U G = 200
> intloop22 C U C U A U A G = 200
> intloop22 C U C U A U C G = 180
> intloop22 C U C U A U G G = 0
> intloop22 C U C U A U U G = 90
> intloop22 C U G U A A A G = 200
> intloop22 C U G U A A C G = 180
> intloop22 C U G U A A G G = -10
> intloop22 C U G U A A U G = 180
> intloop22 C U G U A C A G = 200
> intloop22 C U G U A C C G = 200
> intloop22 C U G U A C G G = 200
> intloop22 C U G U A C U G = 200
> intloop22 C U G U A G A G = 200
> intloop22 C U G U A G C G = 200
> intloop22 C U G U A G G G = -40
> intloop22 C U G U A G U G = 150
> intloop22 C U G U A U A G = 200
> intloop22 C U G U A U C G = 70
> intloop22 C U G U A U G G = -310
> intloop22 C U G U A U U G = 60
> intloop22 C U U U A A A G = 200
> intloop22 C U U U A A C G = 200
> intloop22 C U U U A A G G = 200
> intloop22 C U U U A A U G = 200
> intloop22 C U U U A C A G = 200
> intloop22 C U U U A C C G = 180
> intloop22 C U U U A C G G = 0
> intloop22 C U U U A C U G = 90
> intloop22 C U U U A G A G = 200
> intloop22 C U U U A G C G = 160
> intloop22 C U U U A G G G = -210
> intloop22 C U U U A G U G = 160
> intloop22 C U U U A U A G = 200
> intloop22 C U U U A U C G = 100
> intloop22 C U U U A U G G = 0
> intloop22 C U U U A U U G = 10
> intloop22 C A A A U A A G = 200
> intloop22 C A A A U A C G = 240
> intloop22 C A A A U A G G = 100
> intloop22 C A A A U A U G = 200
> intloop22 C A A A U C A G = 160
> intloop22 C A A A U C C G = 190
> intloop22 C A A A U C G G = 60
> intloop22 C A A A U C U G = 200
> intloop22 C A A A U G A G = 100
> intloop22 C A A A U G C G = 130
> intloop22 C A A A U G G G = 0
> intloop22 C A A A U G U G = 200
> intloop22 C A A A U U A G = 200
> intloop22 C A A A U U C G = 200
> intloop22 C A A A U U G G = 200
> intloop22 C A A A U U U G = 200
> intloop22 C A C A U A A G = 200
> intloop22 C A C A U A C G = 240
> intloop22 C A C A U A G G = 100
> intloop22 C A C A U A U G = 200
> intloop22 C A C A U C A G = 260
> intloop22 C A C A U C C G = 240
> intloop22 C A C A U C G G = 200
> intloop22 C A C A U C U G = 200
> intloop22 C A C A U G A G = 200
> intloop22 C A C A U G C G = 200
> intloop22 C A C A U G G G = 200
> intloop22 C A C A U G U G = 200
> intloop22 C A C A U U A G = 260
> intloop22 C A C A U U C G = 240
> intloop22 C A C A U U G G = 200
> intloop22 C A C A U U U G = 200
> intloop22 C A G A U A A G = 100
> intloop22 C A G A U A C G = 130
> intloop22 C A G A U A G G = 0
> intloop22 C A G A U A U G = 200
> intloop22 C A G A U C A G = 200
> intloop22 C A G A U C C G = 200
> intloop22 C A G A U C G G = 200
> intloop22 C A G A U C U G = 200
> intloop22 C A G A U G A G = 140
> intloop22 C A G A U G C G = 170
> intloop22 C A G A U G G G = 40
> intloop22 C A G A U G U G = 200
> intloop22 C A G A U U A G = 20
> intloop22 C A G A U U C G = -40
> intloop22 C A G A U U G G = 0
> intloop22 C A G A U U U G = 200
> intloop22 C A U A U A A G = 200
> intloop22 C A U A U A C G = 200
> intloop22 C A U A U A G G = 200
> intloop22 C A U A U A U G = 200
> intloop22 C A U A U C A G = 230
> intloop22 C A U A U C C G = 210
> intloop22 C A U A U C G G = 170
> intloop22 C A U A U C U G = 200
> intloop22 C A U A U G A G = 150
> intloop22 C A U A U G C G = 80
> intloop22 C A U A U G G G = 130
> intloop22 C A U A U G U G = 200
> intloop22 C A U A U U A G = 220
> intloop22 C A U A U U C G = 150
> intloop22 C A U A U U G G = 200
> intloop22 C A U A U U U G = 200
> intloop22 C C A A U A A G = 190
> intloop22 C C A A U A C G = 280
> intloop22 C C A A U A G G = 200
> intloop22 C C A A U A U G = 270
> intloop22 C C A A U C A G = 150
> intloop22 C C A A U C C G = 180
> intloop22 C C A A U C G G = 200
> intloop22 C C A A U C U G = 160
> intloop22 C C A A U G A G = 90
> intloop22 C C A A U G C G = 220
> intloop22 C C A A U G G G = 200
> intloop22 C C A A U G U G = 200
> intloop22 C C A A U U A G = 200
> intloop22 C C A A U U C G = 200
> intloop22 C C A A U U G G = 200
> intloop22 C C A A U U U G = 200
> intloop22 C C C A U A A G = 190
> intloop22 C C C A U A C G = 220
> intloop22 C C C A U A G G = 200
> intloop22 C C C A U A U G = 210
> intloop22 C C C A U C A G = 190
> intloop22 C C C A U C C G = 220
> intloop22 C C C A U C G G = 200
> intloop22 C C C A U C U G = 210
> intloop22 C C C A U G A G = 200
> intloop22 C C C A U G C G = 200
> intloop22 C C C A U G G G = 200
> intloop22 C C C A U G U G = 200
> intloop22 C C C A U U A G = 190
> intloop22 C C C A U U C G = 220
> intloop22 C C C A U U G G = 200
> intloop22 C C C A U U U G = 210
> intloop22 C C G A U A A G = 90
> intloop22 C C G A U A C G = 220
> intloop22 C C G A U A G G = 200
> intloop22 C C G A U A U G = 200
> intloop22 C C G A U C A G = 200
> intloop22 C C G A U C C G = 200
> intloop22 C C G A U C G G = 200
> intloop22 C C G A U C U G = 200
> intloop22 C C G A U G A G = 130
> intloop22 C C G A U G C G = 220
> intloop22 C C G A U G G G = 200
> intloop22 C C G A U G U G = 200
> intloop22 C C G A U U A G = -90
> intloop22 C C G A U U C G = 40
> intloop22 C C G A U U G G = 200
> intloop22 C C G A U U U G = 30
> intloop22 C C U A U A A G = 200
> intloop22 C C U A U A C G = 200
> intloop22 C C U A U A G G = 200
> intloop22 C C U A U A U G = 200
> intloop22 C C U A U C A G = 160
> intloop22 C C U A U C C G = 190
> intloop22 C C U A U C G G = 200
> intloop22 C C U A U C U G = 180
> intloop22 C C U A U G A G = 40
> intloop22 C C U A U G C G = 170
> intloop22 C C U A U G G G = 200
> intloop22 C C U A U G U G = 150
> intloop22 C C U A U U A G = 110
> intloop22 C C U A U U C G = 140
> intloop22 C C U A U U G G = 200
> intloop22 C C U A U U U G = 120
> intloop22 C G A A U A A G = 100
> intloop22 C G A A U A C G = 200
> intloop22 C G A A U A G G = 180
> intloop22 C G A A U A U G = 180
> intloop22 C G A A U C A G = 60
> intloop22 C G A A U C C G = 200
> intloop22 C G A A U C G G = 130
> intloop22 C G A A U C U G = 40
> intloop22 C G A A U G A G = 0
> intloop22 C G A A U G C G = 200
> intloop22 C G A A U G G G = 70
> intloop22 C G A A U G U G = 160
> intloop22 C G A A U U A G = 200
> intloop22 C G A A U U C G = 200
> intloop22 C G A A U U G G = 200
> intloop22 C G A A U U U G = 200
> intloop22 C G C A U A A G = 100
> intloop22 C G C A U A C G = 200
> intloop22 C G C A U A G G = 180
> intloop22 C G C A U A U G = 80
> intloop22 C G C A U C A G = 200
> intloop22 C G C A U C C G = 200
> intloop22 C G C A U C G G = 240
> intloop22 C G C A U C U G = 180
> intloop22 C G C A U G A G = 200
> intloop22 C G C A U G C G = 200
> intloop22 C G C A U G G G = 200
> intloop22 C G C A U G U G = 200
> intloop22 C G C A U U A G = 200
> intloop22 C G C A U U C G = 200
> intloop22 C G C A U U G G = 240
> intloop22 C G C A U U U G = 180
> intloop22 C G G A U A A G = 0
> intloop22 C G G A U A C G = 200
> intloop22 C G G A U A G G = 70
> intloop22 C G G A U A U G = 160
> intloop22 C G G A U C A G = 200
> intloop22 C G G A U C C G = 200
> intloop22 C G G A U C G G = 200
> intloop22 C G G A U C U G = 200
> intloop22 C G G A U G A G = 40
> intloop22 C G G A U G C G = 200
> intloop22 C G G A U G G G = 110
> intloop22 C G G A U G U G = 120
> intloop22 C G G A U U A G = 0
> intloop22 C G G A U U C G = 200
> intloop22 C G G A U U G G = 0
> intloop22 C G G A U U U G = -200
> intloop22 C G U A U A A G = 200
> intloop22 C G U A U A C G = 200
> intloop22 C G U A U A G G = 200
> intloop22 C G U A U A U G = 200
> intloop22 C G U A U C A G = 170
> intloop22 C G U A U C C G = 200
> intloop22 C G U A U C G G = 210
> intloop22 C G U A U C U G = 150
> intloop22 C G U A U G A G = 130
> intloop22 C G U A U G C G = 200
> intloop22 C G U A U G G G = 120
> intloop22 C G U A U G U G = -70
> intloop22 C G U A U U A G = 200
> intloop22 C G U A U U C G = 200
> intloop22 C G U A U U G G = 190
> intloop22 C G U A U U U G = 180
> intloop22 C U A A U A A G = 200
> intloop22 C U A A U A C G = 270
> intloop22 C U A A U A G G = 30
> intloop22 C U A A U A U G = 220
> intloop22 C U A A U C A G = 200
> intloop22 C U A A U C C G = 160
> intloop22 C U A A U C G G = -110
> intloop22 C U A A U C U G = 70
> intloop22 C U A A U G A G = 200
> intloop22 C U A A U G C G = 200
> intloop22 C U A A U G G G = 10
> intloop22 C U A A U G U G = 190
> intloop22 C U A A U U A G = 200
> intloop22 C U A A U U C G = 200
> intloop22 C U A A U U G G = 200
> intloop22 C U A A U U U G = 200
> intloop22 C U C A U A A G = 200
> intloop22 C U C A U A C G = 210
> intloop22 C U C A U A G G = -70
> intloop22 C U C A U A U G = 120
> intloop22 C U C A U C A G = 200
> intloop22 C U C A U C C G = 210
> intloop22 C U C A U C G G = 30
> intloop22 C U C A U C U G = 120
> intloop22 C U C A U G A G = 200
> intloop22 C U C A U G C G = 200
> intloop22 C U C A U G G G = 200
> intloop22 C U C A U G U G = 200
> intloop22 C U C A U U A G = 200
> intloop22 C U C A U U C G = 210
> intloop22 C U C A U U G G = 30
> intloop22 C U C A U U U G = 120
> intloop22 C U G A U A A G = 200
> intloop22 C U G A U A C G = 200
> intloop22 C U G A U A G G = 10
> intloop22 C U G A U A U G = 190
> intloop22 C U G A U C A G = 200
> intloop22 C U G A U C C G = 200
> intloop22 C U G A U C G G = 200
> intloop22 C U G A U C U G = 200
> intloop22 C U G A U G A G = 200
> intloop22 C U G A U G C G = 200
> intloop22 C U G A U G G G = -30
> intloop22 C U G A U G U G = 150
> intloop22 C U G A U U A G = 200
> intloop22 C U G A U U C G = 30
> intloop22 C U G A U U G G = -350
> intloop22 C U G A U U U G = 20
> intloop22 C U U A U A A G = 200
> intloop22 C U U A U A C G = 200
> intloop22 C U U A U A G G = 200
> intloop22 C U U A U A U G = 200
> intloop22 C U U A U C A G = 200
> intloop22 C U U A U C C G = 180
> intloop22 C U U A U C G G = 0
> intloop22 C U U A U C U G = 90
> intloop22 C U U A U G A G = 200
> intloop22 C U U A U G C G = 150
> intloop22 C U U A U G G G = -220
> intloop22 C U U A U G U G = 150
> intloop22 C U U A U U A G = 200
> intloop22 C U U A U U C G = 120
> intloop22 C U U A U U G G = 30
> intloop22 C U U A U U U G = 30
> intloop22 G A A G C A A C = 50
> intloop22 G A A G C A C C = 110
> intloop22 G A A G C A G C = 40
> intloop22 G A A G C A U C = 200
> intloop22 G A A G C C A C = 130
> intloop22 G A A G C C C C = 100
> intloop22 G A A G C C G C = 70
> intloop22 G A A G C C U C = 200
> intloop22 G A A G C G A C = -20
> intloop22 G A A G C G C C = 70
> intloop22 G A A G C G G C = -50
> intloop22 G A A G C G U C = 200
> intloop22 G A A G C U A C = 200
> intloop22 G A A G C U C C = 200
> intloop22 G A A G C U G C = 200
> intloop22 G A A G C U U C = 200
> intloop22 G A C G C A A C = 60
> intloop22 G A C G C A C C = 110
> intloop22 G A C G C A G C = 50
> intloop22 G A C G C A U C = 200
> intloop22 G A C G C C A C = 220
> intloop22 G A C G C C C C = 190
> intloop22 G A C G C C G C = 70
> intloop22 G A C G C C U C = 200
> intloop22 G A C G C G A C = 200
> intloop22 G A C G C G C C = 200
> intloop22 G A C G C G G C = 200
> intloop22 G A C G C G U C = 200
> intloop22 G A C G C U A C = 200
> intloop22 G A C G C U C C = 110
> intloop22 G A C G C U G C = 50
> intloop22 G A C G C U U C = 200
> intloop22 G A G G C A A C = 0
> intloop22 G A G G C A C C = -100
> intloop22 G A G G C A G C = -70
> intloop22 G A G G C A U C = 200
> intloop22 G A G G C C A C = 200
> intloop22 G A G G C C C C = 200
> intloop22 G A G G C C G C = 200
> intloop22 G A G G C C U C = 200
> intloop22 G A G G C G A C = 110
> intloop22 G A G G C G C C = 80
> intloop22 G A G G C G G C = -20
> intloop22 G A G G C G U C = 200
> intloop22 G A G G C U A C = -10
> intloop22 G A G G C U C C = -160
> intloop22 G A G G C U G C = -60
> intloop22 G A G G C U U C = 200
> intloop22 G A U G C A A C = 200
> intloop22 G A U G C A C C = 200
> intloop22 G A U G C A G C = 200
> intloop22 G A U G C A U C = 200
> intloop22 G A U G C C A C = 200
> intloop22 G A U G C C C C = 110
> intloop22 G A U G C C G C = 100
> intloop22 G A U G C C U C = 200
> intloop22 G A U G C G A C = 90
> intloop22 G A U G C G C C = -10
> intloop22 G A U G C G G C = 60
> intloop22 G A U G C G U C = 200
> intloop22 G A U G C U A C = 140
> intloop22 G A U G C U C C = 30
> intloop22 G A U G C U G C = 140
> intloop22 G A U G C U U C = 200
> intloop22 G C A G C A A C = 110
> intloop22 G C A G C A C C = 170
> intloop22 G C A G C A G C = 200
> intloop22 G C A G C A U C = 180
> intloop22 G C A G C C A C = 100
> intloop22 G C A G C C C C = 100
> intloop22 G C A G C C G C = 200
> intloop22 G C A G C C U C = 110
> intloop22 G C A G C G A C = -40
> intloop22 G C A G C G C C = 110
> intloop22 G C A G C G G C = 200
> intloop22 G C A G C G U C = 120
> intloop22 G C A G C U A C = 200
> intloop22 G C A G C U C C = 200
> intloop22 G C A G C U G C = 200
> intloop22 G C A G C U U C = 200
> intloop22 G C C G C A A C = 150
> intloop22 G C C G C A C C = 150
> intloop22 G C C G C A G C = 200
> intloop22 G C C G C A U C = 150
> intloop22 G C C G C C A C = 130
> intloop22 G C C G C C C C = 130
> intloop22 G C C G C C G C = 200
> intloop22 G C C G C C U C = 140
> intloop22 G C C G C G A C = 200
> intloop22 G C C G C G C C = 200
> intloop22 G C C G C G G C = 200
> intloop22 G C C G C G U C = 200
> intloop22 G C C G C U A C = 120
> intloop22 G C C G C U C C = 120
> intloop22 G C C G C U G C = 200
> intloop22 G C C G C U U C = 120
> intloop22 G C G G C A A C = -70
> intloop22 G C G G C A C C = -60
> intloop22 G C G G C A G C = 200
> intloop22 G C G G C A U C = 120
> intloop22 G C G G C C A C = 200
> intloop22 G C G G C C C C = 200
> intloop22 G C G G C C G C = 200
> intloop22 G C G G C C U C = 200
> intloop22 G C G G C G A C = 90
> intloop22 G C G G C G C C = 150
> intloop22 G C G G C G G C = 200
> intloop22 G C G G C G U C = 150
> intloop22 G C G G C U A C = -160
> intloop22 G C G G C U C C = -60
> intloop22 G C G G C U G C = 200
> intloop22 G C G G C U U C = -50
> intloop22 G C U G C A A C = 200
> intloop22 G C U G C A C C = 200
> intloop22 G C U G C A G C = 200
> intloop22 G C U G C A U C = 200
> intloop22 G C U G C C A C = 120
> intloop22 G C U G C C C C = 120
> intloop22 G C U G C C G C = 200
> intloop22 G C U G C C U C = 120
> intloop22 G C U G C G A C = 0
> intloop22 G C U G C G C C = 100
> intloop22 G C U G C G G C = 200
> intloop22 G C U G C G U C = 100
> intloop22 G C U G C U A C = 30
> intloop22 G C U G C U C C = 30
> intloop22 G C U G C U G C = 200
> intloop22 G C U G C U U C = 30
> intloop22 G G A G C A A C = -30
> intloop22 G G A G C A C C = 200
> intloop22 G G A G C A G C = 100
> intloop22 G G A G C A U C = -50
> intloop22 G G A G C C A C = -70
> intloop22 G G A G C C C C = 200
> intloop22 G G A G C C G C = 90
> intloop22 G G A G C C U C = -150
> intloop22 G G A G C G A C = -170
> intloop22 G G A G C G C C = 200
> intloop22 G G A G C G G C = 0
> intloop22 G G A G C G U C = -130
> intloop22 G G A G C U A C = 200
> intloop22 G G A G C U C C = 200
> intloop22 G G A G C U G C = 200
> intloop22 G G A G C U U C = 200
> intloop22 G G C G C A A C = 10
> intloop22 G G C G C A C C = 200
> intloop22 G G C G C A G C = 140
> intloop22 G G C G C A U C = -60
> intloop22 G G C G C C A C = 70
> intloop22 G G C G C C C C = 200
> intloop22 G G C G C C G C = 180
> intloop22 G G C G C C U C = -20
> intloop22 G G C G C G A C = 200
> intloop22 G G C G C G C C = 200
> intloop22 G G C G C G G C = 200
> intloop22 G G C G C G U C = 200
> intloop22 G G C G C U A C = 40
> intloop22 G G C G C U C C = 200
> intloop22 G G C G C U G C = 170
> intloop22 G G C G C U U C = -10
> intloop22 G G G G C A A C = -160
> intloop22 G G G G C A C C = 200
> intloop22 G G G G C A G C = 0
> intloop22 G G G G C A U C = -60
> intloop22 G G G G C C A C = 200
> intloop22 G G G G C C C C = 200
> intloop22 G G G G C C G C = 200
> intloop22 G G G G C C U C = 200
> intloop22 G G G G C G A C = -90
> intloop22 G G G G C G C C = 200
> intloop22 G G G G C G G C = 80
> intloop22 G G G G C G U C = -60
> intloop22 G G G G C U A C = -160
> intloop22 G G G G C U C C = 200
> intloop22 G G G G C U G C = -70
> intloop22 G G G G C U U C = -410
> intloop22 G G U G C A A C = 200
> intloop22 G G U G C A C C = 200
> intloop22 G G U G C A G C = 200
> intloop22 G G U G C A U C = 200
> intloop22 G G U G C C A C = 40
> intloop22 G G U G C C C C = 200
> intloop22 G G U G C C G C = 170
> intloop22 G G U G C C U C = -30
> intloop22 G G U G C G A C = 30
> intloop22 G G U G C G C C = 200
> intloop22 G G U G C G G C = 90
> intloop22 G G U G C G U C = -240
> intloop22 G G U G C U A C = 50
> intloop22 G G U G C U C C = 200
> intloop22 G G U G C U G C = 120
> intloop22 G G U G C U U C = 10
> intloop22 G U A G C A A C = 200
> intloop22 G U A G C A C C = 70
> intloop22 G U A G C A G C = 10
> intloop22 G U A G C A U C = 150
> intloop22 G U A G C C A C = 200
> intloop22 G U A G C C C C = 0
> intloop22 G U A G C C G C = -190
> intloop22 G U A G C C U C = -20
> intloop22 G U A G C G A C = 200
> intloop22 G U A G C G C C = 20
> intloop22 G U A G C G G C = -90
> intloop22 G U A G C G U C = 90
> intloop22 G U A G C U A C = 200
> intloop22 G U A G C U C C = 200
> intloop22 G U A G C U G C = 200
> intloop22 G U A G C U U C = 200
> intloop22 G U C G C A A C = 200
> intloop22 G U C G C A C C = 50
> intloop22 G U C G C A G C = -70
> intloop22 G U C G C A U C = 0
> intloop22 G U C G C C A C = 200
> intloop22 G U C G C C C C = 30
> intloop22 G U C G C C G C = -30
> intloop22 G U C G C C U C = -10
> intloop22 G U C G C G A C = 200
> intloop22 G U C G C G C C = 200
> intloop22 G U C G C G G C = 200
> intloop22 G U C G C G U C = 200
> intloop22 G U C G C U A C = 200
> intloop22 G U C G C U C C = 20
> intloop22 G U C G C U G C = -70
> intloop22 G U C G C U U C = 40
> intloop22 G U G G C A A C = 200
> intloop22 G U G G C A C C = 20
> intloop22 G U G G C A G C = -80
> intloop22 G U G G C A U C = 90
> intloop22 G U G G C C A C = 200
> intloop22 G U G G C C C C = 200
> intloop22 G U G G C C G C = 200
> intloop22 G U G G C C U C = 200
> intloop22 G U G G C G A C = 200
> intloop22 G U G G C G C C = 50
> intloop22 G U G G C G G C = -100
> intloop22 G U G G C G U C = 110
> intloop22 G U G G C U A C = 200
> intloop22 G U G G C U C C = -160
> intloop22 G U G G C U G C = -440
> intloop22 G U G G C U U C = -100
> intloop22 G U U G C A A C = 200
> intloop22 G U U G C A C C = 200
> intloop22 G U U G C A G C = 200
> intloop22 G U U G C A U C = 200
> intloop22 G U U G C C A C = 200
> intloop22 G U U G C C C C = 170
> intloop22 G U U G C C G C = -70
> intloop22 G U U G C C U C = 20
> intloop22 G U U G C G A C = 200
> intloop22 G U U G C G C C = 0
> intloop22 G U U G C G G C = -300
> intloop22 G U U G C G U C = 60
> intloop22 G U U G C U A C = 200
> intloop22 G U U G C U C C = 10
> intloop22 G U U G C U G C = -100
> intloop22 G U U G C U U C = 60
> intloop22 G A A C G A A C = 150
> intloop22 G A A C G A C C = 120
> intloop22 G A A C G A G C = 10
> intloop22 G A A C G A U C = 200
> intloop22 G A A C G C A C = 120
> intloop22 G A A C G C C C = 90
> intloop22 G A A C G C G C = -10
> intloop22 G A A C G C U C = 200
> intloop22 G A A C G G A C = -50
> intloop22 G A A C G G C C = -80
> intloop22 G A A C G G G C = -190
> intloop22 G A A C G G U C = 200
> intloop22 G A A C G U A C = 200
> intloop22 G A A C G U C C = 200
> intloop22 G A A C G U G C = 200
> intloop22 G A A C G U U C = 200
> intloop22 G A C C G A A C = 120
> intloop22 G A C C G A C C = 90
> intloop22 G A C C G A G C = -20
> intloop22 G A C C G A U C = 200
> intloop22 G A C C G C A C = 180
> intloop22 G A C C G C C C = 90
> intloop22 G A C C G C G C = 90
> intloop22 G A C C G C U C = 200
> intloop22 G A C C G G A C = 200
> intloop22 G A C C G G C C = 200
> intloop22 G A C C G G G C = 200
> intloop22 G A C C G G U C = 200
> intloop22 G A C C G U A C = 80
> intloop22 G A C C G U C C = 0
> intloop22 G A C C G U G C = -10
> intloop22 G A C C G U U C = 200
> intloop22 G A G C G A A C = 10
> intloop22 G A G C G A C C = -20
> intloop22 G A G C G A G C = -130
> intloop22 G A G C G A U C = 200
> intloop22 G A G C G C A C = 200
> intloop22 G A G C G C C C = 200
> intloop22 G A G C G C G C = 200
> intloop22 G A G C G C U C = 200
> intloop22 G A G C G G A C = 110
> intloop22 G A G C G G C C = 80
> intloop22 G A G C G G G C = -20
> intloop22 G A G C G G U C = 200
> intloop22 G A G C G U A C = -70
> intloop22 G A G C G U C C = -200
> intloop22 G A G C G U G C = -130
> intloop22 G A G C G U U C = 200
> intloop22 G A U C G A A C = 200
> intloop22 G A U C G A C C = 200
> intloop22 G A U C G A G C = 200
> intloop22 G A U C G A U C = 200
> intloop22 G A U C G C A C = 190
> intloop22 G A U C G C C C = 100
> intloop22 G A U C G C G C = 90
> intloop22 G A U C G C U C = 200
> intloop22 G A U C G G A C = -30
> intloop22 G A U C G G C C = -160
> intloop22 G A U C G G G C = -90
> intloop22 G A U C G G U C = 200
> intloop22 G A U C G U A C = 150
> intloop22 G A U C G U C C = 20
> intloop22 G A U C G U G C = 90
> intloop22 G A U C G U U C = 200
> intloop22 G C A C G A A C = 120
> intloop22 G C A C G A C C = 180
> intloop22 G C A C G A G C = 200
> intloop22 G C A C G A U C = 190
> intloop22 G C A C G C A C = 100
> intloop22 G C A C G C C C = 100
> intloop22 G C A C G C G C = 200
> intloop22 G C A C G C U C = 100
> intloop22 G C A C G G A C = -80
> intloop22 G C A C G G C C = 20
> intloop22 G C A C G G G C = 200
> intloop22 G C A C G G U C = 30
> intloop22 G C A C G U A C = 200
> intloop22 G C A C G U C C = 200
> intloop22 G C A C G U G C = 200
> intloop22 G C A C G U U C = 200
> intloop22 G C C C G A A C = 90
> intloop22 G C C C G A C C = 90
> intloop22 G C C C G A G C = 200
> intloop22 G C C C G A U C = 100
> intloop22 G C C C G C A C = 100
> intloop22 G C C C G C C C = 100
> intloop22 G C C C G C G C = 200
> intloop22 G C C C G C U C = 100
> intloop22 G C C C G G A C = 200
> intloop22 G C C C G G C C = 200
> intloop22 G C C C G G G C = 200
> intloop22 G C C C G G U C = 200
> intloop22 G C C C G U A C = 0
> intloop22 G C C C G U C C = 0
> intloop22 G C C C G U G C = 200
> intloop22 G C C C G U U C = 0
> intloop22 G C G C G A A C = -10
> intloop22 G C G C G A C C = 90
> intloop22 G C G C G A G C = 200
> intloop22 G C G C G A U C = 90
> intloop22 G C G C G C A C = 200
> intloop22 G C G C G C C C = 200
> intloop22 G C G C G C G C = 200
> intloop22 G C G C G C U C = 200
> intloop22 G C G C G G A C = 90
> intloop22 G C G C G G C C = 150
> intloop22 G C G C G G G C = 200
> intloop22 G C G C G G U C = 150
> intloop22 G C G C G U A C = -190
> intloop22 G C G C G U C C = -90
> intloop22 G C G C G U G C = 200
> intloop22 G C G C G U U C = -90
> intloop22 G C U C G A A C = 200
> intloop22 G C U C G A C C = 200
> intloop22 G C U C G A G C = 200
> intloop22 G C U C G A U C = 200
> intloop22 G C U C G C A C = 100
> intloop22 G C U C G C C C = 100
> intloop22 G C U C G C G C = 200
> intloop22 G C U C G C U C = 110
> intloop22 G C U C G G A C = -150
> intloop22 G C U C G G C C = -50
> intloop22 G C U C G G G C = 200
> intloop22 G C U C G G U C = -50
> intloop22 G C U C G U A C = 20
> intloop22 G C U C G U C C = 20
> intloop22 G C U C G U G C = 200
> intloop22 G C U C G U U C = 30
> intloop22 G G A C G A A C = -50
> intloop22 G G A C G A C C = 200
> intloop22 G G A C G A G C = 110
> intloop22 G G A C G A U C = -30
> intloop22 G G A C G C A C = -80
> intloop22 G G A C G C C C = 200
> intloop22 G G A C G C G C = 90
> intloop22 G G A C G C U C = -150
> intloop22 G G A C G G A C = -260
> intloop22 G G A C G G C C = 200
> intloop22 G G A C G G G C = -90
> intloop22 G G A C G G U C = -150
> intloop22 G G A C G U A C = 200
> intloop22 G G A C G U C C = 200
> intloop22 G G A C G U G C = 200
> intloop22 G G A C G U U C = 200
> intloop22 G G C C G A A C = -80
> intloop22 G G C C G A C C = 200
> intloop22 G G C C G A G C = 80
> intloop22 G G C C G A U C = -160
> intloop22 G G C C G C A C = 20
> intloop22 G G C C G C C C = 200
> intloop22 G G C C G C G C = 150
> intloop22 G G C C G C U C = -50
> intloop22 G G C C G G A C = 200
> intloop22 G G C C G G C C = 200
> intloop22 G G C C G G G C = 200
> intloop22 G G C C G G U C = 200
> intloop22 G G C C G U A C = -80
> intloop22 G G C C G U C C = 200
> intloop22 G G C C G U G C = 50
> intloop22 G G C C G U U C = -150
> intloop22 G G G C G A A C = -190
> intloop22 G G G C G A C C = 200
> intloop22 G G G C G A G C = -20
> intloop22 G G G C G A U C = -90
> intloop22 G G G C G C A C = 200
> intloop22 G G G C G C C C = 200
> intloop22 G G G C G C G C = 200
> intloop22 G G G C G C U C = 200
> intloop22 G G G C G G A C = -90
> intloop22 G G G C G G C C = 200
> intloop22 G G G C G G G C = 80
> intloop22 G G G C G G U C = -60
> intloop22 G G G C G U A C = -190
> intloop22 G G G C G U C C = 200
> intloop22 G G G C G U G C = -100
> intloop22 G G G C G U U C = -450
> intloop22 G G U C G A A C = 200
> intloop22 G G U C G A C C = 200
> intloop22 G G U C G A G C = 200
> intloop22 G G U C G A U C = 200
> intloop22 G G U C G C A C = 30
> intloop22 G G U C G C C C = 200
> intloop22 G G U C G C G C = 150
> intloop22 G G U C G C U C = -50
> intloop22 G G U C G G A C = -150
> intloop22 G G U C G G C C = 200
> intloop22 G G U C G G G C = -60
> intloop22 G G U C G G U C = -410
> intloop22 G G U C G U A C = 30
> intloop22 G G U C G U C C = 200
> intloop22 G G U C G U G C = 110
> intloop22 G G U C G U U C = -50
> intloop22 G U A C G A A C = 200
> intloop22 G U A C G A C C = 80
> intloop22 G U A C G A G C = -70
> intloop22 G U A C G A U C = 150
> intloop22 G U A C G C A C = 200
> intloop22 G U A C G C C C = 0
> intloop22 G U A C G C G C = -190
> intloop22 G U A C G C U C = 20
> intloop22 G U A C G G A C = 200
> intloop22 G U A C G G C C = -80
> intloop22 G U A C G G G C = -190
> intloop22 G U A C G G U C = 30
> intloop22 G U A C G U A C = 200
> intloop22 G U A C G U C C = 200
> intloop22 G U A C G U G C = 200
> intloop22 G U A C G U U C = 200
> intloop22 G U C C G A A C = 200
> intloop22 G U C C G A C C = 0
> intloop22 G U C C G A G C = -200
> intloop22 G U C C G A U C = 20
> intloop22 G U C C G C A C = 200
> intloop22 G U C C G C C C = 0
> intloop22 G U C C G C G C = -90
> intloop22 G U C C G C U C = 20
> intloop22 G U C C G G A C = 200
> intloop22 G U C C G G C C = 200
> intloop22 G U C C G G G C = 200
> intloop22 G U C C G G U C = 200
> intloop22 G U C C G U A C = 200
> intloop22 G U C C G U C C = -100
> intloop22 G U C C G U G C = -190
> intloop22 G U C C G U U C = -70
> intloop22 G U G C G A A C = 200
> intloop22 G U G C G A C C = -10
> intloop22 G U G C G A G C = -130
> intloop22 G U G C G A U C = 90
> intloop22 G U G C G C A C = 200
> intloop22 G U G C G C C C = 200
> intloop22 G U G C G C G C = 200
> intloop22 G U G C G C U C = 200
> intloop22 G U G C G G A C = 200
> intloop22 G U G C G G C C = 50
> intloop22 G U G C G G G C = -100
> intloop22 G U G C G G U C = 110
> intloop22 G U G C G U A C = 200
> intloop22 G U G C G U C C = -190
> intloop22 G U G C G U G C = -490
> intloop22 G U G C G U U C = -90
> intloop22 G U U C G A A C = 200
> intloop22 G U U C G A C C = 200
> intloop22 G U U C G A G C = 200
> intloop22 G U U C G A U C = 200
> intloop22 G U U C G C A C = 200
> intloop22 G U U C G C C C = 0
> intloop22 G U U C G C G C = -90
> intloop22 G U U C G C U C = 30
> intloop22 G U U C G G A C = 200
> intloop22 G U U C G G C C = -150
> intloop22 G U U C G G G C = -450
> intloop22 G U U C G G U C = -50
> intloop22 G U U C G U A C = 200
> intloop22 G U U C G U C C = -70
> intloop22 G U U C G U G C = -90
> intloop22 G U U C G U U C = -50
> intloop22 G A A U G A A C = 210
> intloop22 G A A U G A C C = 180
> intloop22 G A A U G A G C = 70
> intloop22 G A A U G A U C = 200
> intloop22 G A A U G C A C = 190
> intloop22 G A A U G C C C = 160
> intloop22 G A A U G C G C = 50
> intloop22 G A A U G C U C = 200
> intloop22 G A A U G G A C = 90
> intloop22 G A A U G G C C = 60
> intloop22 G A A U G G G C = -50
> intloop22 G A A U G G U C = 200
> intloop22 G A A U G U A C = 200
> intloop22 G A A U G U C C = 200
> intloop22 G A A U G U G C = 200
> intloop22 G A A U G U U C = 200
> intloop22 G A C U G A A C = 200
> intloop22 G A C U G A C C = 170
> intloop22 G A C U G A G C = 60
> intloop22 G A C U G A U C = 200
> intloop22 G A C U G C A C = 240
> intloop22 G A C U G C C C = 150
> intloop22 G A C U G C G C = 140
> intloop22 G A C U G C U C = 200
> intloop22 G A C U G G A C = 200
> intloop22 G A C U G G C C = 200
> intloop22 G A C U G G G C = 200
> intloop22 G A C U G G U C = 200
> intloop22 G A C U G U A C = 240
> intloop22 G A C U G U C C = 150
> intloop22 G A C U G U G C = 140
> intloop22 G A C U G U U C = 200
> intloop22 G A G U G A A C = 90
> intloop22 G A G U G A C C = 60
> intloop22 G A G U G A G C = -50
> intloop22 G A G U G A U C = 200
> intloop22 G A G U G C A C = 200
> intloop22 G A G U G C C C = 200
> intloop22 G A G U G C G C = 200
> intloop22 G A G U G C U C = 200
> intloop22 G A G U G G A C = 140
> intloop22 G A G U G G C C = 110
> intloop22 G A G U G G G C = 0
> intloop22 G A G U G G U C = 200
> intloop22 G A G U G U A C = 70
> intloop22 G A G U G U C C = -60
> intloop22 G A G U G U G C = 10
> intloop22 G A G U G U U C = 200
> intloop22 G A U U G A A C = 200
> intloop22 G A U U G A C C = 200
> intloop22 G A U U G A G C = 200
> intloop22 G A U U G A U C = 200
> intloop22 G A U U G C A C = 240
> intloop22 G A U U G C C C = 150
> intloop22 G A U U G C G C = 140
> intloop22 G A U U G C U C = 200
> intloop22 G A U U G G A C = 170
> intloop22 G A U U G G C C = 40
> intloop22 G A U U G G G C = 110
> intloop22 G A U U G G U C = 200
> intloop22 G A U U G U A C = 200
> intloop22 G A U U G U C C = 70
> intloop22 G A U U G U G C = 150
> intloop22 G A U U G U U C = 200
> intloop22 G C A U G A A C = 190
> intloop22 G C A U G A C C = 250
> intloop22 G C A U G A G C = 200
> intloop22 G C A U G A U C = 250
> intloop22 G C A U G C A C = 160
> intloop22 G C A U G C C C = 160
> intloop22 G C A U G C G C = 200
> intloop22 G C A U G C U C = 170
> intloop22 G C A U G G A C = 60
> intloop22 G C A U G G C C = 160
> intloop22 G C A U G G G C = 200
> intloop22 G C A U G G U C = 170
> intloop22 G C A U G U A C = 200
> intloop22 G C A U G U C C = 200
> intloop22 G C A U G U G C = 200
> intloop22 G C A U G U U C = 200
> intloop22 G C C U G A A C = 170
> intloop22 G C C U G A C C = 170
> intloop22 G C C U G A G C = 200
> intloop22 G C C U G A U C = 180
> intloop22 G C C U G C A C = 160
> intloop22 G C C U G C C C = 160
> intloop22 G C C U G C G C = 200
> intloop22 G C C U G C U C = 160
> intloop22 G C C U G G A C = 200
> intloop22 G C C U G G C C = 200
> intloop22 G C C U G G G C = 200
> intloop22 G C C U G G U C = 200
> intloop22 G C C U G U A C = 160
> intloop22 G C C U G U C C = 160
> intloop22 G C C U G U G C = 200
> intloop22 G C C U G U U C = 160
> intloop22 G C G U G A A C = 60
> intloop22 G C G U G A C C = 160
> intloop22 G C G U G A G C = 200
> intloop22 G C G U G A U C = 170
> intloop22 G C G U G C A C = 200
> intloop22 G C G U G C C C = 200
> intloop22 G C G U G C G C = 200
> intloop22 G C G U G C U C = 200
> intloop22 G C G U G G A C = 120
> intloop22 G C G U G G C C = 180
> intloop22 G C G U G G G C = 200
> intloop22 G C G U G G U C = 180
> intloop22 G C G U G U A C = -50
> intloop22 G C G U G U C C = 50
> intloop22 G C G U G U G C = 200
> intloop22 G C G U G U U C = 50
> intloop22 G C U U G A A C = 200
> intloop22 G C U U G A C C = 200
> intloop22 G C U U G A G C = 200
> intloop22 G C U U G A U C = 200
> intloop22 G C U U G C A C = 160
> intloop22 G C U U G C C C = 160
> intloop22 G C U U G C G C = 200
> intloop22 G C U U G C U C = 160
> intloop22 G C U U G G A C = 40
> intloop22 G C U U G G C C = 140
> intloop22 G C U U G G G C = 200
> intloop22 G C U U G G U C = 150
> intloop22 G C U U G U A C = 80
> intloop22 G C U U G U C C = 80
> intloop22 G C U U G U G C = 200
> intloop22 G C U U G U U C = 80
> intloop22 G G A U G A A C = 10
> intloop22 G G A U G A C C = 200
> intloop22 G G A U G A G C = 180
> intloop22 G G A U G A U C = 40
> intloop22 G G A U G C A C = -10
> intloop22 G G A U G C C C = 200
> intloop22 G G A U G C G C = 150
> intloop22 G G A U G C U C = -90
> intloop22 G G A U G G A C = -110
> intloop22 G G A U G G C C = 200
> intloop22 G G A U G G G C = 50
> intloop22 G G A U G G U C = -10
> intloop22 G G A U G U A C = 200
> intloop22 G G A U G U C C = 200
> intloop22 G G A U G U G C = 200
> intloop22 G G A U G U U C = 200
> intloop22 G G C U G A A C = 0
> intloop22 G G C U G A C C = 200
> intloop22 G G C U G A G C = 160
> intloop22 G G C U G A U C = -80
> intloop22 G G C U G C A C = 80
> intloop22 G G C U G C C C = 200
> intloop22 G G C U G C G C = 210
> intloop22 G G C U G C U C = 10
> intloop22 G G C U G G A C = 200
> intloop22 G G C U G G C C = 200
> intloop22 G G C U G G G C = 200
> intloop22 G G C U G G U C = 200
> intloop22 G G C U G U A C = 80
> intloop22 G G C U G U C C = 200
> intloop22 G G C U G U G C = 210
> intloop22 G G C U G U U C = 10
> intloop22 G G G U G A A C = -110
> intloop22 G G G U G A C C = 200
> intloop22 G G G U G A G C = 50
> intloop22 G G G U G A U C = -10
> intloop22 G G G U G C A C = 200
> intloop22 G G G U G C C C = 200
> intloop22 G G G U G C G C = 200
> intloop22 G G G U G C U C = 200
> intloop22 G G G U G G A C = -60
> intloop22 G G G U G G C C = 200
> intloop22 G G G U G G G C = 110
> intloop22 G G G U G G U C = -30
> intloop22 G G G U G U A C = -50
> intloop22 G G G U G U C C = 200
> intloop22 G G G U G U G C = 40
> intloop22 G G G U G U U C = -310
> intloop22 G G U U G A A C = 200
> intloop22 G G U U G A C C = 200
> intloop22 G G U U G A G C = 200
> intloop22 G G U U G A U C = 200
> intloop22 G G U U G C A C = 80
> intloop22 G G U U G C C C = 200
> intloop22 G G U U G C G C = 210
> intloop22 G G U U G C U C = 10
> intloop22 G G U U G G A C = 50
> intloop22 G G U U G G C C = 200
> intloop22 G G U U G G G C = 130
> intloop22 G G U U G G U C = -210
> intloop22 G G U U G U A C = 80
> intloop22 G G U U G U C C = 200
> intloop22 G G U U G U G C = 170
> intloop22 G G U U G U U C = 10
> intloop22 G U A U G A A C = 200
> intloop22 G U A U G A C C = 150
> intloop22 G U A U G A G C = 0
> intloop22 G U A U G A U C = 210
> intloop22 G U A U G C A C = 200
> intloop22 G U A U G C C C = 60
> intloop22 G U A U G C G C = -130
> intloop22 G U A U G C U C = 90
> intloop22 G U A U G G A C = 200
> intloop22 G U A U G G C C = 70
> intloop22 G U A U G G G C = -50
> intloop22 G U A U G G U C = 170
> intloop22 G U A U G U A C = 200
> intloop22 G U A U G U C C = 200
> intloop22 G U A U G U G C = 200
> intloop22 G U A U G U U C = 200
> intloop22 G U C U G A A C = 200
> intloop22 G U C U G A C C = 70
> intloop22 G U C U G A G C = -120
> intloop22 G U C U G A U C = 100
> intloop22 G U C U G C A C = 200
> intloop22 G U C U G C C C = 60
> intloop22 G U C U G C G C = -30
> intloop22 G U C U G C U C = 80
> intloop22 G U C U G G A C = 200
> intloop22 G U C U G G C C = 200
> intloop22 G U C U G G G C = 200
> intloop22 G U C U G G U C = 200
> intloop22 G U C U G U A C = 200
> intloop22 G U C U G U C C = 60
> intloop22 G U C U G U G C = -30
> intloop22 G U C U G U U C = 80
> intloop22 G U G U G A A C = 200
> intloop22 G U G U G A C C = 70
> intloop22 G U G U G A G C = -50
> intloop22 G U G U G A U C = 170
> intloop22 G U G U G C A C = 200
> intloop22 G U G U G C C C = 200
> intloop22 G U G U G C G C = 200
> intloop22 G U G U G C U C = 200
> intloop22 G U G U G G A C = 200
> intloop22 G U G U G G C C = 80
> intloop22 G U G U G G G C = -70
> intloop22 G U G U G G U C = 140
> intloop22 G U G U G U A C = 200
> intloop22 G U G U G U C C = -50
> intloop22 G U G U G U G C = -350
> intloop22 G U G U G U U C = 50
> intloop22 G U U U G A A C = 200
> intloop22 G U U U G A C C = 200
> intloop22 G U U U G A G C = 200
> intloop22 G U U U G A U C = 200
> intloop22 G U U U G C A C = 200
> intloop22 G U U U G C C C = 60
> intloop22 G U U U G C G C = -30
> intloop22 G U U U G C U C = 80
> intloop22 G U U U G G A C = 200
> intloop22 G U U U G G C C = 50
> intloop22 G U U U G G G C = -250
> intloop22 G U U U G G U C = 150
> intloop22 G U U U G U A C = 200
> intloop22 G U U U G U C C = -20
> intloop22 G U U U G U G C = -30
> intloop22 G U U U G U U C = 0
> intloop22 G A A G U A A C = 210
> intloop22 G A A G U A C C = 180
> intloop22 G A A G U A G C = 70
> intloop22 G A A G U A U C = 200
> intloop22 G A A G U C A C = 170
> intloop22 G A A G U C C C = 140
> intloop22 G A A G U C G C = 30
> intloop22 G A A G U C U C = 200
> intloop22 G A A G U G A C = 110
> intloop22 G A A G U G C C = 80
> intloop22 G A A G U G G C = -30
> intloop22 G A A G U G U C = 200
> intloop22 G A A G U U A C = 200
> intloop22 G A A G U U C C = 200
> intloop22 G A A G U U G C = 200
> intloop22 G A A G U U U C = 200
> intloop22 G A C G U A A C = 210
> intloop22 G A C G U A C C = 180
> intloop22 G A C G U A G C = 70
> intloop22 G A C G U A U C = 200
> intloop22 G A C G U C A C = 270
> intloop22 G A C G U C C C = 180
> intloop22 G A C G U C G C = 170
> intloop22 G A C G U C U C = 200
> intloop22 G A C G U G A C = 200
> intloop22 G A C G U G C C = 200
> intloop22 G A C G U G G C = 200
> intloop22 G A C G U G U C = 200
> intloop22 G A C G U U A C = 270
> intloop22 G A C G U U C C = 180
> intloop22 G A C G U U G C = 170
> intloop22 G A C G U U U C = 200
> intloop22 G A G G U A A C = 110
> intloop22 G A G G U A C C = 80
> intloop22 G A G G U A G C = -30
> intloop22 G A G G U A U C = 200
> intloop22 G A G G U C A C = 200
> intloop22 G A G G U C C C = 200
> intloop22 G A G G U C G C = 200
> intloop22 G A G G U C U C = 200
> intloop22 G A G G U G A C = 150
> intloop22 G A G G U G C C = 120
> intloop22 G A G G U G G C = 10
> intloop22 G A G G U G U C = 200
> intloop22 G A G G U U A C = 30
> intloop22 G A G G U U C C = -100
> intloop22 G A G G U U G C = -30
> intloop22 G A G G U U U C = 200
> intloop22 G A U G U A A C = 200
> intloop22 G A U G U A C C = 200
> intloop22 G A U G U A G C = 200
> intloop22 G A U G U A U C = 200
> intloop22 G A U G U C A C = 240
> intloop22 G A U G U C C C = 150
> intloop22 G A U G U C G C = 140
> intloop22 G A U G U C U C = 200
> intloop22 G A U G U G A C = 160
> intloop22 G A U G U G C C = 30
> intloop22 G A U G U G G C = 100
> intloop22 G A U G U G U C = 200
> intloop22 G A U G U U A C = 230
> intloop22 G A U G U U C C = 100
> intloop22 G A U G U U G C = 170
> intloop22 G A U G U U U C = 200
> intloop22 G C A G U A A C = 190
> intloop22 G C A G U A C C = 250
> intloop22 G C A G U A G C = 200
> intloop22 G C A G U A U C = 250
> intloop22 G C A G U C A C = 140
> intloop22 G C A G U C C C = 140
> intloop22 G C A G U C G C = 200
> intloop22 G C A G U C U C = 150
> intloop22 G C A G U G A C = 80
> intloop22 G C A G U G C C = 180
> intloop22 G C A G U G G C = 200
> intloop22 G C A G U G U C = 190
> intloop22 G C A G U U A C = 200
> intloop22 G C A G U U C C = 200
> intloop22 G C A G U U G C = 200
> intloop22 G C A G U U U C = 200
> intloop22 G C C G U A A C = 190
> intloop22 G C C G U A C C = 190
> intloop22 G C C G U A G C = 200
> intloop22 G C C G U A U C = 190
> intloop22 G C C G U C A C = 190
> intloop22 G C C G U C C C = 190
> intloop22 G C C G U C G C = 200
> intloop22 G C C G U C U C = 190
> intloop22 G C C G U G A C = 200
> intloop22 G C C G U G C C = 200
> intloop22 G C C G U G G C = 200
> intloop22 G C C G U G U C = 200
> intloop22 G C C G U U A C = 190
> intloop22 G C C G U U C C = 190
> intloop22 G C C G U U G C = 200
> intloop22 G C C G U U U C = 190
> intloop22 G C G G U A A C = 80
> intloop22 G C G G U A C C = 180
> intloop22 G C G G U A G C = 200
> intloop22 G C G G U A U C = 190
> intloop22 G C G G U C A C = 200
> intloop22 G C G G U C C C = 200
> intloop22 G C G G U C G C = 200
> intloop22 G C G G U C U C = 200
> intloop22 G C G G U G A C = 120
> intloop22 G C G G U G C C = 180
> intloop22 G C G G U G G C = 200
> intloop22 G C G G U G U C = 190
> intloop22 G C G G U U A C = -90
> intloop22 G C G G U U C C = 10
> intloop22 G C G G U U G C = 200
> intloop22 G C G G U U U C = 10
> intloop22 G C U G U A A C = 200
> intloop22 G C U G U A C C = 200
> intloop22 G C U G U A G C = 200
> intloop22 G C U G U A U C = 200
> intloop22 G C U G U C A C = 160
> intloop22 G C U G U C C C = 160
> intloop22 G C U G U C G C = 200
> intloop22 G C U G U C U C = 160
> intloop22 G C U G U G A C = 30
> intloop22 G C U G U G C C = 130
> intloop22 G C U G U G G C = 200
> intloop22 G C U G U G U C = 140
> intloop22 G C U G U U A C = 100
> intloop22 G C U G U U C C = 100
> intloop22 G C U G U U G C = 200
> intloop22 G C U G U U U C = 110
> intloop22 G G A G U A A C = 10
> intloop22 G G A G U A C C = 200
> intloop22 G G A G U A G C = 180
> intloop22 G G A G U A U C = 40
> intloop22 G G A G U C A C = -30
> intloop22 G G A G U C C C = 200
> intloop22 G G A G U C G C = 130
> intloop22 G G A G U C U C = -110
> intloop22 G G A G U G A C = -90
> intloop22 G G A G U G C C = 200
> intloop22 G G A G U G G C = 70
> intloop22 G G A G U G U C = 10
> intloop22 G G A G U U A C = 200
> intloop22 G G A G U U C C = 200
> intloop22 G G A G U U G C = 200
> intloop22 G G A G U U U C = 200
> intloop22 G G C G U A A C = 10
> intloop22 G G C G U A C C = 200
> intloop22 G G C G U A G C = 180
> intloop22 G G C G U A U C = -60
> intloop22 G G C G U C A C = 110
> intloop22 G G C G U C C C = 200
> intloop22 G G C G U C G C = 240
> intloop22 G G C G U C U C = 40
> intloop22 G G C G U G A C = 200
> intloop22 G G C G U G C C = 200
> intloop22 G G C G U G G C = 200
> intloop22 G G C G U G U C = 200
> intloop22 G G C G U U A C = 110
> intloop22 G G C G U U C C = 200
> intloop22 G G C G U U G C = 240
> intloop22 G G C G U U U C = 40
> intloop22 G G G G U A A C = -90
> intloop22 G G G G U A C C = 200
> intloop22 G G G G U A G C = 70
> intloop22 G G G G U A U C = 10
> intloop22 G G G G U C A C = 200
> intloop22 G G G G U C C C = 200
> intloop22 G G G G U C G C = 200
> intloop22 G G G G U C U C = 200
> intloop22 G G G G U G A C = -50
> intloop22 G G G G U G C C = 200
> intloop22 G G G G U G G C = 110
> intloop22 G G G G U G U C = -30
> intloop22 G G G G U U A C = -90
> intloop22 G G G G U U C C = 200
> intloop22 G G G G U U G C = 0
> intloop22 G G G G U U U C = -350
> intloop22 G G U G U A A C = 200
> intloop22 G G U G U A C C = 200
> intloop22 G G U G U A G C = 200
> intloop22 G G U G U A U C = 200
> intloop22 G G U G U C A C = 80
> intloop22 G G U G U C C C = 200
> intloop22 G G U G U C G C = 210
> intloop22 G G U G U C U C = 10
> intloop22 G G U G U G A C = 40
> intloop22 G G U G U G C C = 200
> intloop22 G G U G U G G C = 120
> intloop22 G G U G U G U C = -220
> intloop22 G G U G U U A C = 110
> intloop22 G G U G U U C C = 200
> intloop22 G G U G U U G C = 190
> intloop22 G G U G U U U C = 30
> intloop22 G U A G U A A C = 200
> intloop22 G U A G U A C C = 150
> intloop22 G U A G U A G C = 0
> intloop22 G U A G U A U C = 210
> intloop22 G U A G U C A C = 200
> intloop22 G U A G U C C C = 40
> intloop22 G U A G U C G C = -150
> intloop22 G U A G U C U C = 70
> intloop22 G U A G U G A C = 200
> intloop22 G U A G U G C C = 90
> intloop22 G U A G U G G C = -30
> intloop22 G U A G U G U C = 190
> intloop22 G U A G U U A C = 200
> intloop22 G U A G U U C C = 200
> intloop22 G U A G U U G C = 200
> intloop22 G U A G U U U C = 200
> intloop22 G U C G U A A C = 200
> intloop22 G U C G U A C C = 90
> intloop22 G U C G U A G C = -100
> intloop22 G U C G U A U C = 110
> intloop22 G U C G U C A C = 200
> intloop22 G U C G U C C C = 90
> intloop22 G U C G U C G C = 0
> intloop22 G U C G U C U C = 110
> intloop22 G U C G U G A C = 200
> intloop22 G U C G U G C C = 200
> intloop22 G U C G U G G C = 200
> intloop22 G U C G U G U C = 200
> intloop22 G U C G U U A C = 200
> intloop22 G U C G U U C C = 90
> intloop22 G U C G U U G C = 0
> intloop22 G U C G U U U C = 110
> intloop22 G U G G U A A C = 200
> intloop22 G U G G U A C C = 90
> intloop22 G U G G U A G C = -30
> intloop22 G U G G U A U C = 190
> intloop22 G U G G U C A C = 200
> intloop22 G U G G U C C C = 200
> intloop22 G U G G U C G C = 200
> intloop22 G U G G U C U C = 200
> intloop22 G U G G U G A C = 200
> intloop22 G U G G U G C C = 80
> intloop22 G U G G U G G C = -70
> intloop22 G U G G U G U C = 150
> intloop22 G U G G U U A C = 200
> intloop22 G U G G U U C C = -90
> intloop22 G U G G U U G C = -390
> intloop22 G U G G U U U C = 10
> intloop22 G U U G U A A C = 200
> intloop22 G U U G U A C C = 200
> intloop22 G U U G U A G C = 200
> intloop22 G U U G U A U C = 200
> intloop22 G U U G U C A C = 200
> intloop22 G U U G U C C C = 60
> intloop22 G U U G U C G C = -30
> intloop22 G U U G U C U C = 80
> intloop22 G U U G U G A C = 200
> intloop22 G U U G U G C C = 40
> intloop22 G U U G U G G C = -260
> intloop22 G U U G U G U C = 140
> intloop22 G U U G U U A C = 200
> intloop22 G U U G U U C C = 0
> intloop22 G U U G U U G C = -10
> intloop22 G U U G U U U C = 30
> intloop22 G A A U A A A C = 210
> intloop22 G A A U A A C C = 180
> intloop22 G A A U A A G C = 70
> intloop22 G A A U A A U C = 200
> intloop22 G A A U A C A C = 190
> intloop22 G A A U A C C C = 160
> intloop22 G A A U A C G C = 50
> intloop22 G A A U A C U C = 200
> intloop22 G A A U A G A C = 90
> intloop22 G A A U A G C C = 60
> intloop22 G A A U A G G C = -50
> intloop22 G A A U A G U C = 200
> intloop22 G A A U A U A C = 200
> intloop22 G A A U A U C C = 200
> intloop22 G A A U A U G C = 200
> intloop22 G A A U A U U C = 200
> intloop22 G A C U A A A C = 200
> intloop22 G A C U A A C C = 170
> intloop22 G A C U A A G C = 60
> intloop22 G A C U A A U C = 200
> intloop22 G A C U A C A C = 240
> intloop22 G A C U A C C C = 150
> intloop22 G A C U A C G C = 140
> intloop22 G A C U A C U C = 200
> intloop22 G A C U A G A C = 200
> intloop22 G A C U A G C C = 200
> intloop22 G A C U A G G C = 200
> intloop22 G A C U A G U C = 200
> intloop22 G A C U A U A C = 240
> intloop22 G A C U A U C C = 150
> intloop22 G A C U A U G C = 140
> intloop22 G A C U A U U C = 200
> intloop22 G A G U A A A C = 90
> intloop22 G A G U A A C C = 60
> intloop22 G A G U A A G C = -50
> intloop22 G A G U A A U C = 200
> intloop22 G A G U A C A C = 200
> intloop22 G A G U A C C C = 200
> intloop22 G A G U A C G C = 200
> intloop22 G A G U A C U C = 200
> intloop22 G A G U A G A C = 140
> intloop22 G A G U A G C C = 110
> intloop22 G A G U A G G C = 0
> intloop22 G A G U A G U C = 200
> intloop22 G A G U A U A C = 70
> intloop22 G A G U A U C C = -60
> intloop22 G A G U A U G C = 10
> intloop22 G A G U A U U C = 200
> intloop22 G A U U A A A C = 200
> intloop22 G A U U A A C C = 200
> intloop22 G A U U A A G C = 200
> intloop22 G A U U A A U C = 200
> intloop22 G A U U A C A C = 240
> intloop22 G A U U A C C C = 150
> intloop22 G A U U A C G C = 140
> intloop22 G A U U A C U C = 200
> intloop22 G A U U A G A C = 170
> intloop22 G A U U A G C C = 40
> intloop22 G A U U A G G C = 110
> intloop22 G A U U A G U C = 200
> intloop22 G A U U A U A C = 200
> intloop22 G A U U A U C C = 70
> intloop22 G A U U A U G C = 150
> intloop22 G A U U A U U C = 200
> intloop22 G C A U A A A C = 190
> intloop22 G C A U A A C C = 250
> intloop22 G C A U A A G C = 200
> intloop22 G C A U A A U C = 250
> intloop22 G C A U A C A C = 160
> intloop22 G C A U A C C C = 160
> intloop22 G C A U A C G C = 200
> intloop22 G C A U A C U C = 170
> intloop22 G C A U A G A C = 60
> intloop22 G C A U A G C C = 160
> intloop22 G C A U A G G C = 200
> intloop22 G C A U A G U C = 170
> intloop22 G C A U A U A C = 200
> intloop22 G C A U A U C C = 200
> intloop22 G C A U A U G C = 200
> intloop22 G C A U A U U C = 200
> intloop22 G C C U A A A C = 170
> intloop22 G C C U A A C C = 170
> intloop22 G C C U A A G C = 200
> intloop22 G C C U A A U C = 180
> intloop22 G C C U A C A C = 160
> intloop22 G C C U A C C C = 160
> intloop22 G C C U A C G C = 200
> intloop22 G C C U A C U C = 160
> intloop22 G C C U A G A C = 200
> intloop22 G C C U A G C C = 200
> intloop22 G C C U A G G C = 200
> intloop22 G C C U A G U C = 200
> intloop22 G C C U A U A C = 160
> intloop22 G C C U A U C C = 160
> intloop22 G C C U A U G C = 200
> intloop22 G C C U A U U C = 160
> intloop22 G C G U A A A C = 60
> intloop22 G C G U A A C C = 160
> intloop22 G C G U A A G C = 200
> intloop22 G C G U A A U C = 170
> intloop22 G C G U A C A C = 200
> intloop22 G C G U A C C C = 200
> intloop22 G C G U A C G C = 200
> intloop22 G C G U A C U C = 200
> intloop22 G C G U A G A C = 120
> intloop22 G C G U A G C C = 180
> intloop22 G C G U A G G C = 200
> intloop22 G C G U A G U C = 180
> intloop22 G C G U A U A C = -50
> intloop22 G C G U A U C C = 50
> intloop22 G C G U A U G C = 200
> intloop22 G C G U A U U C = 50
> intloop22 G C U U A A A C = 200
> intloop22 G C U U A A C C = 200
> intloop22 G C U U A A G C = 200
> intloop22 G C U U A A U C = 200
> intloop22 G C U U A C A C = 160
> intloop22 G C U U A C C C = 160
> intloop22 G C U U A C G C = 200
> intloop22 G C U U A C U C = 160
> intloop22 G C U U A G A C = 40
> intloop22 G C U U A G C C = 140
> intloop22 G C U U A G G C = 200
> intloop22 G C U U A G U C = 150
> intloop22 G C U U A U A C = 80
> intloop22 G C U U A U C C = 80
> intloop22 G C U U A U G C = 200
> intloop22 G C U U A U U C = 80
> intloop22 G G A U A A A C = 10
> intloop22 G G A U A A C C = 200
> intloop22 G G A U A A G C = 180
> intloop22 G G A U A A U C = 40
> intloop22 G G A U A C A C = -10
> intloop22 G G A U A C C C = 200
> intloop22 G G A U A C G C = 150
> intloop22 G G A U A C U C = -90
> intloop22 G G A U A G A C = -110
> intloop22 G G A U A G C C = 200
> intloop22 G G A U A G G C = 50
> intloop22 G G A U A G U C = -10
> intloop22 G G A U A U A C = 200
> intloop22 G G A U A U C C = 200
> intloop22 G G A U A U G C = 200
> intloop22 G G A U A U U C = 200
> intloop22 G G C U A A A C = 0
> intloop22 G G C U A A C C = 200
> intloop22 G G C U A A G C = 160
> intloop22 G G C U A A U C = -80
> intloop22 G G C U A C A C = 80
> intloop22 G G C U A C C C = 200
> intloop22 G G C U A C G C = 210
> intloop22 G G C U A C U C = 10
> intloop22 G G C U A G A C = 200
> intloop22 G G C U A G C C = 200
> intloop22 G G C U A G G C = 200
> intloop22 G G C U A G U C = 200
> intloop22 G G C U A U A C = 80
> intloop22 G G C U A U C C = 200
> intloop22 G G C U A U G C = 210
> intloop22 G G C U A U U C = 10
> intloop22 G G G U A A A C = -110
> intloop22 G G G U A A C C = 200
> intloop22 G G G U A A G C = 50
> intloop22 G G G U A A U C = -10
> intloop22 G G G U A C A C = 200
> intloop22 G G G U A C C C = 200
> intloop22 G G G U A C G C = 200
> intloop22 G G G U A C U C = 200
> intloop22 G G G U A G A C = -60
> intloop22 G G G U A G C C = 200
> intloop22 G G G U A G G C = 110
> intloop22 G G G U A G U C = -30
> intloop22 G G G U A U A C = -50
> intloop22 G G G U A U C C = 200
> intloop22 G G G U A U G C = 40
> intloop22 G G G U A U U C = -310
> intloop22 G G U U A A A C = 200
> intloop22 G G U U A A C C = 200
> intloop22 G G U U A A G C = 200
> intloop22 G G U U A A U C = 200
> intloop22 G G U U A C A C = 80
> intloop22 G G U U A C C C = 200
> intloop22 G G U U A C G C = 210
> intloop22 G G U U A C U C = 10
> intloop22 G G U U A G A C = 50
> intloop22 G G U U A G C C = 200
> intloop22 G G U U A G G C = 130
> intloop22 G G U U A G U C = -210
> intloop22 G G U U A U A C = 80
> intloop22 G G U U A U C C = 200
> intloop22 G G U U A U G C = 170
> intloop22 G G U U A U U C = 10
> intloop22 G U A U A A A C = 200
> intloop22 G U A U A A C C = 150
> intloop22 G U A U A A G C = 0
> intloop22 G U A U A A U C = 210
> intloop22 G U A U A C A C = 200
> intloop22 G U A U A C C C = 60
> intloop22 G U A U A C G C = -130
> intloop22 G U A U A C U C = 90
> intloop22 G U A U A G A C = 200
> intloop22 G U A U A G C C = 70
> intloop22 G U A U A G G C = -50
> intloop22 G U A U A G U C = 170
> intloop22 G U A U A U A C = 200
> intloop22 G U A U A U C C = 200
> intloop22 G U A U A U G C = 200
> intloop22 G U A U A U U C = 200
> intloop22 G U C U A A A C = 200
> intloop22 G U C U A A C C = 70
> intloop22 G U C U A A G C = -120
> intloop22 G U C U A A U C = 100
> intloop22 G U C U A C A C = 200
> intloop22 G U C U A C C C = 60
> intloop22 G U C U A C G C = -30
> intloop22 G U C U A C U C = 80
> intloop22 G U C U A G A C = 200
> intloop22 G U C U A G C C = 200
> intloop22 G U C U A G G C = 200
> intloop22 G U C U A G U C = 200
> intloop22 G U C U A U A C = 200
> intloop22 G U C U A U C C = 60
> intloop22 G U C U A U G C = -30
> intloop22 G U C U A U U C = 80
> intloop22 G U G U A A A C = 200
> intloop22 G U G U A A C C = 70
> intloop22 G U G U A A G C = -50
> intloop22 G U G U A A U C = 170
> intloop22 G U G U A C A C = 200
> intloop22 G U G U A C C C = 200
> intloop22 G U G U A C G C = 200
> intloop22 G U G U A C U C = 200
> intloop22 G U G U A G A C = 200
> intloop22 G U G U A G C C = 80
> intloop22 G U G U A G G C = -70
> intloop22 G U G U A G U C = 140
> intloop22 G U G U A U A C = 200
> intloop22 G U G U A U C C = -50
> intloop22 G U G U A U G C = -350
> intloop22 G U G U A U U C = 50
> intloop22 G U U U A A A C = 200
> intloop22 G U U U A A C C = 200
> intloop22 G U U U A A G C = 200
> intloop22 G U U U A A U C = 200
> intloop22 G U U U A C A C = 200
> intloop22 G U U U A C C C = 60
> intloop22 G U U U A C G C = -30
> intloop22 G U U U A C U C = 80
> intloop22 G U U U A G A C = 200
> intloop22 G U U U A G C C = 50
> intloop22 G U U U A G G C = -250
> intloop22 G U U U A G U C = 150
> intloop22 G U U U A U A C = 200
> intloop22 G U U U A U C C = -20
> intloop22 G U U U A U G C = -30
> intloop22 G U U U A U U C = 0
> intloop22 G A A A U A A C = 210
> intloop22 G A A A U A C C = 180
> intloop22 G A A A U A G C = 70
> intloop22 G A A A U A U C = 200
> intloop22 G A A A U C A C = 170
> intloop22 G A A A U C C C = 140
> intloop22 G A A A U C G C = 30
> intloop22 G A A A U C U C = 200
> intloop22 G A A A U G A C = 110
> intloop22 G A A A U G C C = 80
> intloop22 G A A A U G G C = -30
> intloop22 G A A A U G U C = 200
> intloop22 G A A A U U A C = 200
> intloop22 G A A A U U C C = 200
> intloop22 G A A A U U G C = 200
> intloop22 G A A A U U U C = 200
> intloop22 G A C A U A A C = 210
> intloop22 G A C A U A C C = 180
> intloop22 G A C A U A G C = 70
> intloop22 G A C A U A U C = 200
> intloop22 G A C A U C A C = 270
> intloop22 G A C A U C C C = 180
> intloop22 G A C A U C G C = 170
> intloop22 G A C A U C U C = 200
> intloop22 G A C A U G A C = 200
> intloop22 G A C A U G C C = 200
> intloop22 G A C A U G G C = 200
> intloop22 G A C A U G U C = 200
> intloop22 G A C A U U A C = 270
> intloop22 G A C A U U C C = 180
> intloop22 G A C A U U G C = 170
> intloop22 G A C A U U U C = 200
> intloop22 G A G A U A A C = 110
> intloop22 G A G A U A C C = 80
> intloop22 G A G A U A G C = -30
> intloop22 G A G A U A U C = 200
> intloop22 G A G A U C A C = 200
> intloop22 G A G A U C C C = 200
> intloop22 G A G A U C G C = 200
> intloop22 G A G A U C U C = 200
> intloop22 G A G A U G A C = 150
> intloop22 G A G A U G C C = 120
> intloop22 G A G A U G G C = 10
> intloop22 G A G A U G U C = 200
> intloop22 G A G A U U A C = 30
> intloop22 G A G A U U C C = -100
> intloop22 G A G A U U G C = -30
> intloop22 G A G A U U U C = 200
> intloop22 G A U A U A A C = 200
> intloop22 G A U A U A C C = 200
> intloop22 G A U A U A G C = 200
> intloop22 G A U A U A U C = 200
> intloop22 G A U A U C A C = 240
> intloop22 G A U A U C C C = 150
> intloop22 G A U A U C G C = 140
> intloop22 G A U A U C U C = 200
> intloop22 G A U A U G A C = 160
> intloop22 G A U A U G C C = 30
> intloop22 G A U A U G G C = 100
> intloop22 G A U A U G U C = 200
> intloop22 G A U A U U A C = 230
> intloop22 G A U A U U C C = 100
> intloop22 G A U A U U G C = 170
> intloop22 G A U A U U U C = 200
> intloop22 G C A A U A A C = 190
> intloop22 G C A A U A C C = 250
> intloop22 G C A A U A G C = 200
> intloop22 G C A A U A U C = 250
> intloop22 G C A A U C A C = 140
> intloop22 G C A A U C C C = 140
> intloop22 G C A A U C G C = 200
> intloop22 G C A A U C U C = 150
> intloop22 G C A A U G A C = 80
> intloop22 G C A A U G C C = 180
> intloop22 G C A A U G G C = 200
> intloop22 G C A A U G U C = 190
> intloop22 G C A A U U A C = 200
> intloop22 G C A A U U C C = 200
> intloop22 G C A A U U G C = 200
> intloop22 G C A A U U U C = 200
> intloop22 G C C A U A A C = 190
> intloop22 G C C A U A C C = 190
> intloop22 G C C A U A G C = 200
> intloop22 G C C A U A U C = 190
> intloop22 G C C A U C A C = 190
> intloop22 G C C A U C C C = 190
> intloop22 G C C A U C G C = 200
> intloop22 G C C A U C U C = 190
> intloop22 G C C A U G A C = 200
> intloop22 G C C A U G C C = 200
> intloop22 G C C A U G G C = 200
> intloop22 G C C A U G U C = 200
> intloop22 G C C A U U A C = 190
> intloop22 G C C A U U C C = 190
> intloop22 G C C A U U G C = 200
> intloop22 G C C A U U U C = 190
> intloop22 G C G A U A A C = 80
> intloop22 G C G A U A C C = 180
> intloop22 G C G A U A G C = 200
> intloop22 G C G A U A U C = 190
> intloop22 G C G A U C A C = 200
> intloop22 G C G A U C C C = 200
> intloop22 G C G A U C G C = 200
> intloop22 G C G A U C U C = 200
> intloop22 G C G A U G A C = 120
> intloop22 G C G A U G C C = 180
> intloop22 G C G A U G G C = 200
> intloop22 G C G A U G U C = 190
> intloop22 G C G A U U A C = -90
> intloop22 G C G A U U C C = 10
> intloop22 G C G A U U G C = 200
> intloop22 G C G A U U U C = 10
> intloop22 G C U A U A A C = 200
> intloop22 G C U A U A C C = 200
> intloop22 G C U A U A G C = 200
> intloop22 G C U A U A U C = 200
> intloop22 G C U A U C A C = 160
> intloop22 G C U A U C C C = 160
> intloop22 G C U A U C G C = 200
> intloop22 G C U A U C U C = 160
> intloop22 G C U A U G A C = 30
> intloop22 G C U A U G C C = 130
> intloop22 G C U A U G G C = 200
> intloop22 G C U A U G U C = 140
> intloop22 G C U A U U A C = 100
> intloop22 G C U A U U C C = 100
> intloop22 G C U A U U G C = 200
> intloop22 G C U A U U U C = 110
> intloop22 G G A A U A A C = 10
> intloop22 G G A A U A C C = 200
> intloop22 G G A A U A G C = 180
> intloop22 G G A A U A U C = 40
> intloop22 G G A A U C A C = -30
> intloop22 G G A A U C C C = 200
> intloop22 G G A A U C G C = 130
> intloop22 G G A A U C U C = -110
> intloop22 G G A A U G A C = -90
> intloop22 G G A A U G C C = 200
> intloop22 G G A A U G G C = 70
> intloop22 G G A A U G U C = 10
> intloop22 G G A A U U A C = 200
> intloop22 G G A A U U C C = 200
> intloop22 G G A A U U G C = 200
> intloop22 G G A A U U U C = 200
> intloop22 G G C A U A A C = 10
> intloop22 G G C A U A C C = 200
> intloop22 G G C A U A G C = 180
> intloop22 G G C A U A U C = -60
> intloop22 G G C A U C A C = 110
> intloop22 G G C A U C C C = 200
> intloop22 G G C A U C G C = 240
> intloop22 G G C A U C U C = 40
> intloop22 G G C A U G A C = 200
> intloop22 G G C A U G C C = 200
> intloop22 G G C A U G G C = 200
> intloop22 G G C A U G U C = 200
> intloop22 G G C A U U A C = 110
> intloop22 G G C A U U C C = 200
> intloop22 G G C A U U G C = 240
> intloop22 G G C A U U U C = 40
> intloop22 G G G A U A A C = -90
> intloop22 G G G A U A C C = 200
> intloop22 G G G A U A G C = 70
> intloop22 G G G A U A U C = 10
> intloop22 G G G A U C A C = 200
> intloop22 G G G A U C C C = 200
> intloop22 G G G A U C G C = 200
> intloop22 G G G A U C U C = 200
> intloop22 G G G A U G A C = -50
> intloop22 G G G A U G C C = 200
> intloop22 G G G A U G G C = 110
> intloop22 G G G A U G U C = -30
> intloop22 G G G A U U A C = -90
> intloop22 G G G A U U C C = 200
> intloop22 G G G A U U G C = 0
> intloop22 G G G A U U U C = -350
> intloop22 G G U A U A A C = 200
> intloop22 G G U A U A C C = 200
> intloop22 G G U A U A G C = 200
> intloop22 G G U A U A U C = 200
> intloop22 G G U A U C A C = 80
> intloop22 G G U A U C C C = 200
> intloop22 G G U A U C G C = 210
> intloop22 G G U A U C U C = 10
> intloop22 G G U A U G A C = 40
> intloop22 G G U A U G C C = 200
> intloop22 G G U A U G G C = 120
> intloop22 G G U A U G U C = -220
> intloop22 G G U A U U A C = 110
> intloop22 G G U A U U C C = 200
> intloop22 G G U A U U G C = 190
> intloop22 G G U A U U U C = 30
> intloop22 G U A A U A A C = 200
> intloop22 G U A A U A C C = 150
> intloop22 G U A A U A G C = 0
> intloop22 G U A A U A U C = 210
> intloop22 G U A A U C A C = 200
> intloop22 G U A A U C C C = 40
> intloop22 G U A A U C G C = -150
> intloop22 G U A A U C U C = 70
> intloop22 G U A A U G A C = 200
> intloop22 G U A A U G C C = 90
> intloop22 G U A A U G G C = -30
> intloop22 G U A A U G U C = 190
> intloop22 G U A A U U A C = 200
> intloop22 G U A A U U C C = 200
> intloop22 G U A A U U G C = 200
> intloop22 G U A A U U U C = 200
> intloop22 G U C A U A A C = 200
> intloop22 G U C A U A C C = 90
> intloop22 G U C A U A G C = -100
> intloop22 G U C A U A U C = 110
> intloop22 G U C A U C A C = 200
> intloop22 G U C A U C C C = 90
> intloop22 G U C A U C G C = 0
> intloop22 G U C A U C U C = 110
> intloop22 G U C A U G A C = 200
> intloop22 G U C A U G C C = 200
> intloop22 G U C A U G G C = 200
> intloop22 G U C A U G U C = 200
> intloop22 G U C A U U A C = 200
> intloop22 G U C A U U C C = 90
> intloop22 G U C A U U G C = 0
> intloop22 G U C A U U U C = 110
> intloop22 G U G A U A A C = 200
> intloop22 G U G A U A C C = 90
> intloop22 G U G A U A G C = -30
> intloop22 G U G A U A U C = 190
> intloop22 G U G A U C A C = 200
> intloop22 G U G A U C C C = 200
> intloop22 G U G A U C G C = 200
> intloop22 G U G A U C U C = 200
> intloop22 G U G A U G A C = 200
> intloop22 G U G A U G C C = 80
> intloop22 G U G A U G G C = -70
> intloop22 G U G A U G U C = 150
> intloop22 G U G A U U A C = 200
> intloop22 G U G A U U C C = -90
> intloop22 G U G A U U G C = -390
> intloop22 G U G A U U U C = 10
> intloop22 G U U A U A A C = 200
> intloop22 G U U A U A C C = 200
> intloop22 G U U A U A G C = 200
> intloop22 G U U A U A U C = 200
> intloop22 G U U A U C A C = 200
> intloop22 G U U A U C C C = 60
> intloop22 G U U A U C G C = -30
> intloop22 G U U A U C U C = 80
> intloop22 G U U A U G A C = 200
> intloop22 G U U A U G C C = 40
> intloop22 G U U A U G G C = -260
> intloop22 G U U A U G U C = 140
> intloop22 G U U A U U A C = 200
> intloop22 G U U A U U C C = 0
> intloop22 G U U A U U G C = -10
> intloop22 G U U A U U U C = 30
> intloop22 G A A G C A A U = 200
> intloop22 G A A G C A C U = 190
> intloop22 G A A G C A G U = 80
> intloop22 G A A G C A U U = 200
> intloop22 G A A G C C A U = 190
> intloop22 G A A G C C C U = 180
> intloop22 G A A G C C G U = 70
> intloop22 G A A G C C U U = 200
> intloop22 G A A G C G A U = 100
> intloop22 G A A G C G C U = 90
> intloop22 G A A G C G G U = -20
> intloop22 G A A G C G U U = 200
> intloop22 G A A G C U A U = 200
> intloop22 G A A G C U C U = 200
> intloop22 G A A G C U G U = 200
> intloop22 G A A G C U U U = 200
> intloop22 G A C G C A A U = 240
> intloop22 G A C G C A C U = 220
> intloop22 G A C G C A G U = 110
> intloop22 G A C G C A U U = 200
> intloop22 G A C G C C A U = 280
> intloop22 G A C G C C C U = 210
> intloop22 G A C G C C G U = 200
> intloop22 G A C G C C U U = 200
> intloop22 G A C G C G A U = 200
> intloop22 G A C G C G C U = 200
> intloop22 G A C G C G G U = 200
> intloop22 G A C G C G U U = 200
> intloop22 G A C G C U A U = 270
> intloop22 G A C G C U C U = 190
> intloop22 G A C G C U G U = 180
> intloop22 G A C G C U U U = 200
> intloop22 G A G G C A A U = 100
> intloop22 G A G G C A C U = 90
> intloop22 G A G G C A G U = -20
> intloop22 G A G G C A U U = 200
> intloop22 G A G G C C A U = 200
> intloop22 G A G G C C C U = 200
> intloop22 G A G G C C G U = 200
> intloop22 G A G G C C U U = 200
> intloop22 G A G G C G A U = 180
> intloop22 G A G G C G C U = 160
> intloop22 G A G G C G G U = 50
> intloop22 G A G G C G U U = 200
> intloop22 G A G G C U A U = 30
> intloop22 G A G G C U C U = -80
> intloop22 G A G G C U G U = -10
> intloop22 G A G G C U U U = 200
> intloop22 G A U G C A A U = 200
> intloop22 G A U G C A C U = 200
> intloop22 G A U G C A G U = 200
> intloop22 G A U G C A U U = 200
> intloop22 G A U G C C A U = 270
> intloop22 G A U G C C C U = 190
> intloop22 G A U G C C G U = 180
> intloop22 G A U G C C U U = 200
> intloop22 G A U G C G A U = 180
> intloop22 G A U G C G C U = 70
> intloop22 G A U G C G G U = 140
> intloop22 G A U G C G U U = 200
> intloop22 G A U G C U A U = 220
> intloop22 G A U G C U C U = 100
> intloop22 G A U G C U G U = 180
> intloop22 G A U G C U U U = 200
> intloop22 G C A G C A A U = 180
> intloop22 G C A G C A C U = 230
> intloop22 G C A G C A G U = 200
> intloop22 G C A G C A U U = 230
> intloop22 G C A G C C A U = 170
> intloop22 G C A G C C C U = 160
> intloop22 G C A G C C G U = 200
> intloop22 G C A G C C U U = 160
> intloop22 G C A G C G A U = 80
> intloop22 G C A G C G C U = 170
> intloop22 G C A G C G G U = 200
> intloop22 G C A G C G U U = 170
> intloop22 G C A G C U A U = 200
> intloop22 G C A G C U C U = 200
> intloop22 G C A G C U G U = 200
> intloop22 G C A G C U U U = 200
> intloop22 G C C G C A A U = 210
> intloop22 G C C G C A C U = 210
> intloop22 G C C G C A G U = 200
> intloop22 G C C G C A U U = 210
> intloop22 G C C G C C A U = 200
> intloop22 G C C G C C C U = 190
> intloop22 G C C G C C G U = 200
> intloop22 G C C G C C U U = 190
> intloop22 G C C G C G A U = 200
> intloop22 G C C G C G C U = 200
> intloop22 G C C G C G G U = 200
> intloop22 G C C G C G U U = 200
> intloop22 G C C G C U A U = 180
> intloop22 G C C G C U C U = 180
> intloop22 G C C G C U G U = 200
> intloop22 G C C G C U U U = 180
> intloop22 G C G G C A A U = 80
> intloop22 G C G G C A C U = 170
> intloop22 G C G G C A G U = 200
> intloop22 G C G G C A U U = 170
> intloop22 G C G G C C A U = 200
> intloop22 G C G G C C C U = 200
> intloop22 G C G G C C G U = 200
> intloop22 G C G G C C U U = 200
> intloop22 G C G G C G A U = 150
> intloop22 G C G G C G C U = 210
> intloop22 G C G G C G G U = 200
> intloop22 G C G G C G U U = 210
> intloop22 G C G G C U A U = -90
> intloop22 G C G G C U C U = 0
> intloop22 G C G G C U G U = 200
> intloop22 G C G G C U U U = 0
> intloop22 G C U G C A A U = 200
> intloop22 G C U G C A C U = 200
> intloop22 G C U G C A G U = 200
> intloop22 G C U G C A U U = 200
> intloop22 G C U G C C A U = 180
> intloop22 G C U G C C C U = 180
> intloop22 G C U G C C G U = 200
> intloop22 G C U G C C U U = 180
> intloop22 G C U G C G A U = 60
> intloop22 G C U G C G C U = 150
> intloop22 G C U G C G G U = 200
> intloop22 G C U G C G U U = 150
> intloop22 G C U G C U A U = 90
> intloop22 G C U G C U C U = 90
> intloop22 G C U G C U G U = 200
> intloop22 G C U G C U U U = 90
> intloop22 G G A G C A A U = 80
> intloop22 G G A G C A C U = 200
> intloop22 G G A G C A G U = 130
> intloop22 G G A G C A U U = 160
> intloop22 G G A G C C A U = 70
> intloop22 G G A G C C C U = 200
> intloop22 G G A G C C G U = 120
> intloop22 G G A G C C U U = 50
> intloop22 G G A G C G A U = -20
> intloop22 G G A G C G C U = 200
> intloop22 G G A G C G G U = 30
> intloop22 G G A G C G U U = 140
> intloop22 G G A G C U A U = 200
> intloop22 G G A G C U C U = 200
> intloop22 G G A G C U G U = 200
> intloop22 G G A G C U U U = 200
> intloop22 G G C G C A A U = 110
> intloop22 G G C G C A C U = 200
> intloop22 G G C G C A G U = 170
> intloop22 G G C G C A U U = 90
> intloop22 G G C G C C A U = 200
> intloop22 G G C G C C C U = 200
> intloop22 G G C G C C G U = 210
> intloop22 G G C G C C U U = 180
> intloop22 G G C G C G A U = 200
> intloop22 G G C G C G C U = 200
> intloop22 G G C G C G G U = 200
> intloop22 G G C G C G U U = 200
> intloop22 G G C G C U A U = 180
> intloop22 G G C G C U C U = 200
> intloop22 G G C G C U G U = 200
> intloop22 G G C G C U U U = 160
> intloop22 G G G G C A A U = -20
> intloop22 G G G G C A C U = 200
> intloop22 G G G G C A G U = 30
> intloop22 G G G G C A U U = 140
> intloop22 G G G G C C A U = 200
> intloop22 G G G G C C C U = 200
> intloop22 G G G G C C G U = 200
> intloop22 G G G G C C U U = 200
> intloop22 G G G G C G A U = 50
> intloop22 G G G G C G C U = 200
> intloop22 G G G G C G G U = 110
> intloop22 G G G G C G U U = 130
> intloop22 G G G G C U A U = -10
> intloop22 G G G G C U C U = 200
> intloop22 G G G G C U G U = -40
> intloop22 G G G G C U U U = -210
> intloop22 G G U G C A A U = 200
> intloop22 G G U G C A C U = 200
> intloop22 G G U G C A G U = 200
> intloop22 G G U G C A U U = 200
> intloop22 G G U G C C A U = 180
> intloop22 G G U G C C C U = 200
> intloop22 G G U G C C G U = 200
> intloop22 G G U G C C U U = 160
> intloop22 G G U G C G A U = 140
> intloop22 G G U G C G C U = 200
> intloop22 G G U G C G G U = 110
> intloop22 G G U G C G U U = -60
> intloop22 G G U G C U A U = 180
> intloop22 G G U G C U C U = 200
> intloop22 G G U G C U G U = 150
> intloop22 G G U G C U U U = 160
> intloop22 G U A G C A A U = 200
> intloop22 G U A G C A C U = 230
> intloop22 G U A G C A G U = 60
> intloop22 G U A G C A U U = 190
> intloop22 G U A G C C A U = 200
> intloop22 G U A G C C C U = 160
> intloop22 G U A G C C G U = -50
> intloop22 G U A G C C U U = 80
> intloop22 G U A G C G A U = 200
> intloop22 G U A G C G C U = 170
> intloop22 G U A G C G G U = 40
> intloop22 G U A G C G U U = 180
> intloop22 G U A G C U A U = 200
> intloop22 G U A G C U C U = 200
> intloop22 G U A G C U G U = 200
> intloop22 G U A G C U U U = 200
> intloop22 G U C G C A A U = 200
> intloop22 G U C G C A C U = 210
> intloop22 G U C G C A G U = 0
> intloop22 G U C G C A U U = 130
> intloop22 G U C G C C A U = 200
> intloop22 G U C G C C C U = 190
> intloop22 G U C G C C G U = 80
> intloop22 G U C G C C U U = 110
> intloop22 G U C G C G A U = 200
> intloop22 G U C G C G C U = 200
> intloop22 G U C G C G G U = 200
> intloop22 G U C G C G U U = 200
> intloop22 G U C G C U A U = 200
> intloop22 G U C G C U C U = 180
> intloop22 G U C G C U G U = 70
> intloop22 G U C G C U U U = 100
> intloop22 G U G G C A A U = 200
> intloop22 G U G G C A C U = 170
> intloop22 G U G G C A G U = 40
> intloop22 G U G G C A U U = 180
> intloop22 G U G G C C A U = 200
> intloop22 G U G G C C C U = 200
> intloop22 G U G G C C G U = 200
> intloop22 G U G G C C U U = 200
> intloop22 G U G G C G A U = 200
> intloop22 G U G G C G C U = 210
> intloop22 G U G G C G G U = 40
> intloop22 G U G G C G U U = 170
> intloop22 G U G G C U A U = 200
> intloop22 G U G G C U C U = 0
> intloop22 G U G G C U G U = -310
> intloop22 G U G G C U U U = 0
> intloop22 G U U G C A A U = 200
> intloop22 G U U G C A C U = 200
> intloop22 G U U G C A G U = 200
> intloop22 G U U G C A U U = 200
> intloop22 G U U G C C A U = 200
> intloop22 G U U G C C C U = 180
> intloop22 G U U G C C G U = 70
> intloop22 G U U G C C U U = 100
> intloop22 G U U G C G A U = 200
> intloop22 G U U G C G C U = 150
> intloop22 G U U G C G G U = -160
> intloop22 G U U G C G U U = 160
> intloop22 G U U G C U A U = 200
> intloop22 G U U G C U C U = 90
> intloop22 G U U G C U G U = 60
> intloop22 G U U G C U U U = 10
> intloop22 G A A C G A A U = 210
> intloop22 G A A C G A C U = 200
> intloop22 G A A C G A G U = 90
> intloop22 G A A C G A U U = 200
> intloop22 G A A C G C A U = 190
> intloop22 G A A C G C C U = 170
> intloop22 G A A C G C G U = 60
> intloop22 G A A C G C U U = 200
> intloop22 G A A C G G A U = 10
> intloop22 G A A C G G C U = 0
> intloop22 G A A C G G G U = -110
> intloop22 G A A C G G U U = 200
> intloop22 G A A C G U A U = 200
> intloop22 G A A C G U C U = 200
> intloop22 G A A C G U G U = 200
> intloop22 G A A C G U U U = 200
> intloop22 G A C C G A A U = 180
> intloop22 G A C C G A C U = 170
> intloop22 G A C C G A G U = 60
> intloop22 G A C C G A U U = 200
> intloop22 G A C C G C A U = 250
> intloop22 G A C C G C C U = 170
> intloop22 G A C C G C G U = 160
> intloop22 G A C C G C U U = 200
> intloop22 G A C C G G A U = 200
> intloop22 G A C C G G C U = 200
> intloop22 G A C C G G G U = 200
> intloop22 G A C C G G U U = 200
> intloop22 G A C C G U A U = 150
> intloop22 G A C C G U C U = 70
> intloop22 G A C C G U G U = 70
> intloop22 G A C C G U U U = 200
> intloop22 G A G C G A A U = 70
> intloop22 G A G C G A C U = 60
> intloop22 G A G C G A G U = -50
> intloop22 G A G C G A U U = 200
> intloop22 G A G C G C A U = 200
> intloop22 G A G C G C C U = 200
> intloop22 G A G C G C G U = 200
> intloop22 G A G C G C U U = 200
> intloop22 G A G C G G A U = 180
> intloop22 G A G C G G C U = 160
> intloop22 G A G C G G G U = 50
> intloop22 G A G C G G U U = 200
> intloop22 G A G C G U A U = 0
> intloop22 G A G C G U C U = -120
> intloop22 G A G C G U G U = -50
> intloop22 G A G C G U U U = 200
> intloop22 G A U C G A A U = 200
> intloop22 G A U C G A C U = 200
> intloop22 G A U C G A G U = 200
> intloop22 G A U C G A U U = 200
> intloop22 G A U C G C A U = 250
> intloop22 G A U C G C C U = 180
> intloop22 G A U C G C G U = 170
> intloop22 G A U C G C U U = 200
> intloop22 G A U C G G A U = 40
> intloop22 G A U C G G C U = -80
> intloop22 G A U C G G G U = -10
> intloop22 G A U C G G U U = 200
> intloop22 G A U C G U A U = 210
> intloop22 G A U C G U C U = 100
> intloop22 G A U C G U G U = 170
> intloop22 G A U C G U U U = 200
> intloop22 G C A C G A A U = 190
> intloop22 G C A C G A C U = 240
> intloop22 G C A C G A G U = 200
> intloop22 G C A C G A U U = 240
> intloop22 G C A C G C A U = 160
> intloop22 G C A C G C C U = 160
> intloop22 G C A C G C G U = 200
> intloop22 G C A C G C U U = 160
> intloop22 G C A C G G A U = -10
> intloop22 G C A C G G C U = 80
> intloop22 G C A C G G G U = 200
> intloop22 G C A C G G U U = 80
> intloop22 G C A C G U A U = 200
> intloop22 G C A C G U C U = 200
> intloop22 G C A C G U G U = 200
> intloop22 G C A C G U U U = 200
> intloop22 G C C C G A A U = 160
> intloop22 G C C C G A C U = 150
> intloop22 G C C C G A G U = 200
> intloop22 G C C C G A U U = 150
> intloop22 G C C C G C A U = 160
> intloop22 G C C C G C C U = 160
> intloop22 G C C C G C G U = 200
> intloop22 G C C C G C U U = 160
> intloop22 G C C C G G A U = 200
> intloop22 G C C C G G C U = 200
> intloop22 G C C C G G G U = 200
> intloop22 G C C C G G U U = 200
> intloop22 G C C C G U A U = 60
> intloop22 G C C C G U C U = 60
> intloop22 G C C C G U G U = 200
> intloop22 G C C C G U U U = 60
> intloop22 G C G C G A A U = 50
> intloop22 G C G C G A C U = 140
> intloop22 G C G C G A G U = 200
> intloop22 G C G C G A U U = 140
> intloop22 G C G C G C A U = 200
> intloop22 G C G C G C C U = 200
> intloop22 G C G C G C G U = 200
> intloop22 G C G C G C U U = 200
> intloop22 G C G C G G A U = 150
> intloop22 G C G C G G C U = 210
> intloop22 G C G C G G G U = 200
> intloop22 G C G C G G U U = 210
> intloop22 G C G C G U A U = -130
> intloop22 G C G C G U C U = -30
> intloop22 G C G C G U G U = 200
> intloop22 G C G C G U U U = -30
> intloop22 G C U C G A A U = 200
> intloop22 G C U C G A C U = 200
> intloop22 G C U C G A G U = 200
> intloop22 G C U C G A U U = 200
> intloop22 G C U C G C A U = 170
> intloop22 G C U C G C C U = 160
> intloop22 G C U C G C G U = 200
> intloop22 G C U C G C U U = 160
> intloop22 G C U C G G A U = -90
> intloop22 G C U C G G C U = 10
> intloop22 G C U C G G G U = 200
> intloop22 G C U C G G U U = 10
> intloop22 G C U C G U A U = 90
> intloop22 G C U C G U C U = 80
> intloop22 G C U C G U G U = 200
> intloop22 G C U C G U U U = 80
> intloop22 G G A C G A A U = 90
> intloop22 G G A C G A C U = 200
> intloop22 G G A C G A G U = 140
> intloop22 G G A C G A U U = 170
> intloop22 G G A C G C A U = 60
> intloop22 G G A C G C C U = 200
> intloop22 G G A C G C G U = 120
> intloop22 G G A C G C U U = 40
> intloop22 G G A C G G A U = -110
> intloop22 G G A C G G C U = 200
> intloop22 G G A C G G G U = -60
> intloop22 G G A C G G U U = 50
> intloop22 G G A C G U A U = 200
> intloop22 G G A C G U C U = 200
> intloop22 G G A C G U G U = 200
> intloop22 G G A C G U U U = 200
> intloop22 G G C C G A A U = 60
> intloop22 G G C C G A C U = 200
> intloop22 G G C C G A G U = 110
> intloop22 G G C C G A U U = 40
> intloop22 G G C C G C A U = 160
> intloop22 G G C C G C C U = 200
> intloop22 G G C C G C G U = 180
> intloop22 G G C C G C U U = 140
> intloop22 G G C C G G A U = 200
> intloop22 G G C C G G C U = 200
> intloop22 G G C C G G G U = 200
> intloop22 G G C C G G U U = 200
> intloop22 G G C C G U A U = 70
> intloop22 G G C C G U C U = 200
> intloop22 G G C C G U G U = 80
> intloop22 G G C C G U U U = 50
> intloop22 G G G C G A A U = -50
> intloop22 G G G C G A C U = 200
> intloop22 G G G C G A G U = 0
> intloop22 G G G C G A U U = 110
> intloop22 G G G C G C A U = 200
> intloop22 G G G C G C C U = 200
> intloop22 G G G C G C G U = 200
> intloop22 G G G C G C U U = 200
> intloop22 G G G C G G A U = 50
> intloop22 G G G C G G C U = 200
> intloop22 G G G C G G G U = 110
> intloop22 G G G C G G U U = 130
> intloop22 G G G C G U A U = -50
> intloop22 G G G C G U C U = 200
> intloop22 G G G C G U G U = -70
> intloop22 G G G C G U U U = -250
> intloop22 G G U C G A A U = 200
> intloop22 G G U C G A C U = 200
> intloop22 G G U C G A G U = 200
> intloop22 G G U C G A U U = 200
> intloop22 G G U C G C A U = 170
> intloop22 G G U C G C C U = 200
> intloop22 G G U C G C G U = 180
> intloop22 G G U C G C U U = 150
> intloop22 G G U C G G A U = -10
> intloop22 G G U C G G C U = 200
> intloop22 G G U C G G G U = -30
> intloop22 G G U C G G U U = -210
> intloop22 G G U C G U A U = 170
> intloop22 G G U C G U C U = 200
> intloop22 G G U C G U G U = 140
> intloop22 G G U C G U U U = 150
> intloop22 G U A C G A A U = 200
> intloop22 G U A C G A C U = 240
> intloop22 G U A C G A G U = 70
> intloop22 G U A C G A U U = 200
> intloop22 G U A C G C A U = 200
> intloop22 G U A C G C C U = 160
> intloop22 G U A C G C G U = -50
> intloop22 G U A C G C U U = 80
> intloop22 G U A C G G A U = 200
> intloop22 G U A C G G C U = 80
> intloop22 G U A C G G G U = -50
> intloop22 G U A C G G U U = 80
> intloop22 G U A C G U A U = 200
> intloop22 G U A C G U C U = 200
> intloop22 G U A C G U G U = 200
> intloop22 G U A C G U U U = 200
> intloop22 G U C C G A A U = 200
> intloop22 G U C C G A C U = 150
> intloop22 G U C C G A G U = -60
> intloop22 G U C C G A U U = 70
> intloop22 G U C C G C A U = 200
> intloop22 G U C C G C C U = 160
> intloop22 G U C C G C G U = 50
> intloop22 G U C C G C U U = 80
> intloop22 G U C C G G A U = 200
> intloop22 G U C C G G C U = 200
> intloop22 G U C C G G G U = 200
> intloop22 G U C C G G U U = 200
> intloop22 G U C C G U A U = 200
> intloop22 G U C C G U C U = 60
> intloop22 G U C C G U G U = -50
> intloop22 G U C C G U U U = -20
> intloop22 G U G C G A A U = 200
> intloop22 G U G C G A C U = 140
> intloop22 G U G C G A G U = 10
> intloop22 G U G C G A U U = 150
> intloop22 G U G C G C A U = 200
> intloop22 G U G C G C C U = 200
> intloop22 G U G C G C G U = 200
> intloop22 G U G C G C U U = 200
> intloop22 G U G C G G A U = 200
> intloop22 G U G C G G C U = 210
> intloop22 G U G C G G G U = 40
> intloop22 G U G C G G U U = 170
> intloop22 G U G C G U A U = 200
> intloop22 G U G C G U C U = -30
> intloop22 G U G C G U G U = -350
> intloop22 G U G C G U U U = -30
> intloop22 G U U C G A A U = 200
> intloop22 G U U C G A C U = 200
> intloop22 G U U C G A G U = 200
> intloop22 G U U C G A U U = 200
> intloop22 G U U C G C A U = 200
> intloop22 G U U C G C C U = 160
> intloop22 G U U C G C G U = 50
> intloop22 G U U C G C U U = 80
> intloop22 G U U C G G A U = 200
> intloop22 G U U C G G C U = 10
> intloop22 G U U C G G G U = -310
> intloop22 G U U C G G U U = 10
> intloop22 G U U C G U A U = 200
> intloop22 G U U C G U C U = 80
> intloop22 G U U C G U G U = 50
> intloop22 G U U C G U U U = 0
> intloop22 G A A U G A A U = 280
> intloop22 G A A U G A C U = 260
> intloop22 G A A U G A G U = 150
> intloop22 G A A U G A U U = 200
> intloop22 G A A U G C A U = 250
> intloop22 G A A U G C C U = 240
> intloop22 G A A U G C G U = 130
> intloop22 G A A U G C U U = 200
> intloop22 G A A U G G A U = 150
> intloop22 G A A U G G C U = 140
> intloop22 G A A U G G G U = 30
> intloop22 G A A U G G U U = 200
> intloop22 G A A U G U A U = 200
> intloop22 G A A U G U C U = 200
> intloop22 G A A U G U G U = 200
> intloop22 G A A U G U U U = 200
> intloop22 G A C U G A A U = 260
> intloop22 G A C U G A C U = 250
> intloop22 G A C U G A G U = 140
> intloop22 G A C U G A U U = 200
> intloop22 G A C U G C A U = 310
> intloop22 G A C U G C C U = 230
> intloop22 G A C U G C G U = 220
> intloop22 G A C U G C U U = 200
> intloop22 G A C U G G A U = 200
> intloop22 G A C U G G C U = 200
> intloop22 G A C U G G G U = 200
> intloop22 G A C U G G U U = 200
> intloop22 G A C U G U A U = 310
> intloop22 G A C U G U C U = 230
> intloop22 G A C U G U G U = 220
> intloop22 G A C U G U U U = 200
> intloop22 G A G U G A A U = 150
> intloop22 G A G U G A C U = 140
> intloop22 G A G U G A G U = 30
> intloop22 G A G U G A U U = 200
> intloop22 G A G U G C A U = 200
> intloop22 G A G U G C C U = 200
> intloop22 G A G U G C G U = 200
> intloop22 G A G U G C U U = 200
> intloop22 G A G U G G A U = 210
> intloop22 G A G U G G C U = 190
> intloop22 G A G U G G G U = 80
> intloop22 G A G U G G U U = 200
> intloop22 G A G U G U A U = 130
> intloop22 G A G U G U C U = 20
> intloop22 G A G U G U G U = 90
> intloop22 G A G U G U U U = 200
> intloop22 G A U U G A A U = 200
> intloop22 G A U U G A C U = 200
> intloop22 G A U U G A G U = 200
> intloop22 G A U U G A U U = 200
> intloop22 G A U U G C A U = 310
> intloop22 G A U U G C C U = 230
> intloop22 G A U U G C G U = 220
> intloop22 G A U U G C U U = 200
> intloop22 G A U U G G A U = 230
> intloop22 G A U U G G C U = 120
> intloop22 G A U U G G G U = 190
> intloop22 G A U U G G U U = 200
> intloop22 G A U U G U A U = 270
> intloop22 G A U U G U C U = 150
> intloop22 G A U U G U G U = 220
> intloop22 G A U U G U U U = 200
> intloop22 G C A U G A A U = 250
> intloop22 G C A U G A C U = 310
> intloop22 G C A U G A G U = 200
> intloop22 G C A U G A U U = 310
> intloop22 G C A U G C A U = 230
> intloop22 G C A U G C C U = 220
> intloop22 G C A U G C G U = 200
> intloop22 G C A U G C U U = 220
> intloop22 G C A U G G A U = 130
> intloop22 G C A U G G C U = 220
> intloop22 G C A U G G G U = 200
> intloop22 G C A U G G U U = 220
> intloop22 G C A U G U A U = 200
> intloop22 G C A U G U C U = 200
> intloop22 G C A U G U G U = 200
> intloop22 G C A U G U U U = 200
> intloop22 G C C U G A A U = 240
> intloop22 G C C U G A C U = 230
> intloop22 G C C U G A G U = 200
> intloop22 G C C U G A U U = 230
> intloop22 G C C U G C A U = 220
> intloop22 G C C U G C C U = 220
> intloop22 G C C U G C G U = 200
> intloop22 G C C U G C U U = 220
> intloop22 G C C U G G A U = 200
> intloop22 G C C U G G C U = 200
> intloop22 G C C U G G G U = 200
> intloop22 G C C U G G U U = 200
> intloop22 G C C U G U A U = 220
> intloop22 G C C U G U C U = 220
> intloop22 G C C U G U G U = 200
> intloop22 G C C U G U U U = 220
> intloop22 G C G U G A A U = 130
> intloop22 G C G U G A C U = 220
> intloop22 G C G U G A G U = 200
> intloop22 G C G U G A U U = 220
> intloop22 G C G U G C A U = 200
> intloop22 G C G U G C C U = 200
> intloop22 G C G U G C G U = 200
> intloop22 G C G U G C U U = 200
> intloop22 G C G U G G A U = 180
> intloop22 G C G U G G C U = 240
> intloop22 G C G U G G G U = 200
> intloop22 G C G U G G U U = 240
> intloop22 G C G U G U A U = 10
> intloop22 G C G U G U C U = 100
> intloop22 G C G U G U G U = 200
> intloop22 G C G U G U U U = 100
> intloop22 G C U U G A A U = 200
> intloop22 G C U U G A C U = 200
> intloop22 G C U U G A G U = 200
> intloop22 G C U U G A U U = 200
> intloop22 G C U U G C A U = 220
> intloop22 G C U U G C C U = 220
> intloop22 G C U U G C G U = 200
> intloop22 G C U U G C U U = 220
> intloop22 G C U U G G A U = 110
> intloop22 G C U U G G C U = 200
> intloop22 G C U U G G G U = 200
> intloop22 G C U U G G U U = 200
> intloop22 G C U U G U A U = 140
> intloop22 G C U U G U C U = 140
> intloop22 G C U U G U G U = 200
> intloop22 G C U U G U U U = 140
> intloop22 G G A U G A A U = 150
> intloop22 G G A U G A C U = 200
> intloop22 G G A U G A G U = 210
> intloop22 G G A U G A U U = 230
> intloop22 G G A U G C A U = 130
> intloop22 G G A U G C C U = 200
> intloop22 G G A U G C G U = 180
> intloop22 G G A U G C U U = 110
> intloop22 G G A U G G A U = 30
> intloop22 G G A U G G C U = 200
> intloop22 G G A U G G G U = 80
> intloop22 G G A U G G U U = 190
> intloop22 G G A U G U A U = 200
> intloop22 G G A U G U C U = 200
> intloop22 G G A U G U G U = 200
> intloop22 G G A U G U U U = 200
> intloop22 G G C U G A A U = 140
> intloop22 G G C U G A C U = 200
> intloop22 G G C U G A G U = 190
> intloop22 G G C U G A U U = 120
> intloop22 G G C U G C A U = 220
> intloop22 G G C U G C C U = 200
> intloop22 G G C U G C G U = 240
> intloop22 G G C U G C U U = 200
> intloop22 G G C U G G A U = 200
> intloop22 G G C U G G C U = 200
> intloop22 G G C U G G G U = 200
> intloop22 G G C U G G U U = 200
> intloop22 G G C U G U A U = 220
> intloop22 G G C U G U C U = 200
> intloop22 G G C U G U G U = 240
> intloop22 G G C U G U U U = 200
> intloop22 G G G U G A A U = 30
> intloop22 G G G U G A C U = 200
> intloop22 G G G U G A G U = 80
> intloop22 G G G U G A U U = 190
> intloop22 G G G U G C A U = 200
> intloop22 G G G U G C C U = 200
> intloop22 G G G U G C G U = 200
> intloop22 G G G U G C U U = 200
> intloop22 G G G U G G A U = 80
> intloop22 G G G U G G C U = 200
> intloop22 G G G U G G G U = 140
> intloop22 G G G U G G U U = 160
> intloop22 G G G U G U A U = 90
> intloop22 G G G U G U C U = 200
> intloop22 G G G U G U G U = 70
> intloop22 G G G U G U U U = -110
> intloop22 G G U U G A A U = 200
> intloop22 G G U U G A C U = 200
> intloop22 G G U U G A G U = 200
> intloop22 G G U U G A U U = 200
> intloop22 G G U U G C A U = 220
> intloop22 G G U U G C C U = 200
> intloop22 G G U U G C G U = 240
> intloop22 G G U U G C U U = 200
> intloop22 G G U U G G A U = 190
> intloop22 G G U U G G C U = 200
> intloop22 G G U U G G G U = 160
> intloop22 G G U U G G U U = -10
> intloop22 G G U U G U A U = 220
> intloop22 G G U U G U C U = 200
> intloop22 G G U U G U G U = 200
> intloop22 G G U U G U U U = 200
> intloop22 G U A U G A A U = 200
> intloop22 G U A U G A C U = 310
> intloop22 G U A U G A G U = 130
> intloop22 G U A U G A U U = 270
> intloop22 G U A U G C A U = 200
> intloop22 G U A U G C C U = 220
> intloop22 G U A U G C G U = 10
> intloop22 G U A U G C U U = 140
> intloop22 G U A U G G A U = 200
> intloop22 G U A U G G C U = 220
> intloop22 G U A U G G G U = 90
> intloop22 G U A U G G U U = 220
> intloop22 G U A U G U A U = 200
> intloop22 G U A U G U C U = 200
> intloop22 G U A U G U G U = 200
> intloop22 G U A U G U U U = 200
> intloop22 G U C U G A A U = 200
> intloop22 G U C U G A C U = 230
> intloop22 G U C U G A G U = 20
> intloop22 G U C U G A U U = 150
> intloop22 G U C U G C A U = 200
> intloop22 G U C U G C C U = 220
> intloop22 G U C U G C G U = 100
> intloop22 G U C U G C U U = 140
> intloop22 G U C U G G A U = 200
> intloop22 G U C U G G C U = 200
> intloop22 G U C U G G G U = 200
> intloop22 G U C U G G U U = 200
> intloop22 G U C U G U A U = 200
> intloop22 G U C U G U C U = 220
> intloop22 G U C U G U G U = 100
> intloop22 G U C U G U U U = 140
> intloop22 G U G U G A A U = 200
> intloop22 G U G U G A C U = 220
> intloop22 G U G U G A G U = 90
> intloop22 G U G U G A U U = 220
> intloop22 G U G U G C A U = 200
> intloop22 G U G U G C C U = 200
> intloop22 G U G U G C G U = 200
> intloop22 G U G U G C U U = 200
> intloop22 G U G U G G A U = 200
> intloop22 G U G U G G C U = 240
> intloop22 G U G U G G G U = 70
> intloop22 G U G U G G U U = 200
> intloop22 G U G U G U A U = 200
> intloop22 G U G U G U C U = 100
> intloop22 G U G U G U G U = -210
> intloop22 G U G U G U U U = 110
> intloop22 G U U U G A A U = 200
> intloop22 G U U U G A C U = 200
> intloop22 G U U U G A G U = 200
> intloop22 G U U U G A U U = 200
> intloop22 G U U U G C A U = 200
> intloop22 G U U U G C C U = 220
> intloop22 G U U U G C G U = 100
> intloop22 G U U U G C U U = 140
> intloop22 G U U U G G A U = 200
> intloop22 G U U U G G C U = 200
> intloop22 G U U U G G G U = -110
> intloop22 G U U U G G U U = 200
> intloop22 G U U U G U A U = 200
> intloop22 G U U U G U C U = 140
> intloop22 G U U U G U G U = 110
> intloop22 G U U U G U U U = 60
> intloop22 G A A G U A A U = 280
> intloop22 G A A G U A C U = 260
> intloop22 G A A G U A G U = 150
> intloop22 G A A G U A U U = 200
> intloop22 G A A G U C A U = 230
> intloop22 G A A G U C C U = 220
> intloop22 G A A G U C G U = 110
> intloop22 G A A G U C U U = 200
> intloop22 G A A G U G A U = 170
> intloop22 G A A G U G C U = 160
> intloop22 G A A G U G G U = 50
> intloop22 G A A G U G U U = 200
> intloop22 G A A G U U A U = 200
> intloop22 G A A G U U C U = 200
> intloop22 G A A G U U G U = 200
> intloop22 G A A G U U U U = 200
> intloop22 G A C G U A A U = 280
> intloop22 G A C G U A C U = 260
> intloop22 G A C G U A G U = 150
> intloop22 G A C G U A U U = 200
> intloop22 G A C G U C A U = 340
> intloop22 G A C G U C C U = 260
> intloop22 G A C G U C G U = 250
> intloop22 G A C G U C U U = 200
> intloop22 G A C G U G A U = 200
> intloop22 G A C G U G C U = 200
> intloop22 G A C G U G G U = 200
> intloop22 G A C G U G U U = 200
> intloop22 G A C G U U A U = 340
> intloop22 G A C G U U C U = 260
> intloop22 G A C G U U G U = 250
> intloop22 G A C G U U U U = 200
> intloop22 G A G G U A A U = 170
> intloop22 G A G G U A C U = 160
> intloop22 G A G G U A G U = 50
> intloop22 G A G G U A U U = 200
> intloop22 G A G G U C A U = 200
> intloop22 G A G G U C C U = 200
> intloop22 G A G G U C G U = 200
> intloop22 G A G G U C U U = 200
> intloop22 G A G G U G A U = 210
> intloop22 G A G G U G C U = 200
> intloop22 G A G G U G G U = 90
> intloop22 G A G G U G U U = 200
> intloop22 G A G G U U A U = 100
> intloop22 G A G G U U C U = -20
> intloop22 G A G G U U G U = 50
> intloop22 G A G G U U U U = 200
> intloop22 G A U G U A A U = 200
> intloop22 G A U G U A C U = 200
> intloop22 G A U G U A G U = 200
> intloop22 G A U G U A U U = 200
> intloop22 G A U G U C A U = 310
> intloop22 G A U G U C C U = 230
> intloop22 G A U G U C G U = 220
> intloop22 G A U G U C U U = 200
> intloop22 G A U G U G A U = 220
> intloop22 G A U G U G C U = 110
> intloop22 G A U G U G G U = 180
> intloop22 G A U G U G U U = 200
> intloop22 G A U G U U A U = 290
> intloop22 G A U G U U C U = 180
> intloop22 G A U G U U G U = 250
> intloop22 G A U G U U U U = 200
> intloop22 G C A G U A A U = 250
> intloop22 G C A G U A C U = 310
> intloop22 G C A G U A G U = 200
> intloop22 G C A G U A U U = 310
> intloop22 G C A G U C A U = 210
> intloop22 G C A G U C C U = 200
> intloop22 G C A G U C G U = 200
> intloop22 G C A G U C U U = 200
> intloop22 G C A G U G A U = 150
> intloop22 G C A G U G C U = 240
> intloop22 G C A G U G G U = 200
> intloop22 G C A G U G U U = 240
> intloop22 G C A G U U A U = 200
> intloop22 G C A G U U C U = 200
> intloop22 G C A G U U G U = 200
> intloop22 G C A G U U U U = 200
> intloop22 G C C G U A A U = 250
> intloop22 G C C G U A C U = 250
> intloop22 G C C G U A G U = 200
> intloop22 G C C G U A U U = 250
> intloop22 G C C G U C A U = 250
> intloop22 G C C G U C C U = 250
> intloop22 G C C G U C G U = 200
> intloop22 G C C G U C U U = 250
> intloop22 G C C G U G A U = 200
> intloop22 G C C G U G C U = 200
> intloop22 G C C G U G G U = 200
> intloop22 G C C G U G U U = 200
> intloop22 G C C G U U A U = 250
> intloop22 G C C G U U C U = 250
> intloop22 G C C G U U G U = 200
> intloop22 G C C G U U U U = 250
> intloop22 G C G G U A A U = 150
> intloop22 G C G G U A C U = 240
> intloop22 G C G G U A G U = 200
> intloop22 G C G G U A U U = 240
> intloop22 G C G G U C A U = 200
> intloop22 G C G G U C C U = 200
> intloop22 G C G G U C G U = 200
> intloop22 G C G G U C U U = 200
> intloop22 G C G G U G A U = 190
> intloop22 G C G G U G C U = 240
> intloop22 G C G G U G G U = 200
> intloop22 G C G G U G U U = 240
> intloop22 G C G G U U A U = -30
> intloop22 G C G G U U C U = 70
> intloop22 G C G G U U G U = 200
> intloop22 G C G G U U U U = 70
> intloop22 G C U G U A A U = 200
> intloop22 G C U G U A C U = 200
> intloop22 G C U G U A G U = 200
> intloop22 G C U G U A U U = 200
> intloop22 G C U G U C A U = 220
> intloop22 G C U G U C C U = 220
> intloop22 G C U G U C G U = 200
> intloop22 G C U G U C U U = 220
> intloop22 G C U G U G A U = 100
> intloop22 G C U G U G C U = 190
> intloop22 G C U G U G G U = 200
> intloop22 G C U G U G U U = 190
> intloop22 G C U G U U A U = 170
> intloop22 G C U G U U C U = 160
> intloop22 G C U G U U G U = 200
> intloop22 G C U G U U U U = 160
> intloop22 G G A G U A A U = 150
> intloop22 G G A G U A C U = 200
> intloop22 G G A G U A G U = 210
> intloop22 G G A G U A U U = 230
> intloop22 G G A G U C A U = 110
> intloop22 G G A G U C C U = 200
> intloop22 G G A G U C G U = 160
> intloop22 G G A G U C U U = 90
> intloop22 G G A G U G A U = 50
> intloop22 G G A G U G C U = 200
> intloop22 G G A G U G G U = 100
> intloop22 G G A G U G U U = 210
> intloop22 G G A G U U A U = 200
> intloop22 G G A G U U C U = 200
> intloop22 G G A G U U G U = 200
> intloop22 G G A G U U U U = 200
> intloop22 G G C G U A A U = 150
> intloop22 G G C G U A C U = 200
> intloop22 G G C G U A G U = 210
> intloop22 G G C G U A U U = 130
> intloop22 G G C G U C A U = 250
> intloop22 G G C G U C C U = 200
> intloop22 G G C G U C G U = 270
> intloop22 G G C G U C U U = 230
> intloop22 G G C G U G A U = 200
> intloop22 G G C G U G C U = 200
> intloop22 G G C G U G G U = 200
> intloop22 G G C G U G U U = 200
> intloop22 G G C G U U A U = 250
> intloop22 G G C G U U C U = 200
> intloop22 G G C G U U G U = 270
> intloop22 G G C G U U U U = 230
> intloop22 G G G G U A A U = 50
> intloop22 G G G G U A C U = 200
> intloop22 G G G G U A G U = 100
> intloop22 G G G G U A U U = 210
> intloop22 G G G G U C A U = 200
> intloop22 G G G G U C C U = 200
> intloop22 G G G G U C G U = 200
> intloop22 G G G G U C U U = 200
> intloop22 G G G G U G A U = 90
> intloop22 G G G G U G C U = 200
> intloop22 G G G G U G G U = 140
> intloop22 G G G G U G U U = 170
> intloop22 G G G G U U A U = 50
> intloop22 G G G G U U C U = 200
> intloop22 G G G G U U G U = 30
> intloop22 G G G G U U U U = -150
> intloop22 G G U G U A A U = 200
> intloop22 G G U G U A C U = 200
> intloop22 G G U G U A G U = 200
> intloop22 G G U G U A U U = 200
> intloop22 G G U G U C A U = 220
> intloop22 G G U G U C C U = 200
> intloop22 G G U G U C G U = 240
> intloop22 G G U G U C U U = 200
> intloop22 G G U G U G A U = 180
> intloop22 G G U G U G C U = 200
> intloop22 G G U G U G G U = 150
> intloop22 G G U G U G U U = -20
> intloop22 G G U G U U A U = 250
> intloop22 G G U G U U C U = 200
> intloop22 G G U G U U G U = 220
> intloop22 G G U G U U U U = 230
> intloop22 G U A G U A A U = 200
> intloop22 G U A G U A C U = 310
> intloop22 G U A G U A G U = 130
> intloop22 G U A G U A U U = 270
> intloop22 G U A G U C A U = 200
> intloop22 G U A G U C C U = 200
> intloop22 G U A G U C G U = -10
> intloop22 G U A G U C U U = 120
> intloop22 G U A G U G A U = 200
> intloop22 G U A G U G C U = 240
> intloop22 G U A G U G G U = 110
> intloop22 G U A G U G U U = 240
> intloop22 G U A G U U A U = 200
> intloop22 G U A G U U C U = 200
> intloop22 G U A G U U G U = 200
> intloop22 G U A G U U U U = 200
> intloop22 G U C G U A A U = 200
> intloop22 G U C G U A C U = 250
> intloop22 G U C G U A G U = 30
> intloop22 G U C G U A U U = 170
> intloop22 G U C G U C A U = 200
> intloop22 G U C G U C C U = 250
> intloop22 G U C G U C G U = 130
> intloop22 G U C G U C U U = 170
> intloop22 G U C G U G A U = 200
> intloop22 G U C G U G C U = 200
> intloop22 G U C G U G G U = 200
> intloop22 G U C G U G U U = 200
> intloop22 G U C G U U A U = 200
> intloop22 G U C G U U C U = 250
> intloop22 G U C G U U G U = 130
> intloop22 G U C G U U U U = 170
> intloop22 G U G G U A A U = 200
> intloop22 G U G G U A C U = 240
> intloop22 G U G G U A G U = 110
> intloop22 G U G G U A U U = 240
> intloop22 G U G G U C A U = 200
> intloop22 G U G G U C C U = 200
> intloop22 G U G G U C G U = 200
> intloop22 G U G G U C U U = 200
> intloop22 G U G G U G A U = 200
> intloop22 G U G G U G C U = 240
> intloop22 G U G G U G G U = 70
> intloop22 G U G G U G U U = 200
> intloop22 G U G G U U A U = 200
> intloop22 G U G G U U C U = 70
> intloop22 G U G G U U G U = -250
> intloop22 G U G G U U U U = 70
> intloop22 G U U G U A A U = 200
> intloop22 G U U G U A C U = 200
> intloop22 G U U G U A G U = 200
> intloop22 G U U G U A U U = 200
> intloop22 G U U G U C A U = 200
> intloop22 G U U G U C C U = 220
> intloop22 G U U G U C G U = 100
> intloop22 G U U G U C U U = 140
> intloop22 G U U G U G A U = 200
> intloop22 G U U G U G C U = 190
> intloop22 G U U G U G G U = -120
> intloop22 G U U G U G U U = 190
> intloop22 G U U G U U A U = 200
> intloop22 G U U G U U C U = 160
> intloop22 G U U G U U G U = 130
> intloop22 G U U G U U U U = 80
> intloop22 G A A U A A A U = 280
> intloop22 G A A U A A C U = 260
> intloop22 G A A U A A G U = 150
> intloop22 G A A U A A U U = 200
> intloop22 G A A U A C A U = 250
> intloop22 G A A U A C C U = 240
> intloop22 G A A U A C G U = 130
> intloop22 G A A U A C U U = 200
> intloop22 G A A U A G A U = 150
> intloop22 G A A U A G C U = 140
> intloop22 G A A U A G G U = 30
> intloop22 G A A U A G U U = 200
> intloop22 G A A U A U A U = 200
> intloop22 G A A U A U C U = 200
> intloop22 G A A U A U G U = 200
> intloop22 G A A U A U U U = 200
> intloop22 G A C U A A A U = 260
> intloop22 G A C U A A C U = 250
> intloop22 G A C U A A G U = 140
> intloop22 G A C U A A U U = 200
> intloop22 G A C U A C A U = 310
> intloop22 G A C U A C C U = 230
> intloop22 G A C U A C G U = 220
> intloop22 G A C U A C U U = 200
> intloop22 G A C U A G A U = 200
> intloop22 G A C U A G C U = 200
> intloop22 G A C U A G G U = 200
> intloop22 G A C U A G U U = 200
> intloop22 G A C U A U A U = 310
> intloop22 G A C U A U C U = 230
> intloop22 G A C U A U G U = 220
> intloop22 G A C U A U U U = 200
> intloop22 G A G U A A A U = 150
> intloop22 G A G U A A C U = 140
> intloop22 G A G U A A G U = 30
> intloop22 G A G U A A U U = 200
> intloop22 G A G U A C A U = 200
> intloop22 G A G U A C C U = 200
> intloop22 G A G U A C G U = 200
> intloop22 G A G U A C U U = 200
> intloop22 G A G U A G A U = 210
> intloop22 G A G U A G C U = 190
> intloop22 G A G U A G G U = 80
> intloop22 G A G U A G U U = 200
> intloop22 G A G U A U A U = 130
> intloop22 G A G U A U C U = 20
> intloop22 G A G U A U G U = 90
> intloop22 G A G U A U U U = 200
> intloop22 G A U U A A A U = 200
> intloop22 G A U U A A C U = 200
> intloop22 G A U U A A G U = 200
> intloop22 G A U U A A U U = 200
> intloop22 G A U U A C A U = 310
> intloop22 G A U U A C C U = 230
> intloop22 G A U U A C G U = 220
> intloop22 G A U U A C U U = 200
> intloop22 G A U U A G A U = 230
> intloop22 G A U U A G C U = 120
> intloop22 G A U U A G G U = 190
> intloop22 G A U U A G U U = 200
> intloop22 G A U U A U A U = 270
> intloop22 G A U U A U C U = 150
> intloop22 G A U U A U G U = 220
> intloop22 G A U U A U U U = 200
> intloop22 G C A U A A A U = 250
> intloop22 G C A U A A C U = 310
> intloop22 G C A U A A G U = 200
> intloop22 G C A U A A U U = 310
> intloop22 G C A U A C A U = 230
> intloop22 G C A U A C C U = 220
> intloop22 G C A U A C G U = 200
> intloop22 G C A U A C U U = 220
> intloop22 G C A U A G A U = 130
> intloop22 G C A U A G C U = 220
> intloop22 G C A U A G G U = 200
> intloop22 G C A U A G U U = 220
> intloop22 G C A U A U A U = 200
> intloop22 G C A U A U C U = 200
> intloop22 G C A U A U G U = 200
> intloop22 G C A U A U U U = 200
> intloop22 G C C U A A A U = 240
> intloop22 G C C U A A C U = 230
> intloop22 G C C U A A G U = 200
> intloop22 G C C U A A U U = 230
> intloop22 G C C U A C A U = 220
> intloop22 G C C U A C C U = 220
> intloop22 G C C U A C G U = 200
> intloop22 G C C U A C U U = 220
> intloop22 G C C U A G A U = 200
> intloop22 G C C U A G C U = 200
> intloop22 G C C U A G G U = 200
> intloop22 G C C U A G U U = 200
> intloop22 G C C U A U A U = 220
> intloop22 G C C U A U C U = 220
> intloop22 G C C U A U G U = 200
> intloop22 G C C U A U U U = 220
> intloop22 G C G U A A A U = 130
> intloop22 G C G U A A C U = 220
> intloop22 G C G U A A G U = 200
> intloop22 G C G U A A U U = 220
> intloop22 G C G U A C A U = 200
> intloop22 G C G U A C C U = 200
> intloop22 G C G U A C G U = 200
> intloop22 G C G U A C U U = 200
> intloop22 G C G U A G A U = 180
> intloop22 G C G U A G C U = 240
> intloop22 G C G U A G G U = 200
> intloop22 G C G U A G U U = 240
> intloop22 G C G U A U A U = 10
> intloop22 G C G U A U C U = 100
> intloop22 G C G U A U G U = 200
> intloop22 G C G U A U U U = 100
> intloop22 G C U U A A A U = 200
> intloop22 G C U U A A C U = 200
> intloop22 G C U U A A G U = 200
> intloop22 G C U U A A U U = 200
> intloop22 G C U U A C A U = 220
> intloop22 G C U U A C C U = 220
> intloop22 G C U U A C G U = 200
> intloop22 G C U U A C U U = 220
> intloop22 G C U U A G A U = 110
> intloop22 G C U U A G C U = 200
> intloop22 G C U U A G G U = 200
> intloop22 G C U U A G U U = 200
> intloop22 G C U U A U A U = 140
> intloop22 G C U U A U C U = 140
> intloop22 G C U U A U G U = 200
> intloop22 G C U U A U U U = 140
> intloop22 G G A U A A A U = 150
> intloop22 G G A U A A C U = 200
> intloop22 G G A U A A G U = 210
> intloop22 G G A U A A U U = 230
> intloop22 G G A U A C A U = 130
> intloop22 G G A U A C C U = 200
> intloop22 G G A U A C G U = 180
> intloop22 G G A U A C U U = 110
> intloop22 G G A U A G A U = 30
> intloop22 G G A U A G C U = 200
> intloop22 G G A U A G G U = 80
> intloop22 G G A U A G U U = 190
> intloop22 G G A U A U A U = 200
> intloop22 G G A U A U C U = 200
> intloop22 G G A U A U G U = 200
> intloop22 G G A U A U U U = 200
> intloop22 G G C U A A A U = 140
> intloop22 G G C U A A C U = 200
> intloop22 G G C U A A G U = 190
> intloop22 G G C U A A U U = 120
> intloop22 G G C U A C A U = 220
> intloop22 G G C U A C C U = 200
> intloop22 G G C U A C G U = 240
> intloop22 G G C U A C U U = 200
> intloop22 G G C U A G A U = 200
> intloop22 G G C U A G C U = 200
> intloop22 G G C U A G G U = 200
> intloop22 G G C U A G U U = 200
> intloop22 G G C U A U A U = 220
> intloop22 G G C U A U C U = 200
> intloop22 G G C U A U G U = 240
> intloop22 G G C U A U U U = 200
> intloop22 G G G U A A A U = 30
> intloop22 G G G U A A C U = 200
> intloop22 G G G U A A G U = 80
> intloop22 G G G U A A U U = 190
> intloop22 G G G U A C A U = 200
> intloop22 G G G U A C C U = 200
> intloop22 G G G U A C G U = 200
> intloop22 G G G U A C U U = 200
> intloop22 G G G U A G A U = 80
> intloop22 G G G U A G C U = 200
> intloop22 G G G U A G G U = 140
> intloop22 G G G U A G U U = 160
> intloop22 G G G U A U A U = 90
> intloop22 G G G U A U C U = 200
> intloop22 G G G U A U G U = 70
> intloop22 G G G U A U U U = -110
> intloop22 G G U U A A A U = 200
> intloop22 G G U U A A C U = 200
> intloop22 G G U U A A G U = 200
> intloop22 G G U U A A U U = 200
> intloop22 G G U U A C A U = 220
> intloop22 G G U U A C C U = 200
> intloop22 G G U U A C G U = 240
> intloop22 G G U U A C U U = 200
> intloop22 G G U U A G A U = 190
> intloop22 G G U U A G C U = 200
> intloop22 G G U U A G G U = 160
> intloop22 G G U U A G U U = -10
> intloop22 G G U U A U A U = 220
> intloop22 G G U U A U C U = 200
> intloop22 G G U U A U G U = 200
> intloop22 G G U U A U U U = 200
> intloop22 G U A U A A A U = 200
> intloop22 G U A U A A C U = 310
> intloop22 G U A U A A G U = 130
> intloop22 G U A U A A U U = 270
> intloop22 G U A U A C A U = 200
> intloop22 G U A U A C C U = 220
> intloop22 G U A U A C G U = 10
> intloop22 G U A U A C U U = 140
> intloop22 G U A U A G A U = 200
> intloop22 G U A U A G C U = 220
> intloop22 G U A U A G G U = 90
> intloop22 G U A U A G U U = 220
> intloop22 G U A U A U A U = 200
> intloop22 G U A U A U C U = 200
> intloop22 G U A U A U G U = 200
> intloop22 G U A U A U U U = 200
> intloop22 G U C U A A A U = 200
> intloop22 G U C U A A C U = 230
> intloop22 G U C U A A G U = 20
> intloop22 G U C U A A U U = 150
> intloop22 G U C U A C A U = 200
> intloop22 G U C U A C C U = 220
> intloop22 G U C U A C G U = 100
> intloop22 G U C U A C U U = 140
> intloop22 G U C U A G A U = 200
> intloop22 G U C U A G C U = 200
> intloop22 G U C U A G G U = 200
> intloop22 G U C U A G U U = 200
> intloop22 G U C U A U A U = 200
> intloop22 G U C U A U C U = 220
> intloop22 G U C U A U G U = 100
> intloop22 G U C U A U U U = 140
> intloop22 G U G U A A A U = 200
> intloop22 G U G U A A C U = 220
> intloop22 G U G U A A G U = 90
> intloop22 G U G U A A U U = 220
> intloop22 G U G U A C A U = 200
> intloop22 G U G U A C C U = 200
> intloop22 G U G U A C G U = 200
> intloop22 G U G U A C U U = 200
> intloop22 G U G U A G A U = 200
> intloop22 G U G U A G C U = 240
> intloop22 G U G U A G G U = 70
> intloop22 G U G U A G U U = 200
> intloop22 G U G U A U A U = 200
> intloop22 G U G U A U C U = 100
> intloop22 G U G U A U G U = -210
> intloop22 G U G U A U U U = 110
> intloop22 G U U U A A A U = 200
> intloop22 G U U U A A C U = 200
> intloop22 G U U U A A G U = 200
> intloop22 G U U U A A U U = 200
> intloop22 G U U U A C A U = 200
> intloop22 G U U U A C C U = 220
> intloop22 G U U U A C G U = 100
> intloop22 G U U U A C U U = 140
> intloop22 G U U U A G A U = 200
> intloop22 G U U U A G C U = 200
> intloop22 G U U U A G G U = -110
> intloop22 G U U U A G U U = 200
> intloop22 G U U U A U A U = 200
> intloop22 G U U U A U C U = 140
> intloop22 G U U U A U G U = 110
> intloop22 G U U U A U U U = 60
> intloop22 G A A A U A A U = 280
> intloop22 G A A A U A C U = 260
> intloop22 G A A A U A G U = 150
> intloop22 G A A A U A U U = 200
> intloop22 G A A A U C A U = 230
> intloop22 G A A A U C C U = 220
> intloop22 G A A A U C G U = 110
> intloop22 G A A A U C U U = 200
> intloop22 G A A A U G A U = 170
> intloop22 G A A A U G C U = 160
> intloop22 G A A A U G G U = 50
> intloop22 G A A A U G U U = 200
> intloop22 G A A A U U A U = 200
> intloop22 G A A A U U C U = 200
> intloop22 G A A A U U G U = 200
> intloop22 G A A A U U U U = 200
> intloop22 G A C A U A A U = 280
> intloop22 G A C A U A C U = 260
> intloop22 G A C A U A G U = 150
> intloop22 G A C A U A U U = 200
> intloop22 G A C A U C A U = 340
> intloop22 G A C A U C C U = 260
> intloop22 G A C A U C G U = 250
> intloop22 G A C A U C U U = 200
> intloop22 G A C A U G A U = 200
> intloop22 G A C A U G C U = 200
> intloop22 G A C A U G G U = 200
> intloop22 G A C A U G U U = 200
> intloop22 G A C A U U A U = 340
> intloop22 G A C A U U C U = 260
> intloop22 G A C A U U G U = 250
> intloop22 G A C A U U U U = 200
> intloop22 G A G A U A A U = 170
> intloop22 G A G A U A C U = 160
> intloop22 G A G A U A G U = 50
> intloop22 G A G A U A U U = 200
> intloop22 G A G A U C A U = 200
> intloop22 G A G A U C C U = 200
> intloop22 G A G A U C G U = 200
> intloop22 G A G A U C U U = 200
> intloop22 G A G A U G A U = 210
> intloop22 G A G A U G C U = 200
> intloop22 G A G A U G G U = 90
> intloop22 G A G A U G U U = 200
> intloop22 G A G A U U A U = 100
> intloop22 G A G A U U C U = -20
> intloop22 G A G A U U G U = 50
> intloop22 G A G A U U U U = 200
> intloop22 G A U A U A A U = 200
> intloop22 G A U A U A C U = 200
> intloop22 G A U A U A G U = 200
> intloop22 G A U A U A U U = 200
> intloop22 G A U A U C A U = 310
> intloop22 G A U A U C C U = 230
> intloop22 G A U A U C G U = 220
> intloop22 G A U A U C U U = 200
> intloop22 G A U A U G A U = 220
> intloop22 G A U A U G C U = 110
> intloop22 G A U A U G G U = 180
> intloop22 G A U A U G U U = 200
> intloop22 G A U A U U A U = 290
> intloop22 G A U A U U C U = 180
> intloop22 G A U A U U G U = 250
> intloop22 G A U A U U U U = 200
> intloop22 G C A A U A A U = 250
> intloop22 G C A A U A C U = 310
> intloop22 G C A A U A G U = 200
> intloop22 G C A A U A U U = 310
> intloop22 G C A A U C A U = 210
> intloop22 G C A A U C C U = 200
> intloop22 G C A A U C G U = 200
> intloop22 G C A A U C U U = 200
> intloop22 G C A A U G A U = 150
> intloop22 G C A A U G C U = 240
> intloop22 G C A A U G G U = 200
> intloop22 G C A A U G U U = 240
> intloop22 G C A A U U A U = 200
> intloop22 G C A A U U C U = 200
> intloop22 G C A A U U G U = 200
> intloop22 G C A A U U U U = 200
> intloop22 G C C A U A A U = 250
> intloop22 G C C A U A C U = 250
> intloop22 G C C A U A G U = 200
> intloop22 G C C A U A U U = 250
> intloop22 G C C A U C A U = 250
> intloop22 G C C A U C C U = 250
> intloop22 G C C A U C G U = 200
> intloop22 G C C A U C U U = 250
> intloop22 G C C A U G A U = 200
> intloop22 G C C A U G C U = 200
> intloop22 G C C A U G G U = 200
> intloop22 G C C A U G U U = 200
> intloop22 G C C A U U A U = 250
> intloop22 G C C A U U C U = 250
> intloop22 G C C A U U G U = 200
> intloop22 G C C A U U U U = 250
> intloop22 G C G A U A A U = 150
> intloop22 G C G A U A C U = 240
> intloop22 G C G A U A G U = 200
> intloop22 G C G A U A U U = 240
> intloop22 G C G A U C A U = 200
> intloop22 G C G A U C C U = 200
> intloop22 G C G A U C G U = 200
> intloop22 G C G A U C U U = 200
> intloop22 G C G A U G A U = 190
> intloop22 G C G A U G C U = 240
> intloop22 G C G A U G G U = 200
> intloop22 G C G A U G U U = 240
> intloop22 G C G A U U A U = -30
> intloop22 G C G A U U C U = 70
> intloop22 G C G A U U G U = 200
> intloop22 G C G A U U U U = 70
> intloop22 G C U A U A A U = 200
> intloop22 G C U A U A C U = 200
> intloop22 G C U A U A G U = 200
> intloop22 G C U A U A U U = 200
> intloop22 G C U A U C A U = 220
> intloop22 G C U A U C C U = 220
> intloop22 G C U A U C G U = 200
> intloop22 G C U A U C U U = 220
> intloop22 G C U A U G A U = 100
> intloop22 G C U A U G C U = 190
> intloop22 G C U A U G G U = 200
> intloop22 G C U A U G U U = 190
> intloop22 G C U A U U A U = 170
> intloop22 G C U A U U C U = 160
> intloop22 G C U A U U G U = 200
> intloop22 G C U A U U U U = 160
> intloop22 G G A A U A A U = 150
> intloop22 G G A A U A C U = 200
> intloop22 G G A A U A G U = 210
> intloop22 G G A A U A U U = 230
> intloop22 G G A A U C A U = 110
> intloop22 G G A A U C C U = 200
> intloop22 G G A A U C G U = 160
> intloop22 G G A A U C U U = 90
> intloop22 G G A A U G A U = 50
> intloop22 G G A A U G C U = 200
> intloop22 G G A A U G G U = 100
> intloop22 G G A A U G U U = 210
> intloop22 G G A A U U A U = 200
> intloop22 G G A A U U C U = 200
> intloop22 G G A A U U G U = 200
> intloop22 G G A A U U U U = 200
> intloop22 G G C A U A A U = 150
> intloop22 G G C A U A C U = 200
> intloop22 G G C A U A G U = 210
> intloop22 G G C A U A U U = 130
> intloop22 G G C A U C A U = 250
> intloop22 G G C A U C C U = 200
> intloop22 G G C A U C G U = 270
> intloop22 G G C A U C U U = 230
> intloop22 G G C A U G A U = 200
> intloop22 G G C A U G C U = 200
> intloop22 G G C A U G G U = 200
> intloop22 G G C A U G U U = 200
> intloop22 G G C A U U A U = 250
> intloop22 G G C A U U C U = 200
> intloop22 G G C A U U G U = 270
> intloop22 G G C A U U U U = 230
> intloop22 G G G A U A A U = 50
> intloop22 G G G A U A C U = 200
> intloop22 G G G A U A G U = 100
> intloop22 G G G A U A U U = 210
> intloop22 G G G A U C A U = 200
> intloop22 G G G A U C C U = 200
> intloop22 G G G A U C G U = 200
> intloop22 G G G A U C U U = 200
> intloop22 G G G A U G A U = 90
> intloop22 G G G A U G C U = 200
> intloop22 G G G A U G G U = 140
> intloop22 G G G A U G U U = 170
> intloop22 G G G A U U A U = 50
> intloop22 G G G A U U C U = 200
> intloop22 G G G A U U G U = 30
> intloop22 G G G A U U U U = -150
> intloop22 G G U A U A A U = 200
> intloop22 G G U A U A C U = 200
> intloop22 G G U A U A G U = 200
> intloop22 G G U A U A U U = 200
> intloop22 G G U A U C A U = 220
> intloop22 G G U A U C C U = 200
> intloop22 G G U A U C G U = 240
> intloop22 G G U A U C U U = 200
> intloop22 G G U A U G A U = 180
> intloop22 G G U A U G C U = 200
> intloop22 G G U A U G G U = 150
> intloop22 G G U A U G U U = -20
> intloop22 G G U A U U A U = 250
> intloop22 G G U A U U C U = 200
> intloop22 G G U A U U G U = 220
> intloop22 G G U A U U U U = 230
> intloop22 G U A A U A A U = 200
> intloop22 G U A A U A C U = 310
> intloop22 G U A A U A G U = 130
> intloop22 G U A A U A U U = 270
> intloop22 G U A A U C A U = 200
> intloop22 G U A A U C C U = 200
> intloop22 G U A A U C G U = -10
> intloop22 G U A A U C U U = 120
> intloop22 G U A A U G A U = 200
> intloop22 G U A A U G C U = 240
> intloop22 G U A A U G G U = 110
> intloop22 G U A A U G U U = 240
> intloop22 G U A A U U A U = 200
> intloop22 G U A A U U C U = 200
> intloop22 G U A A U U G U = 200
> intloop22 G U A A U U U U = 200
> intloop22 G U C A U A A U = 200
> intloop22 G U C A U A C U = 250
> intloop22 G U C A U A G U = 30
> intloop22 G U C A U A U U = 170
> intloop22 G U C A U C A U = 200
> intloop22 G U C A U C C U = 250
> intloop22 G U C A U C G U = 130
> intloop22 G U C A U C U U = 170
> intloop22 G U C A U G A U = 200
> intloop22 G U C A U G C U = 200
> intloop22 G U C A U G G U = 200
> intloop22 G U C A U G U U = 200
> intloop22 G U C A U U A U = 200
> intloop22 G U C A U U C U = 250
> intloop22 G U C A U U G U = 130
> intloop22 G U C A U U U U = 170
> intloop22 G U G A U A A U = 200
> intloop22 G U G A U A C U = 240
> intloop22 G U G A U A G U = 110
> intloop22 G U G A U A U U = 240
> intloop22 G U G A U C A U = 200
> intloop22 G U G A U C C U = 200
> intloop22 G U G A U C G U = 200
> intloop22 G U G A U C U U = 200
> intloop22 G U G A U G A U = 200
> intloop22 G U G A U G C U = 240
> intloop22 G U G A U G G U = 70
> intloop22 G U G A U G U U = 200
> intloop22 G U G A U U A U = 200
> intloop22 G U G A U U C U = 70
> intloop22 G U G A U U G U = -250
> intloop22 G U G A U U U U = 70
> intloop22 G U U A U A A U = 200
> intloop22 G U U A U A C U = 200
> intloop22 G U U A U A G U = 200
> intloop22 G U U A U A U U = 200
> intloop22 G U U A U C A U = 200
> intloop22 G U U A U C C U = 220
> intloop22 G U U A U C G U = 100
> intloop22 G U U A U C U U = 140
> intloop22 G U U A U G A U = 200
> intloop22 G U U A U G C U = 190
> intloop22 G U U A U G G U = -120
> intloop22 G U U A U G U U = 190
> intloop22 G U U A U U A U = 200
> intloop22 G U U A U U C U = 160
> intloop22 G U U A U U G U = 130
> intloop22 G U U A U U U U = 80
> intloop22 U A A G C A A G = 200
> intloop22 U A A G C A C G = 200
> intloop22 U A A G C A G G = 100
> intloop22 U A A G C A U G = 200
> intloop22 U A A G C C A G = 190
> intloop22 U A A G C C C G = 190
> intloop22 U A A G C C G G = 90
> intloop22 U A A G C C U G = 200
> intloop22 U A A G C G A G = 100
> intloop22 U A A G C G C G = 100
> intloop22 U A A G C G G G = 0
> intloop22 U A A G C G U G = 200
> intloop22 U A A G C U A G = 200
> intloop22 U A A G C U C G = 200
> intloop22 U A A G C U G G = 200
> intloop22 U A A G C U U G = 200
> intloop22 U A C G C A A G = 240
> intloop22 U A C G C A C G = 240
> intloop22 U A C G C A G G = 130
> intloop22 U A C G C A U G = 200
> intloop22 U A C G C C A G = 280
> intloop22 U A C G C C C G = 220
> intloop22 U A C G C C G G = 220
> intloop22 U A C G C C U G = 200
> intloop22 U A C G C G A G = 200
> intloop22 U A C G C G C G = 200
> intloop22 U A C G C G G G = 200
> intloop22 U A C G C G U G = 200
> intloop22 U A C G C U A G = 270
> intloop22 U A C G C U C G = 210
> intloop22 U A C G C U G G = 200
> intloop22 U A C G C U U G = 200
> intloop22 U A G G C A A G = 100
> intloop22 U A G G C A C G = 100
> intloop22 U A G G C A G G = 0
> intloop22 U A G G C A U G = 200
> intloop22 U A G G C C A G = 200
> intloop22 U A G G C C C G = 200
> intloop22 U A G G C C G G = 200
> intloop22 U A G G C C U G = 200
> intloop22 U A G G C G A G = 180
> intloop22 U A G G C G C G = 180
> intloop22 U A G G C G G G = 70
> intloop22 U A G G C G U G = 200
> intloop22 U A G G C U A G = 30
> intloop22 U A G G C U C G = -70
> intloop22 U A G G C U G G = 10
> intloop22 U A G G C U U G = 200
> intloop22 U A U G C A A G = 200
> intloop22 U A U G C A C G = 200
> intloop22 U A U G C A G G = 200
> intloop22 U A U G C A U G = 200
> intloop22 U A U G C C A G = 270
> intloop22 U A U G C C C G = 210
> intloop22 U A U G C C G G = 200
> intloop22 U A U G C C U G = 200
> intloop22 U A U G C G A G = 180
> intloop22 U A U G C G C G = 80
> intloop22 U A U G C G G G = 160
> intloop22 U A U G C G U G = 200
> intloop22 U A U G C U A G = 220
> intloop22 U A U G C U C G = 120
> intloop22 U A U G C U G G = 190
> intloop22 U A U G C U U G = 200
> intloop22 U C A G C A A G = 160
> intloop22 U C A G C A C G = 260
> intloop22 U C A G C A G G = 200
> intloop22 U C A G C A U G = 230
> intloop22 U C A G C C A G = 150
> intloop22 U C A G C C C G = 190
> intloop22 U C A G C C G G = 200
> intloop22 U C A G C C U G = 160
> intloop22 U C A G C G A G = 60
> intloop22 U C A G C G C G = 200
> intloop22 U C A G C G G G = 200
> intloop22 U C A G C G U G = 170
> intloop22 U C A G C U A G = 200
> intloop22 U C A G C U C G = 200
> intloop22 U C A G C U G G = 200
> intloop22 U C A G C U U G = 200
> intloop22 U C C G C A A G = 190
> intloop22 U C C G C A C G = 240
> intloop22 U C C G C A G G = 200
> intloop22 U C C G C A U G = 210
> intloop22 U C C G C C A G = 180
> intloop22 U C C G C C C G = 220
> intloop22 U C C G C C G G = 200
> intloop22 U C C G C C U G = 190
> intloop22 U C C G C G A G = 200
> intloop22 U C C G C G C G = 200
> intloop22 U C C G C G G G = 200
> intloop22 U C C G C G U G = 200
> intloop22 U C C G C U A G = 160
> intloop22 U C C G C U C G = 210
> intloop22 U C C G C U G G = 200
> intloop22 U C C G C U U G = 180
> intloop22 U C G G C A A G = 60
> intloop22 U C G G C A C G = 200
> intloop22 U C G G C A G G = 200
> intloop22 U C G G C A U G = 170
> intloop22 U C G G C C A G = 200
> intloop22 U C G G C C C G = 200
> intloop22 U C G G C C G G = 200
> intloop22 U C G G C C U G = 200
> intloop22 U C G G C G A G = 130
> intloop22 U C G G C G C G = 240
> intloop22 U C G G C G G G = 200
> intloop22 U C G G C G U G = 210
> intloop22 U C G G C U A G = -110
> intloop22 U C G G C U C G = 30
> intloop22 U C G G C U G G = 200
> intloop22 U C G G C U U G = 0
> intloop22 U C U G C A A G = 200
> intloop22 U C U G C A C G = 200
> intloop22 U C U G C A G G = 200
> intloop22 U C U G C A U G = 200
> intloop22 U C U G C C A G = 160
> intloop22 U C U G C C C G = 210
> intloop22 U C U G C C G G = 200
> intloop22 U C U G C C U G = 180
> intloop22 U C U G C G A G = 40
> intloop22 U C U G C G C G = 180
> intloop22 U C U G C G G G = 200
> intloop22 U C U G C G U G = 150
> intloop22 U C U G C U A G = 70
> intloop22 U C U G C U C G = 120
> intloop22 U C U G C U G G = 200
> intloop22 U C U G C U U G = 90
> intloop22 U G A G C A A G = 100
> intloop22 U G A G C A C G = 200
> intloop22 U G A G C A G G = 140
> intloop22 U G A G C A U G = 150
> intloop22 U G A G C C A G = 90
> intloop22 U G A G C C C G = 200
> intloop22 U G A G C C G G = 130
> intloop22 U G A G C C U G = 40
> intloop22 U G A G C G A G = 0
> intloop22 U G A G C G C G = 200
> intloop22 U G A G C G G G = 40
> intloop22 U G A G C G U G = 130
> intloop22 U G A G C U A G = 200
> intloop22 U G A G C U C G = 200
> intloop22 U G A G C U G G = 200
> intloop22 U G A G C U U G = 200
> intloop22 U G C G C A A G = 130
> intloop22 U G C G C A C G = 200
> intloop22 U G C G C A G G = 170
> intloop22 U G C G C A U G = 80
> intloop22 U G C G C C A G = 220
> intloop22 U G C G C C C G = 200
> intloop22 U G C G C C G G = 220
> intloop22 U G C G C C U G = 170
> intloop22 U G C G C G A G = 200
> intloop22 U G C G C G C G = 200
> intloop22 U G C G C G G G = 200
> intloop22 U G C G C G U G = 200
> intloop22 U G C G C U A G = 200
> intloop22 U G C G C U C G = 200
> intloop22 U G C G C U G G = 200
> intloop22 U G C G C U U G = 150
> intloop22 U G G G C A A G = 0
> intloop22 U G G G C A C G = 200
> intloop22 U G G G C A G G = 40
> intloop22 U G G G C A U G = 130
> intloop22 U G G G C C A G = 200
> intloop22 U G G G C C C G = 200
> intloop22 U G G G C C G G = 200
> intloop22 U G G G C C U G = 200
> intloop22 U G G G C G A G = 70
> intloop22 U G G G C G C G = 200
> intloop22 U G G G C G G G = 110
> intloop22 U G G G C G U G = 120
> intloop22 U G G G C U A G = 10
> intloop22 U G G G C U C G = 200
> intloop22 U G G G C U G G = -30
> intloop22 U G G G C U U G = -220
> intloop22 U G U G C A A G = 200
> intloop22 U G U G C A C G = 200
> intloop22 U G U G C A G G = 200
> intloop22 U G U G C A U G = 200
> intloop22 U G U G C C A G = 200
> intloop22 U G U G C C C G = 200
> intloop22 U G U G C C G G = 200
> intloop22 U G U G C C U G = 150
> intloop22 U G U G C G A G = 160
> intloop22 U G U G C G C G = 200
> intloop22 U G U G C G G G = 120
> intloop22 U G U G C G U G = -70
> intloop22 U G U G C U A G = 190
> intloop22 U G U G C U C G = 200
> intloop22 U G U G C U G G = 150
> intloop22 U G U G C U U G = 150
> intloop22 U U A G C A A G = 200
> intloop22 U U A G C A C G = 260
> intloop22 U U A G C A G G = 20
> intloop22 U U A G C A U G = 220
> intloop22 U U A G C C A G = 200
> intloop22 U U A G C C C G = 190
> intloop22 U U A G C C G G = -90
> intloop22 U U A G C C U G = 110
> intloop22 U U A G C G A G = 200
> intloop22 U U A G C G C G = 200
> intloop22 U U A G C G G G = 0
> intloop22 U U A G C G U G = 200
> intloop22 U U A G C U A G = 200
> intloop22 U U A G C U C G = 200
> intloop22 U U A G C U G G = 200
> intloop22 U U A G C U U G = 200
> intloop22 U U C G C A A G = 200
> intloop22 U U C G C A C G = 240
> intloop22 U U C G C A G G = -40
> intloop22 U U C G C A U G = 150
> intloop22 U U C G C C A G = 200
> intloop22 U U C G C C C G = 220
> intloop22 U U C G C C G G = 40
> intloop22 U U C G C C U G = 140
> intloop22 U U C G C G A G = 200
> intloop22 U U C G C G C G = 200
> intloop22 U U C G C G G G = 200
> intloop22 U U C G C G U G = 200
> intloop22 U U C G C U A G = 200
> intloop22 U U C G C U C G = 210
> intloop22 U U C G C U G G = 30
> intloop22 U U C G C U U G = 120
> intloop22 U U G G C A A G = 200
> intloop22 U U G G C A C G = 200
> intloop22 U U G G C A G G = 0
> intloop22 U U G G C A U G = 200
> intloop22 U U G G C C A G = 200
> intloop22 U U G G C C C G = 200
> intloop22 U U G G C C G G = 200
> intloop22 U U G G C C U G = 200
> intloop22 U U G G C G A G = 200
> intloop22 U U G G C G C G = 240
> intloop22 U U G G C G G G = 0
> intloop22 U U G G C G U G = 190
> intloop22 U U G G C U A G = 200
> intloop22 U U G G C U C G = 30
> intloop22 U U G G C U G G = -350
> intloop22 U U G G C U U G = 30
> intloop22 U U U G C A A G = 200
> intloop22 U U U G C A C G = 200
> intloop22 U U U G C A G G = 200
> intloop22 U U U G C A U G = 200
> intloop22 U U U G C C A G = 200
> intloop22 U U U G C C C G = 210
> intloop22 U U U G C C G G = 30
> intloop22 U U U G C C U G = 120
> intloop22 U U U G C G A G = 200
> intloop22 U U U G C G C G = 180
> intloop22 U U U G C G G G = -200
> intloop22 U U U G C G U G = 180
> intloop22 U U U G C U A G = 200
> intloop22 U U U G C U C G = 120
> intloop22 U U U G C U G G = 20
> intloop22 U U U G C U U G = 30
> intloop22 U A A C G A A G = 210
> intloop22 U A A C G A C G = 210
> intloop22 U A A C G A G G = 110
> intloop22 U A A C G A U G = 200
> intloop22 U A A C G C A G = 190
> intloop22 U A A C G C C G = 190
> intloop22 U A A C G C G G = 80
> intloop22 U A A C G C U G = 200
> intloop22 U A A C G G A G = 10
> intloop22 U A A C G G C G = 10
> intloop22 U A A C G G G G = -90
> intloop22 U A A C G G U G = 200
> intloop22 U A A C G U A G = 200
> intloop22 U A A C G U C G = 200
> intloop22 U A A C G U G G = 200
> intloop22 U A A C G U U G = 200
> intloop22 U A C C G A A G = 180
> intloop22 U A C C G A C G = 180
> intloop22 U A C C G A G G = 80
> intloop22 U A C C G A U G = 200
> intloop22 U A C C G C A G = 250
> intloop22 U A C C G C C G = 190
> intloop22 U A C C G C G G = 180
> intloop22 U A C C G C U G = 200
> intloop22 U A C C G G A G = 200
> intloop22 U A C C G G C G = 200
> intloop22 U A C C G G G G = 200
> intloop22 U A C C G G U G = 200
> intloop22 U A C C G U A G = 150
> intloop22 U A C C G U C G = 90
> intloop22 U A C C G U G G = 90
> intloop22 U A C C G U U G = 200
> intloop22 U A G C G A A G = 70
> intloop22 U A G C G A C G = 70
> intloop22 U A G C G A G G = -30
> intloop22 U A G C G A U G = 200
> intloop22 U A G C G C A G = 200
> intloop22 U A G C G C C G = 200
> intloop22 U A G C G C G G = 200
> intloop22 U A G C G C U G = 200
> intloop22 U A G C G G A G = 180
> intloop22 U A G C G G C G = 180
> intloop22 U A G C G G G G = 70
> intloop22 U A G C G G U G = 200
> intloop22 U A G C G U A G = 0
> intloop22 U A G C G U C G = -100
> intloop22 U A G C G U G G = -30
> intloop22 U A G C G U U G = 200
> intloop22 U A U C G A A G = 200
> intloop22 U A U C G A C G = 200
> intloop22 U A U C G A G G = 200
> intloop22 U A U C G A U G = 200
> intloop22 U A U C G C A G = 250
> intloop22 U A U C G C C G = 190
> intloop22 U A U C G C G G = 190
> intloop22 U A U C G C U G = 200
> intloop22 U A U C G G A G = 40
> intloop22 U A U C G G C G = -60
> intloop22 U A U C G G G G = 10
> intloop22 U A U C G G U G = 200
> intloop22 U A U C G U A G = 210
> intloop22 U A U C G U C G = 110
> intloop22 U A U C G U G G = 190
> intloop22 U A U C G U U G = 200
> intloop22 U C A C G A A G = 170
> intloop22 U C A C G A C G = 270
> intloop22 U C A C G A G G = 200
> intloop22 U C A C G A U G = 240
> intloop22 U C A C G C A G = 140
> intloop22 U C A C G C C G = 190
> intloop22 U C A C G C G G = 200
> intloop22 U C A C G C U G = 160
> intloop22 U C A C G G A G = -30
> intloop22 U C A C G G C G = 110
> intloop22 U C A C G G G G = 200
> intloop22 U C A C G G U G = 80
> intloop22 U C A C G U A G = 200
> intloop22 U C A C G U C G = 200
> intloop22 U C A C G U G G = 200
> intloop22 U C A C G U U G = 200
> intloop22 U C C C G A A G = 140
> intloop22 U C C C G A C G = 180
> intloop22 U C C C G A G G = 200
> intloop22 U C C C G A U G = 150
> intloop22 U C C C G C A G = 140
> intloop22 U C C C G C C G = 190
> intloop22 U C C C G C G G = 200
> intloop22 U C C C G C U G = 160
> intloop22 U C C C G G A G = 200
> intloop22 U C C C G G C G = 200
> intloop22 U C C C G G G G = 200
> intloop22 U C C C G G U G = 200
> intloop22 U C C C G U A G = 40
> intloop22 U C C C G U C G = 90
> intloop22 U C C C G U G G = 200
> intloop22 U C C C G U U G = 60
> intloop22 U C G C G A A G = 30
> intloop22 U C G C G A C G = 170
> intloop22 U C G C G A G G = 200
> intloop22 U C G C G A U G = 140
> intloop22 U C G C G C A G = 200
> intloop22 U C G C G C C G = 200
> intloop22 U C G C G C G G = 200
> intloop22 U C G C G C U G = 200
> intloop22 U C G C G G A G = 130
> intloop22 U C G C G G C G = 240
> intloop22 U C G C G G G G = 200
> intloop22 U C G C G G U G = 210
> intloop22 U C G C G U A G = -150
> intloop22 U C G C G U C G = 0
> intloop22 U C G C G U G G = 200
> intloop22 U C G C G U U G = -30
> intloop22 U C U C G A A G = 200
> intloop22 U C U C G A C G = 200
> intloop22 U C U C G A G G = 200
> intloop22 U C U C G A U G = 200
> intloop22 U C U C G C A G = 150
> intloop22 U C U C G C C G = 190
> intloop22 U C U C G C G G = 200
> intloop22 U C U C G C U G = 160
> intloop22 U C U C G G A G = -110
> intloop22 U C U C G G C G = 40
> intloop22 U C U C G G G G = 200
> intloop22 U C U C G G U G = 10
> intloop22 U C U C G U A G = 70
> intloop22 U C U C G U C G = 110
> intloop22 U C U C G U G G = 200
> intloop22 U C U C G U U G = 80
> intloop22 U G A C G A A G = 110
> intloop22 U G A C G A C G = 200
> intloop22 U G A C G A G G = 150
> intloop22 U G A C G A U G = 160
> intloop22 U G A C G C A G = 80
> intloop22 U G A C G C C G = 200
> intloop22 U G A C G C G G = 120
> intloop22 U G A C G C U G = 30
> intloop22 U G A C G G A G = -90
> intloop22 U G A C G G C G = 200
> intloop22 U G A C G G G G = -50
> intloop22 U G A C G G U G = 40
> intloop22 U G A C G U A G = 200
> intloop22 U G A C G U C G = 200
> intloop22 U G A C G U G G = 200
> intloop22 U G A C G U U G = 200
> intloop22 U G C C G A A G = 80
> intloop22 U G C C G A C G = 200
> intloop22 U G C C G A G G = 120
> intloop22 U G C C G A U G = 30
> intloop22 U G C C G C A G = 180
> intloop22 U G C C G C C G = 200
> intloop22 U G C C G C G G = 180
> intloop22 U G C C G C U G = 130
> intloop22 U G C C G G A G = 200
> intloop22 U G C C G G C G = 200
> intloop22 U G C C G G G G = 200
> intloop22 U G C C G G U G = 200
> intloop22 U G C C G U A G = 90
> intloop22 U G C C G U C G = 200
> intloop22 U G C C G U G G = 80
> intloop22 U G C C G U U G = 40
> intloop22 U G G C G A A G = -30
> intloop22 U G G C G A C G = 200
> intloop22 U G G C G A G G = 10
> intloop22 U G G C G A U G = 100
> intloop22 U G G C G C A G = 200
> intloop22 U G G C G C C G = 200
> intloop22 U G G C G C G G = 200
> intloop22 U G G C G C U G = 200
> intloop22 U G G C G G A G = 70
> intloop22 U G G C G G C G = 200
> intloop22 U G G C G G G G = 110
> intloop22 U G G C G G U G = 120
> intloop22 U G G C G U A G = -30
> intloop22 U G G C G U C G = 200
> intloop22 U G G C G U G G = -70
> intloop22 U G G C G U U G = -260
> intloop22 U G U C G A A G = 200
> intloop22 U G U C G A C G = 200
> intloop22 U G U C G A G G = 200
> intloop22 U G U C G A U G = 200
> intloop22 U G U C G C A G = 190
> intloop22 U G U C G C C G = 200
> intloop22 U G U C G C G G = 190
> intloop22 U G U C G C U G = 140
> intloop22 U G U C G G A G = 10
> intloop22 U G U C G G C G = 200
> intloop22 U G U C G G G G = -30
> intloop22 U G U C G G U G = -220
> intloop22 U G U C G U A G = 190
> intloop22 U G U C G U C G = 200
> intloop22 U G U C G U G G = 150
> intloop22 U G U C G U U G = 140
> intloop22 U U A C G A A G = 200
> intloop22 U U A C G A C G = 270
> intloop22 U U A C G A G G = 30
> intloop22 U U A C G A U G = 230
> intloop22 U U A C G C A G = 200
> intloop22 U U A C G C C G = 190
> intloop22 U U A C G C G G = -90
> intloop22 U U A C G C U G = 100
> intloop22 U U A C G G A G = 200
> intloop22 U U A C G G C G = 110
> intloop22 U U A C G G G G = -90
> intloop22 U U A C G G U G = 110
> intloop22 U U A C G U A G = 200
> intloop22 U U A C G U C G = 200
> intloop22 U U A C G U G G = 200
> intloop22 U U A C G U U G = 200
> intloop22 U U C C G A A G = 200
> intloop22 U U C C G A C G = 180
> intloop22 U U C C G A G G = -100
> intloop22 U U C C G A U G = 100
> intloop22 U U C C G C A G = 200
> intloop22 U U C C G C C G = 190
> intloop22 U U C C G C G G = 10
> intloop22 U U C C G C U G = 100
> intloop22 U U C C G G A G = 200
> intloop22 U U C C G G C G = 200
> intloop22 U U C C G G G G = 200
> intloop22 U U C C G G U G = 200
> intloop22 U U C C G U A G = 200
> intloop22 U U C C G U C G = 90
> intloop22 U U C C G U G G = -90
> intloop22 U U C C G U U G = 0
> intloop22 U U G C G A A G = 200
> intloop22 U U G C G A C G = 170
> intloop22 U U G C G A G G = -30
> intloop22 U U G C G A U G = 170
> intloop22 U U G C G C A G = 200
> intloop22 U U G C G C C G = 200
> intloop22 U U G C G C G G = 200
> intloop22 U U G C G C U G = 200
> intloop22 U U G C G G A G = 200
> intloop22 U U G C G G C G = 240
> intloop22 U U G C G G G G = 0
> intloop22 U U G C G G U G = 190
> intloop22 U U G C G U A G = 200
> intloop22 U U G C G U C G = 0
> intloop22 U U G C G U G G = -390
> intloop22 U U G C G U U G = -10
> intloop22 U U U C G A A G = 200
> intloop22 U U U C G A C G = 200
> intloop22 U U U C G A G G = 200
> intloop22 U U U C G A U G = 200
> intloop22 U U U C G C A G = 200
> intloop22 U U U C G C C G = 190
> intloop22 U U U C G C G G = 10
> intloop22 U U U C G C U G = 110
> intloop22 U U U C G G A G = 200
> intloop22 U U U C G G C G = 40
> intloop22 U U U C G G G G = -350
> intloop22 U U U C G G U G = 30
> intloop22 U U U C G U A G = 200
> intloop22 U U U C G U C G = 110
> intloop22 U U U C G U G G = 10
> intloop22 U U U C G U U G = 30
> intloop22 U A A U G A A G = 280
> intloop22 U A A U G A C G = 280
> intloop22 U A A U G A G G = 170
> intloop22 U A A U G A U G = 200
> intloop22 U A A U G C A G = 250
> intloop22 U A A U G C C G = 250
> intloop22 U A A U G C G G = 150
> intloop22 U A A U G C U G = 200
> intloop22 U A A U G G A G = 150
> intloop22 U A A U G G C G = 150
> intloop22 U A A U G G G G = 50
> intloop22 U A A U G G U G = 200
> intloop22 U A A U G U A G = 200
> intloop22 U A A U G U C G = 200
> intloop22 U A A U G U G G = 200
> intloop22 U A A U G U U G = 200
> intloop22 U A C U G A A G = 260
> intloop22 U A C U G A C G = 260
> intloop22 U A C U G A G G = 160
> intloop22 U A C U G A U G = 200
> intloop22 U A C U G C A G = 310
> intloop22 U A C U G C C G = 250
> intloop22 U A C U G C G G = 240
> intloop22 U A C U G C U G = 200
> intloop22 U A C U G G A G = 200
> intloop22 U A C U G G C G = 200
> intloop22 U A C U G G G G = 200
> intloop22 U A C U G G U G = 200
> intloop22 U A C U G U A G = 310
> intloop22 U A C U G U C G = 250
> intloop22 U A C U G U G G = 240
> intloop22 U A C U G U U G = 200
> intloop22 U A G U G A A G = 150
> intloop22 U A G U G A C G = 150
> intloop22 U A G U G A G G = 50
> intloop22 U A G U G A U G = 200
> intloop22 U A G U G C A G = 200
> intloop22 U A G U G C C G = 200
> intloop22 U A G U G C G G = 200
> intloop22 U A G U G C U G = 200
> intloop22 U A G U G G A G = 210
> intloop22 U A G U G G C G = 210
> intloop22 U A G U G G G G = 100
> intloop22 U A G U G G U G = 200
> intloop22 U A G U G U A G = 130
> intloop22 U A G U G U C G = 30
> intloop22 U A G U G U G G = 110
> intloop22 U A G U G U U G = 200
> intloop22 U A U U G A A G = 200
> intloop22 U A U U G A C G = 200
> intloop22 U A U U G A G G = 200
> intloop22 U A U U G A U G = 200
> intloop22 U A U U G C A G = 310
> intloop22 U A U U G C C G = 250
> intloop22 U A U U G C G G = 240
> intloop22 U A U U G C U G = 200
> intloop22 U A U U G G A G = 230
> intloop22 U A U U G G C G = 130
> intloop22 U A U U G G G G = 210
> intloop22 U A U U G G U G = 200
> intloop22 U A U U G U A G = 270
> intloop22 U A U U G U C G = 170
> intloop22 U A U U G U G G = 240
> intloop22 U A U U G U U G = 200
> intloop22 U C A U G A A G = 230
> intloop22 U C A U G A C G = 340
> intloop22 U C A U G A G G = 200
> intloop22 U C A U G A U G = 310
> intloop22 U C A U G C A G = 210
> intloop22 U C A U G C C G = 250
> intloop22 U C A U G C G G = 200
> intloop22 U C A U G C U G = 220
> intloop22 U C A U G G A G = 110
> intloop22 U C A U G G C G = 250
> intloop22 U C A U G G G G = 200
> intloop22 U C A U G G U G = 220
> intloop22 U C A U G U A G = 200
> intloop22 U C A U G U C G = 200
> intloop22 U C A U G U G G = 200
> intloop22 U C A U G U U G = 200
> intloop22 U C C U G A A G = 220
> intloop22 U C C U G A C G = 260
> intloop22 U C C U G A G G = 200
> intloop22 U C C U G A U G = 230
> intloop22 U C C U G C A G = 200
> intloop22 U C C U G C C G = 250
> intloop22 U C C U G C G G = 200
> intloop22 U C C U G C U G = 220
> intloop22 U C C U G G A G = 200
> intloop22 U C C U G G C G = 200
> intloop22 U C C U G G G G = 200
> intloop22 U C C U G G U G = 200
> intloop22 U C C U G U A G = 200
> intloop22 U C C U G U C G = 250
> intloop22 U C C U G U G G = 200
> intloop22 U C C U G U U G = 220
> intloop22 U C G U G A A G = 110
> intloop22 U C G U G A C G = 250
> intloop22 U C G U G A G G = 200
> intloop22 U C G U G A U G = 220
> intloop22 U C G U G C A G = 200
> intloop22 U C G U G C C G = 200
> intloop22 U C G U G C G G = 200
> intloop22 U C G U G C U G = 200
> intloop22 U C G U G G A G = 160
> intloop22 U C G U G G C G = 270
> intloop22 U C G U G G G G = 200
> intloop22 U C G U G G U G = 240
> intloop22 U C G U G U A G = -10
> intloop22 U C G U G U C G = 130
> intloop22 U C G U G U G G = 200
> intloop22 U C G U G U U G = 100
> intloop22 U C U U G A A G = 200
> intloop22 U C U U G A C G = 200
> intloop22 U C U U G A G G = 200
> intloop22 U C U U G A U G = 200
> intloop22 U C U U G C A G = 200
> intloop22 U C U U G C C G = 250
> intloop22 U C U U G C G G = 200
> intloop22 U C U U G C U G = 220
> intloop22 U C U U G G A G = 90
> intloop22 U C U U G G C G = 230
> intloop22 U C U U G G G G = 200
> intloop22 U C U U G G U G = 200
> intloop22 U C U U G U A G = 120
> intloop22 U C U U G U C G = 170
> intloop22 U C U U G U G G = 200
> intloop22 U C U U G U U G = 140
> intloop22 U G A U G A A G = 170
> intloop22 U G A U G A C G = 200
> intloop22 U G A U G A G G = 210
> intloop22 U G A U G A U G = 220
> intloop22 U G A U G C A G = 150
> intloop22 U G A U G C C G = 200
> intloop22 U G A U G C G G = 190
> intloop22 U G A U G C U G = 100
> intloop22 U G A U G G A G = 50
> intloop22 U G A U G G C G = 200
> intloop22 U G A U G G G G = 90
> intloop22 U G A U G G U G = 180
> intloop22 U G A U G U A G = 200
> intloop22 U G A U G U C G = 200
> intloop22 U G A U G U G G = 200
> intloop22 U G A U G U U G = 200
> intloop22 U G C U G A A G = 160
> intloop22 U G C U G A C G = 200
> intloop22 U G C U G A G G = 200
> intloop22 U G C U G A U G = 110
> intloop22 U G C U G C A G = 240
> intloop22 U G C U G C C G = 200
> intloop22 U G C U G C G G = 240
> intloop22 U G C U G C U G = 190
> intloop22 U G C U G G A G = 200
> intloop22 U G C U G G C G = 200
> intloop22 U G C U G G G G = 200
> intloop22 U G C U G G U G = 200
> intloop22 U G C U G U A G = 240
> intloop22 U G C U G U C G = 200
> intloop22 U G C U G U G G = 240
> intloop22 U G C U G U U G = 190
> intloop22 U G G U G A A G = 50
> intloop22 U G G U G A C G = 200
> intloop22 U G G U G A G G = 90
> intloop22 U G G U G A U G = 180
> intloop22 U G G U G C A G = 200
> intloop22 U G G U G C C G = 200
> intloop22 U G G U G C G G = 200
> intloop22 U G G U G C U G = 200
> intloop22 U G G U G G A G = 100
> intloop22 U G G U G G C G = 200
> intloop22 U G G U G G G G = 140
> intloop22 U G G U G G U G = 150
> intloop22 U G G U G U A G = 110
> intloop22 U G G U G U C G = 200
> intloop22 U G G U G U G G = 70
> intloop22 U G G U G U U G = -120
> intloop22 U G U U G A A G = 200
> intloop22 U G U U G A C G = 200
> intloop22 U G U U G A G G = 200
> intloop22 U G U U G A U G = 200
> intloop22 U G U U G C A G = 240
> intloop22 U G U U G C C G = 200
> intloop22 U G U U G C G G = 240
> intloop22 U G U U G C U G = 190
> intloop22 U G U U G G A G = 210
> intloop22 U G U U G G C G = 200
> intloop22 U G U U G G G G = 170
> intloop22 U G U U G G U G = -20
> intloop22 U G U U G U A G = 240
> intloop22 U G U U G U C G = 200
> intloop22 U G U U G U G G = 200
> intloop22 U G U U G U U G = 190
> intloop22 U U A U G A A G = 200
> intloop22 U U A U G A C G = 340
> intloop22 U U A U G A G G = 100
> intloop22 U U A U G A U G = 290
> intloop22 U U A U G C A G = 200
> intloop22 U U A U G C C G = 250
> intloop22 U U A U G C G G = -30
> intloop22 U U A U G C U G = 170
> intloop22 U U A U G G A G = 200
> intloop22 U U A U G G C G = 250
> intloop22 U U A U G G G G = 50
> intloop22 U U A U G G U G = 250
> intloop22 U U A U G U A G = 200
> intloop22 U U A U G U C G = 200
> intloop22 U U A U G U G G = 200
> intloop22 U U A U G U U G = 200
> intloop22 U U C U G A A G = 200
> intloop22 U U C U G A C G = 260
> intloop22 U U C U G A G G = -20
> intloop22 U U C U G A U G = 180
> intloop22 U U C U G C A G = 200
> intloop22 U U C U G C C G = 250
> intloop22 U U C U G C G G = 70
> intloop22 U U C U G C U G = 160
> intloop22 U U C U G G A G = 200
> intloop22 U U C U G G C G = 200
> intloop22 U U C U G G G G = 200
> intloop22 U U C U G G U G = 200
> intloop22 U U C U G U A G = 200
> intloop22 U U C U G U C G = 250
> intloop22 U U C U G U G G = 70
> intloop22 U U C U G U U G = 160
> intloop22 U U G U G A A G = 200
> intloop22 U U G U G A C G = 250
> intloop22 U U G U G A G G = 50
> intloop22 U U G U G A U G = 250
> intloop22 U U G U G C A G = 200
> intloop22 U U G U G C C G = 200
> intloop22 U U G U G C G G = 200
> intloop22 U U G U G C U G = 200
> intloop22 U U G U G G A G = 200
> intloop22 U U G U G G C G = 270
> intloop22 U U G U G G G G = 30
> intloop22 U U G U G G U G = 220
> intloop22 U U G U G U A G = 200
> intloop22 U U G U G U C G = 130
> intloop22 U U G U G U G G = -250
> intloop22 U U G U G U U G = 130
> intloop22 U U U U G A A G = 200
> intloop22 U U U U G A C G = 200
> intloop22 U U U U G A G G = 200
> intloop22 U U U U G A U G = 200
> intloop22 U U U U G C A G = 200
> intloop22 U U U U G C C G = 250
> intloop22 U U U U G C G G = 70
> intloop22 U U U U G C U G = 160
> intloop22 U U U U G G A G = 200
> intloop22 U U U U G G C G = 230
> intloop22 U U U U G G G G = -150
> intloop22 U U U U G G U G = 230
> intloop22 U U U U G U A G = 200
> intloop22 U U U U G U C G = 170
> intloop22 U U U U G U G G = 70
> intloop22 U U U U G U U G = 80
> intloop22 U A A G U A A G = 280
> intloop22 U A A G U A C G = 280
> intloop22 U A A G U A G G = 170
> intloop22 U A A G U A U G = 200
> intloop22 U A A G U C A G = 230
> intloop22 U A A G U C C G = 230
> intloop22 U A A G U C G G = 130
> intloop22 U A A G U C U G = 200
> intloop22 U A A G U G A G = 170
> intloop22 U A A G U G C G = 170
> intloop22 U A A G U G G G = 70
> intloop22 U A A G U G U G = 200
> intloop22 U A A G U U A G = 200
> intloop22 U A A G U U C G = 200
> intloop22 U A A G U U G G = 200
> intloop22 U A A G U U U G = 200
> intloop22 U A C G U A A G = 280
> intloop22 U A C G U A C G = 280
> intloop22 U A C G U A G G = 170
> intloop22 U A C G U A U G = 200
> intloop22 U A C G U C A G = 340
> intloop22 U A C G U C C G = 280
> intloop22 U A C G U C G G = 270
> intloop22 U A C G U C U G = 200
> intloop22 U A C G U G A G = 200
> intloop22 U A C G U G C G = 200
> intloop22 U A C G U G G G = 200
> intloop22 U A C G U G U G = 200
> intloop22 U A C G U U A G = 340
> intloop22 U A C G U U C G = 280
> intloop22 U A C G U U G G = 270
> intloop22 U A C G U U U G = 200
> intloop22 U A G G U A A G = 170
> intloop22 U A G G U A C G = 170
> intloop22 U A G G U A G G = 70
> intloop22 U A G G U A U G = 200
> intloop22 U A G G U C A G = 200
> intloop22 U A G G U C C G = 200
> intloop22 U A G G U C G G = 200
> intloop22 U A G G U C U G = 200
> intloop22 U A G G U G A G = 210
> intloop22 U A G G U G C G = 210
> intloop22 U A G G U G G G = 110
> intloop22 U A G G U G U G = 200
> intloop22 U A G G U U A G = 100
> intloop22 U A G G U U C G = 0
> intloop22 U A G G U U G G = 70
> intloop22 U A G G U U U G = 200
> intloop22 U A U G U A A G = 200
> intloop22 U A U G U A C G = 200
> intloop22 U A U G U A G G = 200
> intloop22 U A U G U A U G = 200
> intloop22 U A U G U C A G = 310
> intloop22 U A U G U C C G = 250
> intloop22 U A U G U C G G = 240
> intloop22 U A U G U C U G = 200
> intloop22 U A U G U G A G = 220
> intloop22 U A U G U G C G = 120
> intloop22 U A U G U G G G = 200
> intloop22 U A U G U G U G = 200
> intloop22 U A U G U U A G = 290
> intloop22 U A U G U U C G = 190
> intloop22 U A U G U U G G = 270
> intloop22 U A U G U U U G = 200
> intloop22 U C A G U A A G = 230
> intloop22 U C A G U A C G = 340
> intloop22 U C A G U A G G = 200
> intloop22 U C A G U A U G = 310
> intloop22 U C A G U C A G = 190
> intloop22 U C A G U C C G = 230
> intloop22 U C A G U C G G = 200
> intloop22 U C A G U C U G = 200
> intloop22 U C A G U G A G = 130
> intloop22 U C A G U G C G = 270
> intloop22 U C A G U G G G = 200
> intloop22 U C A G U G U G = 240
> intloop22 U C A G U U A G = 200
> intloop22 U C A G U U C G = 200
> intloop22 U C A G U U G G = 200
> intloop22 U C A G U U U G = 200
> intloop22 U C C G U A A G = 230
> intloop22 U C C G U A C G = 280
> intloop22 U C C G U A G G = 200
> intloop22 U C C G U A U G = 250
> intloop22 U C C G U C A G = 230
> intloop22 U C C G U C C G = 280
> intloop22 U C C G U C G G = 200
> intloop22 U C C G U C U G = 250
> intloop22 U C C G U G A G = 200
> intloop22 U C C G U G C G = 200
> intloop22 U C C G U G G G = 200
> intloop22 U C C G U G U G = 200
> intloop22 U C C G U U A G = 230
> intloop22 U C C G U U C G = 280
> intloop22 U C C G U U G G = 200
> intloop22 U C C G U U U G = 250
> intloop22 U C G G U A A G = 130
> intloop22 U C G G U A C G = 270
> intloop22 U C G G U A G G = 200
> intloop22 U C G G U A U G = 240
> intloop22 U C G G U C A G = 200
> intloop22 U C G G U C C G = 200
> intloop22 U C G G U C G G = 200
> intloop22 U C G G U C U G = 200
> intloop22 U C G G U G A G = 170
> intloop22 U C G G U G C G = 270
> intloop22 U C G G U G G G = 200
> intloop22 U C G G U G U G = 240
> intloop22 U C G G U U A G = -50
> intloop22 U C G G U U C G = 100
> intloop22 U C G G U U G G = 200
> intloop22 U C G G U U U G = 70
> intloop22 U C U G U A A G = 200
> intloop22 U C U G U A C G = 200
> intloop22 U C U G U A G G = 200
> intloop22 U C U G U A U G = 200
> intloop22 U C U G U C A G = 200
> intloop22 U C U G U C C G = 250
> intloop22 U C U G U C G G = 200
> intloop22 U C U G U C U G = 220
> intloop22 U C U G U G A G = 80
> intloop22 U C U G U G C G = 220
> intloop22 U C U G U G G G = 200
> intloop22 U C U G U G U G = 190
> intloop22 U C U G U U A G = 150
> intloop22 U C U G U U C G = 190
> intloop22 U C U G U U G G = 200
> intloop22 U C U G U U U G = 160
> intloop22 U G A G U A A G = 170
> intloop22 U G A G U A C G = 200
> intloop22 U G A G U A G G = 210
> intloop22 U G A G U A U G = 220
> intloop22 U G A G U C A G = 130
> intloop22 U G A G U C C G = 200
> intloop22 U G A G U C G G = 170
> intloop22 U G A G U C U G = 80
> intloop22 U G A G U G A G = 70
> intloop22 U G A G U G C G = 200
> intloop22 U G A G U G G G = 110
> intloop22 U G A G U G U G = 200
> intloop22 U G A G U U A G = 200
> intloop22 U G A G U U C G = 200
> intloop22 U G A G U U G G = 200
> intloop22 U G A G U U U G = 200
> intloop22 U G C G U A A G = 170
> intloop22 U G C G U A C G = 200
> intloop22 U G C G U A G G = 210
> intloop22 U G C G U A U G = 120
> intloop22 U G C G U C A G = 270
> intloop22 U G C G U C C G = 200
> intloop22 U G C G U C G G = 270
> intloop22 U G C G U C U G = 220
> intloop22 U G C G U G A G = 200
> intloop22 U G C G U G C G = 200
> intloop22 U G C G U G G G = 200
> intloop22 U G C G U G U G = 200
> intloop22 U G C G U U A G = 270
> intloop22 U G C G U U C G = 200
> intloop22 U G C G U U G G = 270
> intloop22 U G C G U U U G = 220
> intloop22 U G G G U A A G = 70
> intloop22 U G G G U A C G = 200
> intloop22 U G G G U A G G = 110
> intloop22 U G G G U A U G = 200
> intloop22 U G G G U C A G = 200
> intloop22 U G G G U C C G = 200
> intloop22 U G G G U C G G = 200
> intloop22 U G G G U C U G = 200
> intloop22 U G G G U G A G = 110
> intloop22 U G G G U G C G = 200
> intloop22 U G G G U G G G = 150
> intloop22 U G G G U G U G = 160
> intloop22 U G G G U U A G = 70
> intloop22 U G G G U U C G = 200
> intloop22 U G G G U U G G = 30
> intloop22 U G G G U U U G = -160
> intloop22 U G U G U A A G = 200
> intloop22 U G U G U A C G = 200
> intloop22 U G U G U A G G = 200
> intloop22 U G U G U A U G = 200
> intloop22 U G U G U C A G = 240
> intloop22 U G U G U C C G = 200
> intloop22 U G U G U C G G = 240
> intloop22 U G U G U C U G = 190
> intloop22 U G U G U G A G = 200
> intloop22 U G U G U G C G = 200
> intloop22 U G U G U G G G = 160
> intloop22 U G U G U G U G = -30
> intloop22 U G U G U U A G = 270
> intloop22 U G U G U U C G = 200
> intloop22 U G U G U U G G = 230
> intloop22 U G U G U U U G = 220
> intloop22 U U A G U A A G = 200
> intloop22 U U A G U A C G = 340
> intloop22 U U A G U A G G = 100
> intloop22 U U A G U A U G = 290
> intloop22 U U A G U C A G = 200
> intloop22 U U A G U C C G = 230
> intloop22 U U A G U C G G = -50
> intloop22 U U A G U C U G = 150
> intloop22 U U A G U G A G = 200
> intloop22 U U A G U G C G = 270
> intloop22 U U A G U G G G = 70
> intloop22 U U A G U G U G = 270
> intloop22 U U A G U U A G = 200
> intloop22 U U A G U U C G = 200
> intloop22 U U A G U U G G = 200
> intloop22 U U A G U U U G = 200
> intloop22 U U C G U A A G = 200
> intloop22 U U C G U A C G = 280
> intloop22 U U C G U A G G = 0
> intloop22 U U C G U A U G = 190
> intloop22 U U C G U C A G = 200
> intloop22 U U C G U C C G = 280
> intloop22 U U C G U C G G = 100
> intloop22 U U C G U C U G = 190
> intloop22 U U C G U G A G = 200
> intloop22 U U C G U G C G = 200
> intloop22 U U C G U G G G = 200
> intloop22 U U C G U G U G = 200
> intloop22 U U C G U U A G = 200
> intloop22 U U C G U U C G = 280
> intloop22 U U C G U U G G = 100
> intloop22 U U C G U U U G = 190
> intloop22 U U G G U A A G = 200
> intloop22 U U G G U A C G = 270
> intloop22 U U G G U A G G = 70
> intloop22 U U G G U A U G = 270
> intloop22 U U G G U C A G = 200
> intloop22 U U G G U C C G = 200
> intloop22 U U G G U C G G = 200
> intloop22 U U G G U C U G = 200
> intloop22 U U G G U G A G = 200
> intloop22 U U G G U G C G = 270
> intloop22 U U G G U G G G = 30
> intloop22 U U G G U G U G = 230
> intloop22 U U G G U U A G = 200
> intloop22 U U G G U U C G = 100
> intloop22 U U G G U U G G = -290
> intloop22 U U G G U U U G = 90
> intloop22 U U U G U A A G = 200
> intloop22 U U U G U A C G = 200
> intloop22 U U U G U A G G = 200
> intloop22 U U U G U A U G = 200
> intloop22 U U U G U C A G = 200
> intloop22 U U U G U C C G = 250
> intloop22 U U U G U C G G = 70
> intloop22 U U U G U C U G = 160
> intloop22 U U U G U G A G = 200
> intloop22 U U U G U G C G = 220
> intloop22 U U U G U G G G = -160
> intloop22 U U U G U G U G = 220
> intloop22 U U U G U U A G = 200
> intloop22 U U U G U U C G = 190
> intloop22 U U U G U U G G = 90
> intloop22 U U U G U U U G = 110
> intloop22 U A A U A A A G = 280
> intloop22 U A A U A A C G = 280
> intloop22 U A A U A A G G = 170
> intloop22 U A A U A A U G = 200
> intloop22 U A A U A C A G = 250
> intloop22 U A A U A C C G = 250
> intloop22 U A A U A C G G = 150
> intloop22 U A A U A C U G = 200
> intloop22 U A A U A G A G = 150
> intloop22 U A A U A G C G = 150
> intloop22 U A A U A G G G = 50
> intloop22 U A A U A G U G = 200
> intloop22 U A A U A U A G = 200
> intloop22 U A A U A U C G = 200
> intloop22 U A A U A U G G = 200
> intloop22 U A A U A U U G = 200
> intloop22 U A C U A A A G = 260
> intloop22 U A C U A A C G = 260
> intloop22 U A C U A A G G = 160
> intloop22 U A C U A A U G = 200
> intloop22 U A C U A C A G = 310
> intloop22 U A C U A C C G = 250
> intloop22 U A C U A C G G = 240
> intloop22 U A C U A C U G = 200
> intloop22 U A C U A G A G = 200
> intloop22 U A C U A G C G = 200
> intloop22 U A C U A G G G = 200
> intloop22 U A C U A G U G = 200
> intloop22 U A C U A U A G = 310
> intloop22 U A C U A U C G = 250
> intloop22 U A C U A U G G = 240
> intloop22 U A C U A U U G = 200
> intloop22 U A G U A A A G = 150
> intloop22 U A G U A A C G = 150
> intloop22 U A G U A A G G = 50
> intloop22 U A G U A A U G = 200
> intloop22 U A G U A C A G = 200
> intloop22 U A G U A C C G = 200
> intloop22 U A G U A C G G = 200
> intloop22 U A G U A C U G = 200
> intloop22 U A G U A G A G = 210
> intloop22 U A G U A G C G = 210
> intloop22 U A G U A G G G = 100
> intloop22 U A G U A G U G = 200
> intloop22 U A G U A U A G = 130
> intloop22 U A G U A U C G = 30
> intloop22 U A G U A U G G = 110
> intloop22 U A G U A U U G = 200
> intloop22 U A U U A A A G = 200
> intloop22 U A U U A A C G = 200
> intloop22 U A U U A A G G = 200
> intloop22 U A U U A A U G = 200
> intloop22 U A U U A C A G = 310
> intloop22 U A U U A C C G = 250
> intloop22 U A U U A C G G = 240
> intloop22 U A U U A C U G = 200
> intloop22 U A U U A G A G = 230
> intloop22 U A U U A G C G = 130
> intloop22 U A U U A G G G = 210
> intloop22 U A U U A G U G = 200
> intloop22 U A U U A U A G = 270
> intloop22 U A U U A U C G = 170
> intloop22 U A U U A U G G = 240
> intloop22 U A U U A U U G = 200
> intloop22 U C A U A A A G = 230
> intloop22 U C A U A A C G = 340
> intloop22 U C A U A A G G = 200
> intloop22 U C A U A A U G = 310
> intloop22 U C A U A C A G = 210
> intloop22 U C A U A C C G = 250
> intloop22 U C A U A C G G = 200
> intloop22 U C A U A C U G = 220
> intloop22 U C A U A G A G = 110
> intloop22 U C A U A G C G = 250
> intloop22 U C A U A G G G = 200
> intloop22 U C A U A G U G = 220
> intloop22 U C A U A U A G = 200
> intloop22 U C A U A U C G = 200
> intloop22 U C A U A U G G = 200
> intloop22 U C A U A U U G = 200
> intloop22 U C C U A A A G = 220
> intloop22 U C C U A A C G = 260
> intloop22 U C C U A A G G = 200
> intloop22 U C C U A A U G = 230
> intloop22 U C C U A C A G = 200
> intloop22 U C C U A C C G = 250
> intloop22 U C C U A C G G = 200
> intloop22 U C C U A C U G = 220
> intloop22 U C C U A G A G = 200
> intloop22 U C C U A G C G = 200
> intloop22 U C C U A G G G = 200
> intloop22 U C C U A G U G = 200
> intloop22 U C C U A U A G = 200
> intloop22 U C C U A U C G = 250
> intloop22 U C C U A U G G = 200
> intloop22 U C C U A U U G = 220
> intloop22 U C G U A A A G = 110
> intloop22 U C G U A A C G = 250
> intloop22 U C G U A A G G = 200
> intloop22 U C G U A A U G = 220
> intloop22 U C G U A C A G = 200
> intloop22 U C G U A C C G = 200
> intloop22 U C G U A C G G = 200
> intloop22 U C G U A C U G = 200
> intloop22 U C G U A G A G = 160
> intloop22 U C G U A G C G = 270
> intloop22 U C G U A G G G = 200
> intloop22 U C G U A G U G = 240
> intloop22 U C G U A U A G = -10
> intloop22 U C G U A U C G = 130
> intloop22 U C G U A U G G = 200
> intloop22 U C G U A U U G = 100
> intloop22 U C U U A A A G = 200
> intloop22 U C U U A A C G = 200
> intloop22 U C U U A A G G = 200
> intloop22 U C U U A A U G = 200
> intloop22 U C U U A C A G = 200
> intloop22 U C U U A C C G = 250
> intloop22 U C U U A C G G = 200
> intloop22 U C U U A C U G = 220
> intloop22 U C U U A G A G = 90
> intloop22 U C U U A G C G = 230
> intloop22 U C U U A G G G = 200
> intloop22 U C U U A G U G = 200
> intloop22 U C U U A U A G = 120
> intloop22 U C U U A U C G = 170
> intloop22 U C U U A U G G = 200
> intloop22 U C U U A U U G = 140
> intloop22 U G A U A A A G = 170
> intloop22 U G A U A A C G = 200
> intloop22 U G A U A A G G = 210
> intloop22 U G A U A A U G = 220
> intloop22 U G A U A C A G = 150
> intloop22 U G A U A C C G = 200
> intloop22 U G A U A C G G = 190
> intloop22 U G A U A C U G = 100
> intloop22 U G A U A G A G = 50
> intloop22 U G A U A G C G = 200
> intloop22 U G A U A G G G = 90
> intloop22 U G A U A G U G = 180
> intloop22 U G A U A U A G = 200
> intloop22 U G A U A U C G = 200
> intloop22 U G A U A U G G = 200
> intloop22 U G A U A U U G = 200
> intloop22 U G C U A A A G = 160
> intloop22 U G C U A A C G = 200
> intloop22 U G C U A A G G = 200
> intloop22 U G C U A A U G = 110
> intloop22 U G C U A C A G = 240
> intloop22 U G C U A C C G = 200
> intloop22 U G C U A C G G = 240
> intloop22 U G C U A C U G = 190
> intloop22 U G C U A G A G = 200
> intloop22 U G C U A G C G = 200
> intloop22 U G C U A G G G = 200
> intloop22 U G C U A G U G = 200
> intloop22 U G C U A U A G = 240
> intloop22 U G C U A U C G = 200
> intloop22 U G C U A U G G = 240
> intloop22 U G C U A U U G = 190
> intloop22 U G G U A A A G = 50
> intloop22 U G G U A A C G = 200
> intloop22 U G G U A A G G = 90
> intloop22 U G G U A A U G = 180
> intloop22 U G G U A C A G = 200
> intloop22 U G G U A C C G = 200
> intloop22 U G G U A C G G = 200
> intloop22 U G G U A C U G = 200
> intloop22 U G G U A G A G = 100
> intloop22 U G G U A G C G = 200
> intloop22 U G G U A G G G = 140
> intloop22 U G G U A G U G = 150
> intloop22 U G G U A U A G = 110
> intloop22 U G G U A U C G = 200
> intloop22 U G G U A U G G = 70
> intloop22 U G G U A U U G = -120
> intloop22 U G U U A A A G = 200
> intloop22 U G U U A A C G = 200
> intloop22 U G U U A A G G = 200
> intloop22 U G U U A A U G = 200
> intloop22 U G U U A C A G = 240
> intloop22 U G U U A C C G = 200
> intloop22 U G U U A C G G = 240
> intloop22 U G U U A C U G = 190
> intloop22 U G U U A G A G = 210
> intloop22 U G U U A G C G = 200
> intloop22 U G U U A G G G = 170
> intloop22 U G U U A G U G = -20
> intloop22 U G U U A U A G = 240
> intloop22 U G U U A U C G = 200
> intloop22 U G U U A U G G = 200
> intloop22 U G U U A U U G = 190
> intloop22 U U A U A A A G = 200
> intloop22 U U A U A A C G = 340
> intloop22 U U A U A A G G = 100
> intloop22 U U A U A A U G = 290
> intloop22 U U A U A C A G = 200
> intloop22 U U A U A C C G = 250
> intloop22 U U A U A C G G = -30
> intloop22 U U A U A C U G = 170
> intloop22 U U A U A G A G = 200
> intloop22 U U A U A G C G = 250
> intloop22 U U A U A G G G = 50
> intloop22 U U A U A G U G = 250
> intloop22 U U A U A U A G = 200
> intloop22 U U A U A U C G = 200
> intloop22 U U A U A U G G = 200
> intloop22 U U A U A U U G = 200
> intloop22 U U C U A A A G = 200
> intloop22 U U C U A A C G = 260
> intloop22 U U C U A A G G = -20
> intloop22 U U C U A A U G = 180
> intloop22 U U C U A C A G = 200
> intloop22 U U C U A C C G = 250
> intloop22 U U C U A C G G = 70
> intloop22 U U C U A C U G = 160
> intloop22 U U C U A G A G = 200
> intloop22 U U C U A G C G = 200
> intloop22 U U C U A G G G = 200
> intloop22 U U C U A G U G = 200
> intloop22 U U C U A U A G = 200
> intloop22 U U C U A U C G = 250
> intloop22 U U C U A U G G = 70
> intloop22 U U C U A U U G = 160
> intloop22 U U G U A A A G = 200
> intloop22 U U G U A A C G = 250
> intloop22 U U G U A A G G = 50
> intloop22 U U G U A A U G = 250
> intloop22 U U G U A C A G = 200
> intloop22 U U G U A C C G = 200
> intloop22 U U G U A C G G = 200
> intloop22 U U G U A C U G = 200
> intloop22 U U G U A G A G = 200
> intloop22 U U G U A G C G = 270
> intloop22 U U G U A G G G = 30
> intloop22 U U G U A G U G = 220
> intloop22 U U G U A U A G = 200
> intloop22 U U G U A U C G = 130
> intloop22 U U G U A U G G = -250
> intloop22 U U G U A U U G = 130
> intloop22 U U U U A A A G = 200
> intloop22 U U U U A A C G = 200
> intloop22 U U U U A A G G = 200
> intloop22 U U U U A A U G = 200
> intloop22 U U U U A C A G = 200
> intloop22 U U U U A C C G = 250
> intloop22 U U U U A C G G = 70
> intloop22 U U U U A C U G = 160
> intloop22 U U U U A G A G = 200
> intloop22 U U U U A G C G = 230
> intloop22 U U U U A G G G = -150
> intloop22 U U U U A G U G = 230
> intloop22 U U U U A U A G = 200
> intloop22 U U U U A U C G = 170
> intloop22 U U U U A U G G = 70
> intloop22 U U U U A U U G = 80
> intloop22 U A A A U A A G = 280
> intloop22 U A A A U A C G = 280
> intloop22 U A A A U A G G = 170
> intloop22 U A A A U A U G = 200
> intloop22 U A A A U C A G = 230
> intloop22 U A A A U C C G = 230
> intloop22 U A A A U C G G = 130
> intloop22 U A A A U C U G = 200
> intloop22 U A A A U G A G = 170
> intloop22 U A A A U G C G = 170
> intloop22 U A A A U G G G = 70
> intloop22 U A A A U G U G = 200
> intloop22 U A A A U U A G = 200
> intloop22 U A A A U U C G = 200
> intloop22 U A A A U U G G = 200
> intloop22 U A A A U U U G = 200
> intloop22 U A C A U A A G = 280
> intloop22 U A C A U A C G = 280
> intloop22 U A C A U A G G = 170
> intloop22 U A C A U A U G = 200
> intloop22 U A C A U C A G = 340
> intloop22 U A C A U C C G = 280
> intloop22 U A C A U C G G = 270
> intloop22 U A C A U C U G = 200
> intloop22 U A C A U G A G = 200
> intloop22 U A C A U G C G = 200
> intloop22 U A C A U G G G = 200
> intloop22 U A C A U G U G = 200
> intloop22 U A C A U U A G = 340
> intloop22 U A C A U U C G = 280
> intloop22 U A C A U U G G = 270
> intloop22 U A C A U U U G = 200
> intloop22 U A G A U A A G = 170
> intloop22 U A G A U A C G = 170
> intloop22 U A G A U A G G = 70
> intloop22 U A G A U A U G = 200
> intloop22 U A G A U C A G = 200
> intloop22 U A G A U C C G = 200
> intloop22 U A G A U C G G = 200
> intloop22 U A G A U C U G = 200
> intloop22 U A G A U G A G = 210
> intloop22 U A G A U G C G = 210
> intloop22 U A G A U G G G = 110
> intloop22 U A G A U G U G = 200
> intloop22 U A G A U U A G = 100
> intloop22 U A G A U U C G = 0
> intloop22 U A G A U U G G = 70
> intloop22 U A G A U U U G = 200
> intloop22 U A U A U A A G = 200
> intloop22 U A U A U A C G = 200
> intloop22 U A U A U A G G = 200
> intloop22 U A U A U A U G = 200
> intloop22 U A U A U C A G = 310
> intloop22 U A U A U C C G = 250
> intloop22 U A U A U C G G = 240
> intloop22 U A U A U C U G = 200
> intloop22 U A U A U G A G = 220
> intloop22 U A U A U G C G = 120
> intloop22 U A U A U G G G = 200
> intloop22 U A U A U G U G = 200
> intloop22 U A U A U U A G = 290
> intloop22 U A U A U U C G = 190
> intloop22 U A U A U U G G = 270
> intloop22 U A U A U U U G = 200
> intloop22 U C A A U A A G = 230
> intloop22 U C A A U A C G = 340
> intloop22 U C A A U A G G = 200
> intloop22 U C A A U A U G = 310
> intloop22 U C A A U C A G = 190
> intloop22 U C A A U C C G = 230
> intloop22 U C A A U C G G = 200
> intloop22 U C A A U C U G = 200
> intloop22 U C A A U G A G = 130
> intloop22 U C A A U G C G = 270
> intloop22 U C A A U G G G = 200
> intloop22 U C A A U G U G = 240
> intloop22 U C A A U U A G = 200
> intloop22 U C A A U U C G = 200
> intloop22 U C A A U U G G = 200
> intloop22 U C A A U U U G = 200
> intloop22 U C C A U A A G = 230
> intloop22 U C C A U A C G = 280
> intloop22 U C C A U A G G = 200
> intloop22 U C C A U A U G = 250
> intloop22 U C C A U C A G = 230
> intloop22 U C C A U C C G = 280
> intloop22 U C C A U C G G = 200
> intloop22 U C C A U C U G = 250
> intloop22 U C C A U G A G = 200
> intloop22 U C C A U G C G = 200
> intloop22 U C C A U G G G = 200
> intloop22 U C C A U G U G = 200
> intloop22 U C C A U U A G = 230
> intloop22 U C C A U U C G = 280
> intloop22 U C C A U U G G = 200
> intloop22 U C C A U U U G = 250
> intloop22 U C G A U A A G = 130
> intloop22 U C G A U A C G = 270
> intloop22 U C G A U A G G = 200
> intloop22 U C G A U A U G = 240
> intloop22 U C G A U C A G = 200
> intloop22 U C G A U C C G = 200
> intloop22 U C G A U C G G = 200
> intloop22 U C G A U C U G = 200
> intloop22 U C G A U G A G = 170
> intloop22 U C G A U G C G = 270
> intloop22 U C G A U G G G = 200
> intloop22 U C G A U G U G = 240
> intloop22 U C G A U U A G = -50
> intloop22 U C G A U U C G = 100
> intloop22 U C G A U U G G = 200
> intloop22 U C G A U U U G = 70
> intloop22 U C U A U A A G = 200
> intloop22 U C U A U A C G = 200
> intloop22 U C U A U A G G = 200
> intloop22 U C U A U A U G = 200
> intloop22 U C U A U C A G = 200
> intloop22 U C U A U C C G = 250
> intloop22 U C U A U C G G = 200
> intloop22 U C U A U C U G = 220
> intloop22 U C U A U G A G = 80
> intloop22 U C U A U G C G = 220
> intloop22 U C U A U G G G = 200
> intloop22 U C U A U G U G = 190
> intloop22 U C U A U U A G = 150
> intloop22 U C U A U U C G = 190
> intloop22 U C U A U U G G = 200
> intloop22 U C U A U U U G = 160
> intloop22 U G A A U A A G = 170
> intloop22 U G A A U A C G = 200
> intloop22 U G A A U A G G = 210
> intloop22 U G A A U A U G = 220
> intloop22 U G A A U C A G = 130
> intloop22 U G A A U C C G = 200
> intloop22 U G A A U C G G = 170
> intloop22 U G A A U C U G = 80
> intloop22 U G A A U G A G = 70
> intloop22 U G A A U G C G = 200
> intloop22 U G A A U G G G = 110
> intloop22 U G A A U G U G = 200
> intloop22 U G A A U U A G = 200
> intloop22 U G A A U U C G = 200
> intloop22 U G A A U U G G = 200
> intloop22 U G A A U U U G = 200
> intloop22 U G C A U A A G = 170
> intloop22 U G C A U A C G = 200
> intloop22 U G C A U A G G = 210
> intloop22 U G C A U A U G = 120
> intloop22 U G C A U C A G = 270
> intloop22 U G C A U C C G = 200
> intloop22 U G C A U C G G = 270
> intloop22 U G C A U C U G = 220
> intloop22 U G C A U G A G = 200
> intloop22 U G C A U G C G = 200
> intloop22 U G C A U G G G = 200
> intloop22 U G C A U G U G = 200
> intloop22 U G C A U U A G = 270
> intloop22 U G C A U U C G = 200
> intloop22 U G C A U U G G = 270
> intloop22 U G C A U U U G = 220
> intloop22 U G G A U A A G = 70
> intloop22 U G G A U A C G = 200
> intloop22 U G G A U A G G = 110
> intloop22 U G G A U A U G = 200
> intloop22 U G G A U C A G = 200
> intloop22 U G G A U C C G = 200
> intloop22 U G G A U C G G = 200
> intloop22 U G G A U C U G = 200
> intloop22 U G G A U G A G = 110
> intloop22 U G G A U G C G = 200
> intloop22 U G G A U G G G = 150
> intloop22 U G G A U G U G = 160
> intloop22 U G G A U U A G = 70
> intloop22 U G G A U U C G = 200
> intloop22 U G G A U U G G = 30
> intloop22 U G G A U U U G = -160
> intloop22 U G U A U A A G = 200
> intloop22 U G U A U A C G = 200
> intloop22 U G U A U A G G = 200
> intloop22 U G U A U A U G = 200
> intloop22 U G U A U C A G = 240
> intloop22 U G U A U C C G = 200
> intloop22 U G U A U C G G = 240
> intloop22 U G U A U C U G = 190
> intloop22 U G U A U G A G = 200
> intloop22 U G U A U G C G = 200
> intloop22 U G U A U G G G = 160
> intloop22 U G U A U G U G = -30
> intloop22 U G U A U U A G = 270
> intloop22 U G U A U U C G = 200
> intloop22 U G U A U U G G = 230
> intloop22 U G U A U U U G = 220
> intloop22 U U A A U A A G = 200
> intloop22 U U A A U A C G = 340
> intloop22 U U A A U A G G = 100
> intloop22 U U A A U A U G = 290
> intloop22 U U A A U C A G = 200
> intloop22 U U A A U C C G = 230
> intloop22 U U A A U C G G = -50
> intloop22 U U A A U C U G = 150
> intloop22 U U A A U G A G = 200
> intloop22 U U A A U G C G = 270
> intloop22 U U A A U G G G = 70
> intloop22 U U A A U G U G = 270
> intloop22 U U A A U U A G = 200
> intloop22 U U A A U U C G = 200
> intloop22 U U A A U U G G = 200
> intloop22 U U A A U U U G = 200
> intloop22 U U C A U A A G = 200
> intloop22 U U C A U A C G = 280
> intloop22 U U C A U A G G = 0
> intloop22 U U C A U A U G = 190
> intloop22 U U C A U C A G = 200
> intloop22 U U C A U C C G = 280
> intloop22 U U C A U C G G = 100
> intloop22 U U C A U C U G = 190
> intloop22 U U C A U G A G = 200
> intloop22 U U C A U G C G = 200
> intloop22 U U C A U G G G = 200
> intloop22 U U C A U G U G = 200
> intloop22 U U C A U U A G = 200
> intloop22 U U C A U U C G = 280
> intloop22 U U C A U U G G = 100
> intloop22 U U C A U U U G = 190
> intloop22 U U G A U A A G = 200
> intloop22 U U G A U A C G = 270
> intloop22 U U G A U A G G = 70
> intloop22 U U G A U A U G = 270
> intloop22 U U G A U C A G = 200
> intloop22 U U G A U C C G = 200
> intloop22 U U G A U C G G = 200
> intloop22 U U G A U C U G = 200
> intloop22 U U G A U G A G = 200
> intloop22 U U G A U G C G = 270
> intloop22 U U G A U G G G = 30
> intloop22 U U G A U G U G = 230
> intloop22 U U G A U U A G = 200
> intloop22 U U G A U U C G = 100
> intloop22 U U G A U U G G = -290
> intloop22 U U G A U U U G = 90
> intloop22 U U U A U A A G = 200
> intloop22 U U U A U A C G = 200
> intloop22 U U U A U A G G = 200
> intloop22 U U U A U A U G = 200
> intloop22 U U U A U C A G = 200
> intloop22 U U U A U C C G = 250
> intloop22 U U U A U C G G = 70
> intloop22 U U U A U C U G = 160
> intloop22 U U U A U G A G = 200
> intloop22 U U U A U G C G = 220
> intloop22 U U U A U G G G = -160
> intloop22 U U U A U G U G = 220
> intloop22 U U U A U U A G = 200
> intloop22 U U U A U U C G = 190
> intloop22 U U U A U U G G = 90
> intloop22 U U U A U U U G = 110
> intloop22 A A A G C A A U = 200
> intloop22 A A A G C A C U = 190
> intloop22 A A A G C A G U = 80
> intloop22 A A A G C A U U = 200
> intloop22 A A A G C C A U = 190
> intloop22 A A A G C C C U = 180
> intloop22 A A A G C C G U = 70
> intloop22 A A A G C C U U = 200
> intloop22 A A A G C G A U = 100
> intloop22 A A A G C G C U = 90
> intloop22 A A A G C G G U = -20
> intloop22 A A A G C G U U = 200
> intloop22 A A A G C U A U = 200
> intloop22 A A A G C U C U = 200
> intloop22 A A A G C U G U = 200
> intloop22 A A A G C U U U = 200
> intloop22 A A C G C A A U = 240
> intloop22 A A C G C A C U = 220
> intloop22 A A C G C A G U = 110
> intloop22 A A C G C A U U = 200
> intloop22 A A C G C C A U = 280
> intloop22 A A C G C C C U = 210
> intloop22 A A C G C C G U = 200
> intloop22 A A C G C C U U = 200
> intloop22 A A C G C G A U = 200
> intloop22 A A C G C G C U = 200
> intloop22 A A C G C G G U = 200
> intloop22 A A C G C G U U = 200
> intloop22 A A C G C U A U = 270
> intloop22 A A C G C U C U = 190
> intloop22 A A C G C U G U = 180
> intloop22 A A C G C U U U = 200
> intloop22 A A G G C A A U = 100
> intloop22 A A G G C A C U = 90
> intloop22 A A G G C A G U = -20
> intloop22 A A G G C A U U = 200
> intloop22 A A G G C C A U = 200
> intloop22 A A G G C C C U = 200
> intloop22 A A G G C C G U = 200
> intloop22 A A G G C C U U = 200
> intloop22 A A G G C G A U = 180
> intloop22 A A G G C G C U = 160
> intloop22 A A G G C G G U = 50
> intloop22 A A G G C G U U = 200
> intloop22 A A G G C U A U = 30
> intloop22 A A G G C U C U = -80
> intloop22 A A G G C U G U = -10
> intloop22 A A G G C U U U = 200
> intloop22 A A U G C A A U = 200
> intloop22 A A U G C A C U = 200
> intloop22 A A U G C A G U = 200
> intloop22 A A U G C A U U = 200
> intloop22 A A U G C C A U = 270
> intloop22 A A U G C C C U = 190
> intloop22 A A U G C C G U = 180
> intloop22 A A U G C C U U = 200
> intloop22 A A U G C G A U = 180
> intloop22 A A U G C G C U = 70
> intloop22 A A U G C G G U = 140
> intloop22 A A U G C G U U = 200
> intloop22 A A U G C U A U = 220
> intloop22 A A U G C U C U = 100
> intloop22 A A U G C U G U = 180
> intloop22 A A U G C U U U = 200
> intloop22 A C A G C A A U = 180
> intloop22 A C A G C A C U = 230
> intloop22 A C A G C A G U = 200
> intloop22 A C A G C A U U = 230
> intloop22 A C A G C C A U = 170
> intloop22 A C A G C C C U = 160
> intloop22 A C A G C C G U = 200
> intloop22 A C A G C C U U = 160
> intloop22 A C A G C G A U = 80
> intloop22 A C A G C G C U = 170
> intloop22 A C A G C G G U = 200
> intloop22 A C A G C G U U = 170
> intloop22 A C A G C U A U = 200
> intloop22 A C A G C U C U = 200
> intloop22 A C A G C U G U = 200
> intloop22 A C A G C U U U = 200
> intloop22 A C C G C A A U = 210
> intloop22 A C C G C A C U = 210
> intloop22 A C C G C A G U = 200
> intloop22 A C C G C A U U = 210
> intloop22 A C C G C C A U = 200
> intloop22 A C C G C C C U = 190
> intloop22 A C C G C C G U = 200
> intloop22 A C C G C C U U = 190
> intloop22 A C C G C G A U = 200
> intloop22 A C C G C G C U = 200
> intloop22 A C C G C G G U = 200
> intloop22 A C C G C G U U = 200
> intloop22 A C C G C U A U = 180
> intloop22 A C C G C U C U = 180
> intloop22 A C C G C U G U = 200
> intloop22 A C C G C U U U = 180
> intloop22 A C G G C A A U = 80
> intloop22 A C G G C A C U = 170
> intloop22 A C G G C A G U = 200
> intloop22 A C G G C A U U = 170
> intloop22 A C G G C C A U = 200
> intloop22 A C G G C C C U = 200
> intloop22 A C G G C C G U = 200
> intloop22 A C G G C C U U = 200
> intloop22 A C G G C G A U = 150
> intloop22 A C G G C G C U = 210
> intloop22 A C G G C G G U = 200
> intloop22 A C G G C G U U = 210
> intloop22 A C G G C U A U = -90
> intloop22 A C G G C U C U = 0
> intloop22 A C G G C U G U = 200
> intloop22 A C G G C U U U = 0
> intloop22 A C U G C A A U = 200
> intloop22 A C U G C A C U = 200
> intloop22 A C U G C A G U = 200
> intloop22 A C U G C A U U = 200
> intloop22 A C U G C C A U = 180
> intloop22 A C U G C C C U = 180
> intloop22 A C U G C C G U = 200
> intloop22 A C U G C C U U = 180
> intloop22 A C U G C G A U = 60
> intloop22 A C U G C G C U = 150
> intloop22 A C U G C G G U = 200
> intloop22 A C U G C G U U = 150
> intloop22 A C U G C U A U = 90
> intloop22 A C U G C U C U = 90
> intloop22 A C U G C U G U = 200
> intloop22 A C U G C U U U = 90
> intloop22 A G A G C A A U = 80
> intloop22 A G A G C A C U = 200
> intloop22 A G A G C A G U = 130
> intloop22 A G A G C A U U = 160
> intloop22 A G A G C C A U = 70
> intloop22 A G A G C C C U = 200
> intloop22 A G A G C C G U = 120
> intloop22 A G A G C C U U = 50
> intloop22 A G A G C G A U = -20
> intloop22 A G A G C G C U = 200
> intloop22 A G A G C G G U = 30
> intloop22 A G A G C G U U = 140
> intloop22 A G A G C U A U = 200
> intloop22 A G A G C U C U = 200
> intloop22 A G A G C U G U = 200
> intloop22 A G A G C U U U = 200
> intloop22 A G C G C A A U = 110
> intloop22 A G C G C A C U = 200
> intloop22 A G C G C A G U = 170
> intloop22 A G C G C A U U = 90
> intloop22 A G C G C C A U = 200
> intloop22 A G C G C C C U = 200
> intloop22 A G C G C C G U = 210
> intloop22 A G C G C C U U = 180
> intloop22 A G C G C G A U = 200
> intloop22 A G C G C G C U = 200
> intloop22 A G C G C G G U = 200
> intloop22 A G C G C G U U = 200
> intloop22 A G C G C U A U = 180
> intloop22 A G C G C U C U = 200
> intloop22 A G C G C U G U = 200
> intloop22 A G C G C U U U = 160
> intloop22 A G G G C A A U = -20
> intloop22 A G G G C A C U = 200
> intloop22 A G G G C A G U = 30
> intloop22 A G G G C A U U = 140
> intloop22 A G G G C C A U = 200
> intloop22 A G G G C C C U = 200
> intloop22 A G G G C C G U = 200
> intloop22 A G G G C C U U = 200
> intloop22 A G G G C G A U = 50
> intloop22 A G G G C G C U = 200
> intloop22 A G G G C G G U = 110
> intloop22 A G G G C G U U = 130
> intloop22 A G G G C U A U = -10
> intloop22 A G G G C U C U = 200
> intloop22 A G G G C U G U = -40
> intloop22 A G G G C U U U = -210
> intloop22 A G U G C A A U = 200
> intloop22 A G U G C A C U = 200
> intloop22 A G U G C A G U = 200
> intloop22 A G U G C A U U = 200
> intloop22 A G U G C C A U = 180
> intloop22 A G U G C C C U = 200
> intloop22 A G U G C C G U = 200
> intloop22 A G U G C C U U = 160
> intloop22 A G U G C G A U = 140
> intloop22 A G U G C G C U = 200
> intloop22 A G U G C G G U = 110
> intloop22 A G U G C G U U = -60
> intloop22 A G U G C U A U = 180
> intloop22 A G U G C U C U = 200
> intloop22 A G U G C U G U = 150
> intloop22 A G U G C U U U = 160
> intloop22 A U A G C A A U = 200
> intloop22 A U A G C A C U = 230
> intloop22 A U A G C A G U = 60
> intloop22 A U A G C A U U = 190
> intloop22 A U A G C C A U = 200
> intloop22 A U A G C C C U = 160
> intloop22 A U A G C C G U = -50
> intloop22 A U A G C C U U = 80
> intloop22 A U A G C G A U = 200
> intloop22 A U A G C G C U = 170
> intloop22 A U A G C G G U = 40
> intloop22 A U A G C G U U = 180
> intloop22 A U A G C U A U = 200
> intloop22 A U A G C U C U = 200
> intloop22 A U A G C U G U = 200
> intloop22 A U A G C U U U = 200
> intloop22 A U C G C A A U = 200
> intloop22 A U C G C A C U = 210
> intloop22 A U C G C A G U = 0
> intloop22 A U C G C A U U = 130
> intloop22 A U C G C C A U = 200
> intloop22 A U C G C C C U = 190
> intloop22 A U C G C C G U = 80
> intloop22 A U C G C C U U = 110
> intloop22 A U C G C G A U = 200
> intloop22 A U C G C G C U = 200
> intloop22 A U C G C G G U = 200
> intloop22 A U C G C G U U = 200
> intloop22 A U C G C U A U = 200
> intloop22 A U C G C U C U = 180
> intloop22 A U C G C U G U = 70
> intloop22 A U C G C U U U = 100
> intloop22 A U G G C A A U = 200
> intloop22 A U G G C A C U = 170
> intloop22 A U G G C A G U = 40
> intloop22 A U G G C A U U = 180
> intloop22 A U G G C C A U = 200
> intloop22 A U G G C C C U = 200
> intloop22 A U G G C C G U = 200
> intloop22 A U G G C C U U = 200
> intloop22 A U G G C G A U = 200
> intloop22 A U G G C G C U = 210
> intloop22 A U G G C G G U = 40
> intloop22 A U G G C G U U = 170
> intloop22 A U G G C U A U = 200
> intloop22 A U G G C U C U = 0
> intloop22 A U G G C U G U = -310
> intloop22 A U G G C U U U = 0
> intloop22 A U U G C A A U = 200
> intloop22 A U U G C A C U = 200
> intloop22 A U U G C A G U = 200
> intloop22 A U U G C A U U = 200
> intloop22 A U U G C C A U = 200
> intloop22 A U U G C C C U = 180
> intloop22 A U U G C C G U = 70
> intloop22 A U U G C C U U = 100
> intloop22 A U U G C G A U = 200
> intloop22 A U U G C G C U = 150
> intloop22 A U U G C G G U = -160
> intloop22 A U U G C G U U = 160
> intloop22 A U U G C U A U = 200
> intloop22 A U U G C U C U = 90
> intloop22 A U U G C U G U = 60
> intloop22 A U U G C U U U = 10
> intloop22 A A A C G A A U = 210
> intloop22 A A A C G A C U = 200
> intloop22 A A A C G A G U = 90
> intloop22 A A A C G A U U = 200
> intloop22 A A A C G C A U = 190
> intloop22 A A A C G C C U = 170
> intloop22 A A A C G C G U = 60
> intloop22 A A A C G C U U = 200
> intloop22 A A A C G G A U = 10
> intloop22 A A A C G G C U = 0
> intloop22 A A A C G G G U = -110
> intloop22 A A A C G G U U = 200
> intloop22 A A A C G U A U = 200
> intloop22 A A A C G U C U = 200
> intloop22 A A A C G U G U = 200
> intloop22 A A A C G U U U = 200
> intloop22 A A C C G A A U = 180
> intloop22 A A C C G A C U = 170
> intloop22 A A C C G A G U = 60
> intloop22 A A C C G A U U = 200
> intloop22 A A C C G C A U = 250
> intloop22 A A C C G C C U = 170
> intloop22 A A C C G C G U = 160
> intloop22 A A C C G C U U = 200
> intloop22 A A C C G G A U = 200
> intloop22 A A C C G G C U = 200
> intloop22 A A C C G G G U = 200
> intloop22 A A C C G G U U = 200
> intloop22 A A C C G U A U = 150
> intloop22 A A C C G U C U = 70
> intloop22 A A C C G U G U = 70
> intloop22 A A C C G U U U = 200
> intloop22 A A G C G A A U = 70
> intloop22 A A G C G A C U = 60
> intloop22 A A G C G A G U = -50
> intloop22 A A G C G A U U = 200
> intloop22 A A G C G C A U = 200
> intloop22 A A G C G C C U = 200
> intloop22 A A G C G C G U = 200
> intloop22 A A G C G C U U = 200
> intloop22 A A G C G G A U = 180
> intloop22 A A G C G G C U = 160
> intloop22 A A G C G G G U = 50
> intloop22 A A G C G G U U = 200
> intloop22 A A G C G U A U = 0
> intloop22 A A G C G U C U = -120
> intloop22 A A G C G U G U = -50
> intloop22 A A G C G U U U = 200
> intloop22 A A U C G A A U = 200
> intloop22 A A U C G A C U = 200
> intloop22 A A U C G A G U = 200
> intloop22 A A U C G A U U = 200
> intloop22 A A U C G C A U = 250
> intloop22 A A U C G C C U = 180
> intloop22 A A U C G C G U = 170
> intloop22 A A U C G C U U = 200
> intloop22 A A U C G G A U = 40
> intloop22 A A U C G G C U = -80
> intloop22 A A U C G G G U = -10
> intloop22 A A U C G G U U = 200
> intloop22 A A U C G U A U = 210
> intloop22 A A U C G U C U = 100
> intloop22 A A U C G U G U = 170
> intloop22 A A U C G U U U = 200
> intloop22 A C A C G A A U = 190
> intloop22 A C A C G A C U = 240
> intloop22 A C A C G A G U = 200
> intloop22 A C A C G A U U = 240
> intloop22 A C A C G C A U = 160
> intloop22 A C A C G C C U = 160
> intloop22 A C A C G C G U = 200
> intloop22 A C A C G C U U = 160
> intloop22 A C A C G G A U = -10
> intloop22 A C A C G G C U = 80
> intloop22 A C A C G G G U = 200
> intloop22 A C A C G G U U = 80
> intloop22 A C A C G U A U = 200
> intloop22 A C A C G U C U = 200
> intloop22 A C A C G U G U = 200
> intloop22 A C A C G U U U = 200
> intloop22 A C C C G A A U = 160
> intloop22 A C C C G A C U = 150
> intloop22 A C C C G A G U = 200
> intloop22 A C C C G A U U = 150
> intloop22 A C C C G C A U = 160
> intloop22 A C C C G C C U = 160
> intloop22 A C C C G C G U = 200
> intloop22 A C C C G C U U = 160
> intloop22 A C C C G G A U = 200
> intloop22 A C C C G G C U = 200
> intloop22 A C C C G G G U = 200
> intloop22 A C C C G G U U = 200
> intloop22 A C C C G U A U = 60
> intloop22 A C C C G U C U = 60
> intloop22 A C C C G U G U = 200
> intloop22 A C C C G U U U = 60
> intloop22 A C G C G A A U = 50
> intloop22 A C G C G A C U = 140
> intloop22 A C G C G A G U = 200
> intloop22 A C G C G A U U = 140
> intloop22 A C G C G C A U = 200
> intloop22 A C G C G C C U = 200
> intloop22 A C G C G C G U = 200
> intloop22 A C G C G C U U = 200
> intloop22 A C G C G G A U = 150
> intloop22 A C G C G G C U = 210
> intloop22 A C G C G G G U = 200
> intloop22 A C G C G G U U = 210
> intloop22 A C G C G U A U = -130
> intloop22 A C G C G U C U = -30
> intloop22 A C G C G U G U = 200
> intloop22 A C G C G U U U = -30
> intloop22 A C U C G A A U = 200
> intloop22 A C U C G A C U = 200
> intloop22 A C U C G A G U = 200
> intloop22 A C U C G A U U = 200
> intloop22 A C U C G C A U = 170
> intloop22 A C U C G C C U = 160
> intloop22 A C U C G C G U = 200
> intloop22 A C U C G C U U = 160
> intloop22 A C U C G G A U = -90
> intloop22 A C U C G G C U = 10
> intloop22 A C U C G G G U = 200
> intloop22 A C U C G G U U = 10
> intloop22 A C U C G U A U = 90
> intloop22 A C U C G U C U = 80
> intloop22 A C U C G U G U = 200
> intloop22 A C U C G U U U = 80
> intloop22 A G A C G A A U = 90
> intloop22 A G A C G A C U = 200
> intloop22 A G A C G A G U = 140
> intloop22 A G A C G A U U = 170
> intloop22 A G A C G C A U = 60
> intloop22 A G A C G C C U = 200
> intloop22 A G A C G C G U = 120
> intloop22 A G A C G C U U = 40
> intloop22 A G A C G G A U = -110
> intloop22 A G A C G G C U = 200
> intloop22 A G A C G G G U = -60
> intloop22 A G A C G G U U = 50
> intloop22 A G A C G U A U = 200
> intloop22 A G A C G U C U = 200
> intloop22 A G A C G U G U = 200
> intloop22 A G A C G U U U = 200
> intloop22 A G C C G A A U = 60
> intloop22 A G C C G A C U = 200
> intloop22 A G C C G A G U = 110
> intloop22 A G C C G A U U = 40
> intloop22 A G C C G C A U = 160
> intloop22 A G C C G C C U = 200
> intloop22 A G C C G C G U = 180
> intloop22 A G C C G C U U = 140
> intloop22 A G C C G G A U = 200
> intloop22 A G C C G G C U = 200
> intloop22 A G C C G G G U = 200
> intloop22 A G C C G G U U = 200
> intloop22 A G C C G U A U = 70
> intloop22 A G C C G U C U = 200
> intloop22 A G C C G U G U = 80
> intloop22 A G C C G U U U = 50
> intloop22 A G G C G A A U = -50
> intloop22 A G G C G A C U = 200
> intloop22 A G G C G A G U = 0
> intloop22 A G G C G A U U = 110
> intloop22 A G G C G C A U = 200
> intloop22 A G G C G C C U = 200
> intloop22 A G G C G C G U = 200
> intloop22 A G G C G C U U = 200
> intloop22 A G G C G G A U = 50
> intloop22 A G G C G G C U = 200
> intloop22 A G G C G G G U = 110
> intloop22 A G G C G G U U = 130
> intloop22 A G G C G U A U = -50
> intloop22 A G G C G U C U = 200
> intloop22 A G G C G U G U = -70
> intloop22 A G G C G U U U = -250
> intloop22 A G U C G A A U = 200
> intloop22 A G U C G A C U = 200
> intloop22 A G U C G A G U = 200
> intloop22 A G U C G A U U = 200
> intloop22 A G U C G C A U = 170
> intloop22 A G U C G C C U = 200
> intloop22 A G U C G C G U = 180
> intloop22 A G U C G C U U = 150
> intloop22 A G U C G G A U = -10
> intloop22 A G U C G G C U = 200
> intloop22 A G U C G G G U = -30
> intloop22 A G U C G G U U = -210
> intloop22 A G U C G U A U = 170
> intloop22 A G U C G U C U = 200
> intloop22 A G U C G U G U = 140
> intloop22 A G U C G U U U = 150
> intloop22 A U A C G A A U = 200
> intloop22 A U A C G A C U = 240
> intloop22 A U A C G A G U = 70
> intloop22 A U A C G A U U = 200
> intloop22 A U A C G C A U = 200
> intloop22 A U A C G C C U = 160
> intloop22 A U A C G C G U = -50
> intloop22 A U A C G C U U = 80
> intloop22 A U A C G G A U = 200
> intloop22 A U A C G G C U = 80
> intloop22 A U A C G G G U = -50
> intloop22 A U A C G G U U = 80
> intloop22 A U A C G U A U = 200
> intloop22 A U A C G U C U = 200
> intloop22 A U A C G U G U = 200
> intloop22 A U A C G U U U = 200
> intloop22 A U C C G A A U = 200
> intloop22 A U C C G A C U = 150
> intloop22 A U C C G A G U = -60
> intloop22 A U C C G A U U = 70
> intloop22 A U C C G C A U = 200
> intloop22 A U C C G C C U = 160
> intloop22 A U C C G C G U = 50
> intloop22 A U C C G C U U = 80
> intloop22 A U C C G G A U = 200
> intloop22 A U C C G G C U = 200
> intloop22 A U C C G G G U = 200
> intloop22 A U C C G G U U = 200
> intloop22 A U C C G U A U = 200
> intloop22 A U C C G U C U = 60
> intloop22 A U C C G U G U = -50
> intloop22 A U C C G U U U = -20
> intloop22 A U G C G A A U = 200
> intloop22 A U G C G A C U = 140
> intloop22 A U G C G A G U = 10
> intloop22 A U G C G A U U = 150
> intloop22 A U G C G C A U = 200
> intloop22 A U G C G C C U = 200
> intloop22 A U G C G C G U = 200
> intloop22 A U G C G C U U = 200
> intloop22 A U G C G G A U = 200
> intloop22 A U G C G G C U = 210
> intloop22 A U G C G G G U = 40
> intloop22 A U G C G G U U = 170
> intloop22 A U G C G U A U = 200
> intloop22 A U G C G U C U = -30
> intloop22 A U G C G U G U = -350
> intloop22 A U G C G U U U = -30
> intloop22 A U U C G A A U = 200
> intloop22 A U U C G A C U = 200
> intloop22 A U U C G A G U = 200
> intloop22 A U U C G A U U = 200
> intloop22 A U U C G C A U = 200
> intloop22 A U U C G C C U = 160
> intloop22 A U U C G C G U = 50
> intloop22 A U U C G C U U = 80
> intloop22 A U U C G G A U = 200
> intloop22 A U U C G G C U = 10
> intloop22 A U U C G G G U = -310
> intloop22 A U U C G G U U = 10
> intloop22 A U U C G U A U = 200
> intloop22 A U U C G U C U = 80
> intloop22 A U U C G U G U = 50
> intloop22 A U U C G U U U = 0
> intloop22 A A A U G A A U = 280
> intloop22 A A A U G A C U = 260
> intloop22 A A A U G A G U = 150
> intloop22 A A A U G A U U = 200
> intloop22 A A A U G C A U = 250
> intloop22 A A A U G C C U = 240
> intloop22 A A A U G C G U = 130
> intloop22 A A A U G C U U = 200
> intloop22 A A A U G G A U = 150
> intloop22 A A A U G G C U = 140
> intloop22 A A A U G G G U = 30
> intloop22 A A A U G G U U = 200
> intloop22 A A A U G U A U = 200
> intloop22 A A A U G U C U = 200
> intloop22 A A A U G U G U = 200
> intloop22 A A A U G U U U = 200
> intloop22 A A C U G A A U = 260
> intloop22 A A C U G A C U = 250
> intloop22 A A C U G A G U = 140
> intloop22 A A C U G A U U = 200
> intloop22 A A C U G C A U = 310
> intloop22 A A C U G C C U = 230
> intloop22 A A C U G C G U = 220
> intloop22 A A C U G C U U = 200
> intloop22 A A C U G G A U = 200
> intloop22 A A C U G G C U = 200
> intloop22 A A C U G G G U = 200
> intloop22 A A C U G G U U = 200
> intloop22 A A C U G U A U = 310
> intloop22 A A C U G U C U = 230
> intloop22 A A C U G U G U = 220
> intloop22 A A C U G U U U = 200
> intloop22 A A G U G A A U = 150
> intloop22 A A G U G A C U = 140
> intloop22 A A G U G A G U = 30
> intloop22 A A G U G A U U = 200
> intloop22 A A G U G C A U = 200
> intloop22 A A G U G C C U = 200
> intloop22 A A G U G C G U = 200
> intloop22 A A G U G C U U = 200
> intloop22 A A G U G G A U = 210
> intloop22 A A G U G G C U = 190
> intloop22 A A G U G G G U = 80
> intloop22 A A G U G G U U = 200
> intloop22 A A G U G U A U = 130
> intloop22 A A G U G U C U = 20
> intloop22 A A G U G U G U = 90
> intloop22 A A G U G U U U = 200
> intloop22 A A U U G A A U = 200
> intloop22 A A U U G A C U = 200
> intloop22 A A U U G A G U = 200
> intloop22 A A U U G A U U = 200
> intloop22 A A U U G C A U = 310
> intloop22 A A U U G C C U = 230
> intloop22 A A U U G C G U = 220
> intloop22 A A U U G C U U = 200
> intloop22 A A U U G G A U = 230
> intloop22 A A U U G G C U = 120
> intloop22 A A U U G G G U = 190
> intloop22 A A U U G G U U = 200
> intloop22 A A U U G U A U = 270
> intloop22 A A U U G U C U = 150
> intloop22 A A U U G U G U = 220
> intloop22 A A U U G U U U = 200
> intloop22 A C A U G A A U = 250
> intloop22 A C A U G A C U = 310
> intloop22 A C A U G A G U = 200
> intloop22 A C A U G A U U = 310
> intloop22 A C A U G C A U = 230
> intloop22 A C A U G C C U = 220
> intloop22 A C A U G C G U = 200
> intloop22 A C A U G C U U = 220
> intloop22 A C A U G G A U = 130
> intloop22 A C A U G G C U = 220
> intloop22 A C A U G G G U = 200
> intloop22 A C A U G G U U = 220
> intloop22 A C A U G U A U = 200
> intloop22 A C A U G U C U = 200
> intloop22 A C A U G U G U = 200
> intloop22 A C A U G U U U = 200
> intloop22 A C C U G A A U = 240
> intloop22 A C C U G A C U = 230
> intloop22 A C C U G A G U = 200
> intloop22 A C C U G A U U = 230
> intloop22 A C C U G C A U = 220
> intloop22 A C C U G C C U = 220
> intloop22 A C C U G C G U = 200
> intloop22 A C C U G C U U = 220
> intloop22 A C C U G G A U = 200
> intloop22 A C C U G G C U = 200
> intloop22 A C C U G G G U = 200
> intloop22 A C C U G G U U = 200
> intloop22 A C C U G U A U = 220
> intloop22 A C C U G U C U = 220
> intloop22 A C C U G U G U = 200
> intloop22 A C C U G U U U = 220
> intloop22 A C G U G A A U = 130
> intloop22 A C G U G A C U = 220
> intloop22 A C G U G A G U = 200
> intloop22 A C G U G A U U = 220
> intloop22 A C G U G C A U = 200
> intloop22 A C G U G C C U = 200
> intloop22 A C G U G C G U = 200
> intloop22 A C G U G C U U = 200
> intloop22 A C G U G G A U = 180
> intloop22 A C G U G G C U = 240
> intloop22 A C G U G G G U = 200
> intloop22 A C G U G G U U = 240
> intloop22 A C G U G U A U = 10
> intloop22 A C G U G U C U = 100
> intloop22 A C G U G U G U = 200
> intloop22 A C G U G U U U = 100
> intloop22 A C U U G A A U = 200
> intloop22 A C U U G A C U = 200
> intloop22 A C U U G A G U = 200
> intloop22 A C U U G A U U = 200
> intloop22 A C U U G C A U = 220
> intloop22 A C U U G C C U = 220
> intloop22 A C U U G C G U = 200
> intloop22 A C U U G C U U = 220
> intloop22 A C U U G G A U = 110
> intloop22 A C U U G G C U = 200
> intloop22 A C U U G G G U = 200
> intloop22 A C U U G G U U = 200
> intloop22 A C U U G U A U = 140
> intloop22 A C U U G U C U = 140
> intloop22 A C U U G U G U = 200
> intloop22 A C U U G U U U = 140
> intloop22 A G A U G A A U = 150
> intloop22 A G A U G A C U = 200
> intloop22 A G A U G A G U = 210
> intloop22 A G A U G A U U = 230
> intloop22 A G A U G C A U = 130
> intloop22 A G A U G C C U = 200
> intloop22 A G A U G C G U = 180
> intloop22 A G A U G C U U = 110
> intloop22 A G A U G G A U = 30
> intloop22 A G A U G G C U = 200
> intloop22 A G A U G G G U = 80
> intloop22 A G A U G G U U = 190
> intloop22 A G A U G U A U = 200
> intloop22 A G A U G U C U = 200
> intloop22 A G A U G U G U = 200
> intloop22 A G A U G U U U = 200
> intloop22 A G C U G A A U = 140
> intloop22 A G C U G A C U = 200
> intloop22 A G C U G A G U = 190
> intloop22 A G C U G A U U = 120
> intloop22 A G C U G C A U = 220
> intloop22 A G C U G C C U = 200
> intloop22 A G C U G C G U = 240
> intloop22 A G C U G C U U = 200
> intloop22 A G C U G G A U = 200
> intloop22 A G C U G G C U = 200
> intloop22 A G C U G G G U = 200
> intloop22 A G C U G G U U = 200
> intloop22 A G C U G U A U = 220
> intloop22 A G C U G U C U = 200
> intloop22 A G C U G U G U = 240
> intloop22 A G C U G U U U = 200
> intloop22 A G G U G A A U = 30
> intloop22 A G G U G A C U = 200
> intloop22 A G G U G A G U = 80
> intloop22 A G G U G A U U = 190
> intloop22 A G G U G C A U = 200
> intloop22 A G G U G C C U = 200
> intloop22 A G G U G C G U = 200
> intloop22 A G G U G C U U = 200
> intloop22 A G G U G G A U = 80
> intloop22 A G G U G G C U = 200
> intloop22 A G G U G G G U = 140
> intloop22 A G G U G G U U = 160
> intloop22 A G G U G U A U = 90
> intloop22 A G G U G U C U = 200
> intloop22 A G G U G U G U = 70
> intloop22 A G G U G U U U = -110
> intloop22 A G U U G A A U = 200
> intloop22 A G U U G A C U = 200
> intloop22 A G U U G A G U = 200
> intloop22 A G U U G A U U = 200
> intloop22 A G U U G C A U = 220
> intloop22 A G U U G C C U = 200
> intloop22 A G U U G C G U = 240
> intloop22 A G U U G C U U = 200
> intloop22 A G U U G G A U = 190
> intloop22 A G U U G G C U = 200
> intloop22 A G U U G G G U = 160
> intloop22 A G U U G G U U = -10
> intloop22 A G U U G U A U = 220
> intloop22 A G U U G U C U = 200
> intloop22 A G U U G U G U = 200
> intloop22 A G U U G U U U = 200
> intloop22 A U A U G A A U = 200
> intloop22 A U A U G A C U = 310
> intloop22 A U A U G A G U = 130
> intloop22 A U A U G A U U = 270
> intloop22 A U A U G C A U = 200
> intloop22 A U A U G C C U = 220
> intloop22 A U A U G C G U = 10
> intloop22 A U A U G C U U = 140
> intloop22 A U A U G G A U = 200
> intloop22 A U A U G G C U = 220
> intloop22 A U A U G G G U = 90
> intloop22 A U A U G G U U = 220
> intloop22 A U A U G U A U = 200
> intloop22 A U A U G U C U = 200
> intloop22 A U A U G U G U = 200
> intloop22 A U A U G U U U = 200
> intloop22 A U C U G A A U = 200
> intloop22 A U C U G A C U = 230
> intloop22 A U C U G A G U = 20
> intloop22 A U C U G A U U = 150
> intloop22 A U C U G C A U = 200
> intloop22 A U C U G C C U = 220
> intloop22 A U C U G C G U = 100
> intloop22 A U C U G C U U = 140
> intloop22 A U C U G G A U = 200
> intloop22 A U C U G G C U = 200
> intloop22 A U C U G G G U = 200
> intloop22 A U C U G G U U = 200
> intloop22 A U C U G U A U = 200
> intloop22 A U C U G U C U = 220
> intloop22 A U C U G U G U = 100
> intloop22 A U C U G U U U = 140
> intloop22 A U G U G A A U = 200
> intloop22 A U G U G A C U = 220
> intloop22 A U G U G A G U = 90
> intloop22 A U G U G A U U = 220
> intloop22 A U G U G C A U = 200
> intloop22 A U G U G C C U = 200
> intloop22 A U G U G C G U = 200
> intloop22 A U G U G C U U = 200
> intloop22 A U G U G G A U = 200
> intloop22 A U G U G G C U = 240
> intloop22 A U G U G G G U = 70
> intloop22 A U G U G G U U = 200
> intloop22 A U G U G U A U = 200
> intloop22 A U G U G U C U = 100
> intloop22 A U G U G U G U = -210
> intloop22 A U G U G U U U = 110
> intloop22 A U U U G A A U = 200
> intloop22 A U U U G A C U = 200
> intloop22 A U U U G A G U = 200
> intloop22 A U U U G A U U = 200
> intloop22 A U U U G C A U = 200
> intloop22 A U U U G C C U = 220
> intloop22 A U U U G C G U = 100
> intloop22 A U U U G C U U = 140
> intloop22 A U U U G G A U = 200
> intloop22 A U U U G G C U = 200
> intloop22 A U U U G G G U = -110
> intloop22 A U U U G G U U = 200
> intloop22 A U U U G U A U = 200
> intloop22 A U U U G U C U = 140
> intloop22 A U U U G U G U = 110
> intloop22 A U U U G U U U = 60
> intloop22 A A A G U A A U = 280
> intloop22 A A A G U A C U = 260
> intloop22 A A A G U A G U = 150
> intloop22 A A A G U A U U = 200
> intloop22 A A A G U C A U = 230
> intloop22 A A A G U C C U = 220
> intloop22 A A A G U C G U = 110
> intloop22 A A A G U C U U = 200
> intloop22 A A A G U G A U = 170
> intloop22 A A A G U G C U = 160
> intloop22 A A A G U G G U = 50
> intloop22 A A A G U G U U = 200
> intloop22 A A A G U U A U = 200
> intloop22 A A A G U U C U = 200
> intloop22 A A A G U U G U = 200
> intloop22 A A A G U U U U = 200
> intloop22 A A C G U A A U = 280
> intloop22 A A C G U A C U = 260
> intloop22 A A C G U A G U = 150
> intloop22 A A C G U A U U = 200
> intloop22 A A C G U C A U = 340
> intloop22 A A C G U C C U = 260
> intloop22 A A C G U C G U = 250
> intloop22 A A C G U C U U = 200
> intloop22 A A C G U G A U = 200
> intloop22 A A C G U G C U = 200
> intloop22 A A C G U G G U = 200
> intloop22 A A C G U G U U = 200
> intloop22 A A C G U U A U = 340
> intloop22 A A C G U U C U = 260
> intloop22 A A C G U U G U = 250
> intloop22 A A C G U U U U = 200
> intloop22 A A G G U A A U = 170
> intloop22 A A G G U A C U = 160
> intloop22 A A G G U A G U = 50
> intloop22 A A G G U A U U = 200
> intloop22 A A G G U C A U = 200
> intloop22 A A G G U C C U = 200
> intloop22 A A G G U C G U = 200
> intloop22 A A G G U C U U = 200
> intloop22 A A G G U G A U = 210
> intloop22 A A G G U G C U = 200
> intloop22 A A G G U G G U = 90
> intloop22 A A G G U G U U = 200
> intloop22 A A G G U U A U = 100
> intloop22 A A G G U U C U = -20
> intloop22 A A G G U U G U = 50
> intloop22 A A G G U U U U = 200
> intloop22 A A U G U A A U = 200
> intloop22 A A U G U A C U = 200
> intloop22 A A U G U A G U = 200
> intloop22 A A U G U A U U = 200
> intloop22 A A U G U C A U = 310
> intloop22 A A U G U C C U = 230
> intloop22 A A U G U C G U = 220
> intloop22 A A U G U C U U = 200
> intloop22 A A U G U G A U = 220
> intloop22 A A U G U G C U = 110
> intloop22 A A U G U G G U = 180
> intloop22 A A U G U G U U = 200
> intloop22 A A U G U U A U = 290
> intloop22 A A U G U U C U = 180
> intloop22 A A U G U U G U = 250
> intloop22 A A U G U U U U = 200
> intloop22 A C A G U A A U = 250
> intloop22 A C A G U A C U = 310
> intloop22 A C A G U A G U = 200
> intloop22 A C A G U A U U = 310
> intloop22 A C A G U C A U = 210
> intloop22 A C A G U C C U = 200
> intloop22 A C A G U C G U = 200
> intloop22 A C A G U C U U = 200
> intloop22 A C A G U G A U = 150
> intloop22 A C A G U G C U = 240
> intloop22 A C A G U G G U = 200
> intloop22 A C A G U G U U = 240
> intloop22 A C A G U U A U = 200
> intloop22 A C A G U U C U = 200
> intloop22 A C A G U U G U = 200
> intloop22 A C A G U U U U = 200
> intloop22 A C C G U A A U = 250
> intloop22 A C C G U A C U = 250
> intloop22 A C C G U A G U = 200
> intloop22 A C C G U A U U = 250
> intloop22 A C C G U C A U = 250
> intloop22 A C C G U C C U = 250
> intloop22 A C C G U C G U = 200
> intloop22 A C C G U C U U = 250
> intloop22 A C C G U G A U = 200
> intloop22 A C C G U G C U = 200
> intloop22 A C C G U G G U = 200
> intloop22 A C C G U G U U = 200
> intloop22 A C C G U U A U = 250
> intloop22 A C C G U U C U = 250
> intloop22 A C C G U U G U = 200
> intloop22 A C C G U U U U = 250
> intloop22 A C G G U A A U = 150
> intloop22 A C G G U A C U = 240
> intloop22 A C G G U A G U = 200
> intloop22 A C G G U A U U = 240
> intloop22 A C G G U C A U = 200
> intloop22 A C G G U C C U = 200
> intloop22 A C G G U C G U = 200
> intloop22 A C G G U C U U = 200
> intloop22 A C G G U G A U = 190
> intloop22 A C G G U G C U = 240
> intloop22 A C G G U G G U = 200
> intloop22 A C G G U G U U = 240
> intloop22 A C G G U U A U = -30
> intloop22 A C G G U U C U = 70
> intloop22 A C G G U U G U = 200
> intloop22 A C G G U U U U = 70
> intloop22 A C U G U A A U = 200
> intloop22 A C U G U A C U = 200
> intloop22 A C U G U A G U = 200
> intloop22 A C U G U A U U = 200
> intloop22 A C U G U C A U = 220
> intloop22 A C U G U C C U = 220
> intloop22 A C U G U C G U = 200
> intloop22 A C U G U C U U = 220
> intloop22 A C U G U G A U = 100
> intloop22 A C U G U G C U = 190
> intloop22 A C U G U G G U = 200
> intloop22 A C U G U G U U = 190
> intloop22 A C U G U U A U = 170
> intloop22 A C U G U U C U = 160
> intloop22 A C U G U U G U = 200
> intloop22 A C U G U U U U = 160
> intloop22 A G A G U A A U = 150
> intloop22 A G A G U A C U = 200
> intloop22 A G A G U A G U = 210
> intloop22 A G A G U A U U = 230
> intloop22 A G A G U C A U = 110
> intloop22 A G A G U C C U = 200
> intloop22 A G A G U C G U = 160
> intloop22 A G A G U C U U = 90
> intloop22 A G A G U G A U = 50
> intloop22 A G A G U G C U = 200
> intloop22 A G A G U G G U = 100
> intloop22 A G A G U G U U = 210
> intloop22 A G A G U U A U = 200
> intloop22 A G A G U U C U = 200
> intloop22 A G A G U U G U = 200
> intloop22 A G A G U U U U = 200
> intloop22 A G C G U A A U = 150
> intloop22 A G C G U A C U = 200
> intloop22 A G C G U A G U = 210
> intloop22 A G C G U A U U = 130
> intloop22 A G C G U C A U = 250
> intloop22 A G C G U C C U = 200
> intloop22 A G C G U C G U = 270
> intloop22 A G C G U C U U = 230
> intloop22 A G C G U G A U = 200
> intloop22 A G C G U G C U = 200
> intloop22 A G C G U G G U = 200
> intloop22 A G C G U G U U = 200
> intloop22 A G C G U U A U = 250
> intloop22 A G C G U U C U = 200
> intloop22 A G C G U U G U = 270
> intloop22 A G C G U U U U = 230
> intloop22 A G G G U A A U = 50
> intloop22 A G G G U A C U = 200
> intloop22 A G G G U A G U = 100
> intloop22 A G G G U A U U = 210
> intloop22 A G G G U C A U = 200
> intloop22 A G G G U C C U = 200
> intloop22 A G G G U C G U = 200
> intloop22 A G G G U C U U = 200
> intloop22 A G G G U G A U = 90
> intloop22 A G G G U G C U = 200
> intloop22 A G G G U G G U = 140
> intloop22 A G G G U G U U = 170
> intloop22 A G G G U U A U = 50
> intloop22 A G G G U U C U = 200
> intloop22 A G G G U U G U = 30
> intloop22 A G G G U U U U = -150
> intloop22 A G U G U A A U = 200
> intloop22 A G U G U A C U = 200
> intloop22 A G U G U A G U = 200
> intloop22 A G U G U A U U = 200
> intloop22 A G U G U C A U = 220
> intloop22 A G U G U C C U = 200
> intloop22 A G U G U C G U = 240
> intloop22 A G U G U C U U = 200
> intloop22 A G U G U G A U = 180
> intloop22 A G U G U G C U = 200
> intloop22 A G U G U G G U = 150
> intloop22 A G U G U G U U = -20
> intloop22 A G U G U U A U = 250
> intloop22 A G U G U U C U = 200
> intloop22 A G U G U U G U = 220
> intloop22 A G U G U U U U = 230
> intloop22 A U A G U A A U = 200
> intloop22 A U A G U A C U = 310
> intloop22 A U A G U A G U = 130
> intloop22 A U A G U A U U = 270
> intloop22 A U A G U C A U = 200
> intloop22 A U A G U C C U = 200
> intloop22 A U A G U C G U = -10
> intloop22 A U A G U C U U = 120
> intloop22 A U A G U G A U = 200
> intloop22 A U A G U G C U = 240
> intloop22 A U A G U G G U = 110
> intloop22 A U A G U G U U = 240
> intloop22 A U A G U U A U = 200
> intloop22 A U A G U U C U = 200
> intloop22 A U A G U U G U = 200
> intloop22 A U A G U U U U = 200
> intloop22 A U C G U A A U = 200
> intloop22 A U C G U A C U = 250
> intloop22 A U C G U A G U = 30
> intloop22 A U C G U A U U = 170
> intloop22 A U C G U C A U = 200
> intloop22 A U C G U C C U = 250
> intloop22 A U C G U C G U = 130
> intloop22 A U C G U C U U = 170
> intloop22 A U C G U G A U = 200
> intloop22 A U C G U G C U = 200
> intloop22 A U C G U G G U = 200
> intloop22 A U C G U G U U = 200
> intloop22 A U C G U U A U = 200
> intloop22 A U C G U U C U = 250
> intloop22 A U C G U U G U = 130
> intloop22 A U C G U U U U = 170
> intloop22 A U G G U A A U = 200
> intloop22 A U G G U A C U = 240
> intloop22 A U G G U A G U = 110
> intloop22 A U G G U A U U = 240
> intloop22 A U G G U C A U = 200
> intloop22 A U G G U C C U = 200
> intloop22 A U G G U C G U = 200
> intloop22 A U G G U C U U = 200
> intloop22 A U G G U G A U = 200
> intloop22 A U G G U G C U = 240
> intloop22 A U G G U G G U = 70
> intloop22 A U G G U G U U = 200
> intloop22 A U G G U U A U = 200
> intloop22 A U G G U U C U = 70
> intloop22 A U G G U U G U = -250
> intloop22 A U G G U U U U = 70
> intloop22 A U U G U A A U = 200
> intloop22 A U U G U A C U = 200
> intloop22 A U U G U A G U = 200
> intloop22 A U U G U A U U = 200
> intloop22 A U U G U C A U = 200
> intloop22 A U U G U C C U = 220
> intloop22 A U U G U C G U = 100
> intloop22 A U U G U C U U = 140
> intloop22 A U U G U G A U = 200
> intloop22 A U U G U G C U = 190
> intloop22 A U U G U G G U = -120
> intloop22 A U U G U G U U = 190
> intloop22 A U U G U U A U = 200
> intloop22 A U U G U U C U = 160
> intloop22 A U U G U U G U = 130
> intloop22 A U U G U U U U = 80
> intloop22 A A A U A A A U = 280
> intloop22 A A A U A A C U = 260
> intloop22 A A A U A A G U = 150
> intloop22 A A A U A A U U = 200
> intloop22 A A A U A C A U = 250
> intloop22 A A A U A C C U = 240
> intloop22 A A A U A C G U = 130
> intloop22 A A A U A C U U = 200
> intloop22 A A A U A G A U = 150
> intloop22 A A A U A G C U = 140
> intloop22 A A A U A G G U = 30
> intloop22 A A A U A G U U = 200
> intloop22 A A A U A U A U = 200
> intloop22 A A A U A U C U = 200
> intloop22 A A A U A U G U = 200
> intloop22 A A A U A U U U = 200
> intloop22 A A C U A A A U = 260
> intloop22 A A C U A A C U = 250
> intloop22 A A C U A A G U = 140
> intloop22 A A C U A A U U = 200
> intloop22 A A C U A C A U = 310
> intloop22 A A C U A C C U = 230
> intloop22 A A C U A C G U = 220
> intloop22 A A C U A C U U = 200
> intloop22 A A C U A G A U = 200
> intloop22 A A C U A G C U = 200
> intloop22 A A C U A G G U = 200
> intloop22 A A C U A G U U = 200
> intloop22 A A C U A U A U = 310
> intloop22 A A C U A U C U = 230
> intloop22 A A C U A U G U = 220
> intloop22 A A C U A U U U = 200
> intloop22 A A G U A A A U = 150
> intloop22 A A G U A A C U = 140
> intloop22 A A G U A A G U = 30
> intloop22 A A G U A A U U = 200
> intloop22 A A G U A C A U = 200
> intloop22 A A G U A C C U = 200
> intloop22 A A G U A C G U = 200
> intloop22 A A G U A C U U = 200
> intloop22 A A G U A G A U = 210
> intloop22 A A G U A G C U = 190
> intloop22 A A G U A G G U = 80
> intloop22 A A G U A G U U = 200
> intloop22 A A G U A U A U = 130
> intloop22 A A G U A U C U = 20
> intloop22 A A G U A U G U = 90
> intloop22 A A G U A U U U = 200
> intloop22 A A U U A A A U = 200
> intloop22 A A U U A A C U = 200
> intloop22 A A U U A A G U = 200
> intloop22 A A U U A A U U = 200
> intloop22 A A U U A C A U = 310
> intloop22 A A U U A C C U = 230
> intloop22 A A U U A C G U = 220
> intloop22 A A U U A C U U = 200
> intloop22 A A U U A G A U = 230
> intloop22 A A U U A G C U = 120
> intloop22 A A U U A G G U = 190
> intloop22 A A U U A G U U = 200
> intloop22 A A U U A U A U = 270
> intloop22 A A U U A U C U = 150
> intloop22 A A U U A U G U = 220
> intloop22 A A U U A U U U = 200
> intloop22 A C A U A A A U = 250
> intloop22 A C A U A A C U = 310
> intloop22 A C A U A A G U = 200
> intloop22 A C A U A A U U = 310
> intloop22 A C A U A C A U = 230
> intloop22 A C A U A C C U = 220
> intloop22 A C A U A C G U = 200
> intloop22 A C A U A C U U = 220
> intloop22 A C A U A G A U = 130
> intloop22 A C A U A G C U = 220
> intloop22 A C A U A G G U = 200
> intloop22 A C A U A G U U = 220
> intloop22 A C A U A U A U = 200
> intloop22 A C A U A U C U = 200
> intloop22 A C A U A U G U = 200
> intloop22 A C A U A U U U = 200
> intloop22 A C C U A A A U = 240
> intloop22 A C C U A A C U = 230
> intloop22 A C C U A A G U = 200
> intloop22 A C C U A A U U = 230
> intloop22 A C C U A C A U = 220
> intloop22 A C C U A C C U = 220
> intloop22 A C C U A C G U = 200
> intloop22 A C C U A C U U = 220
> intloop22 A C C U A G A U = 200
> intloop22 A C C U A G C U = 200
> intloop22 A C C U A G G U = 200
> intloop22 A C C U A G U U = 200
> intloop22 A C C U A U A U = 220
> intloop22 A C C U A U C U = 220
> intloop22 A C C U A U G U = 200
> intloop22 A C C U A U U U = 220
> intloop22 A C G U A A A U = 130
> intloop22 A C G U A A C U = 220
> intloop22 A C G U A A G U = 200
> intloop22 A C G U A A U U = 220
> intloop22 A C G U A C A U = 200
> intloop22 A C G U A C C U = 200
> intloop22 A C G U A C G U = 200
> intloop22 A C G U A C U U = 200
> intloop22 A C G U A G A U = 180
> intloop22 A C G U A G C U = 240
> intloop22 A C G U A G G U = 200
> intloop22 A C G U A G U U = 240
> intloop22 A C G U A U A U = 10
> intloop22 A C G U A U C U = 100
> intloop22 A C G U A U G U = 200
> intloop22 A C G U A U U U = 100
> intloop22 A C U U A A A U = 200
> intloop22 A C U U A A C U = 200
> intloop22 A C U U A A G U = 200
> intloop22 A C U U A A U U = 200
> intloop22 A C U U A C A U = 220
> intloop22 A C U U A C C U = 220
> intloop22 A C U U A C G U = 200
> intloop22 A C U U A C U U = 220
> intloop22 A C U U A G A U = 110
> intloop22 A C U U A G C U = 200
> intloop22 A C U U A G G U = 200
> intloop22 A C U U A G U U = 200
> intloop22 A C U U A U A U = 140
> intloop22 A C U U A U C U = 140
> intloop22 A C U U A U G U = 200
> intloop22 A C U U A U U U = 140
> intloop22 A G A U A A A U = 150
> intloop22 A G A U A A C U = 200
> intloop22 A G A U A A G U = 210
> intloop22 A G A U A A U U = 230
> intloop22 A G A U A C A U = 130
> intloop22 A G A U A C C U = 200
> intloop22 A G A U A C G U = 180
> intloop22 A G A U A C U U = 110
> intloop22 A G A U A G A U = 30
> intloop22 A G A U A G C U = 200
> intloop22 A G A U A G G U = 80
> intloop22 A G A U A G U U = 190
> intloop22 A G A U A U A U = 200
> intloop22 A G A U A U C U = 200
> intloop22 A G A U A U G U = 200
> intloop22 A G A U A U U U = 200
> intloop22 A G C U A A A U = 140
> intloop22 A G C U A A C U = 200
> intloop22 A G C U A A G U = 190
> intloop22 A G C U A A U U = 120
> intloop22 A G C U A C A U = 220
> intloop22 A G C U A C C U = 200
> intloop22 A G C U A C G U = 240
> intloop22 A G C U A C U U = 200
> intloop22 A G C U A G A U = 200
> intloop22 A G C U A G C U = 200
> intloop22 A G C U A G G U = 200
> intloop22 A G C U A G U U = 200
> intloop22 A G C U A U A U = 220
> intloop22 A G C U A U C U = 200
> intloop22 A G C U A U G U = 240
> intloop22 A G C U A U U U = 200
> intloop22 A G G U A A A U = 30
> intloop22 A G G U A A C U = 200
> intloop22 A G G U A A G U = 80
> intloop22 A G G U A A U U = 190
> intloop22 A G G U A C A U = 200
> intloop22 A G G U A C C U = 200
> intloop22 A G G U A C G U = 200
> intloop22 A G G U A C U U = 200
> intloop22 A G G U A G A U = 80
> intloop22 A G G U A G C U = 200
> intloop22 A G G U A G G U = 140
> intloop22 A G G U A G U U = 160
> intloop22 A G G U A U A U = 90
> intloop22 A G G U A U C U = 200
> intloop22 A G G U A U G U = 70
> intloop22 A G G U A U U U = -110
> intloop22 A G U U A A A U = 200
> intloop22 A G U U A A C U = 200
> intloop22 A G U U A A G U = 200
> intloop22 A G U U A A U U = 200
> intloop22 A G U U A C A U = 220
> intloop22 A G U U A C C U = 200
> intloop22 A G U U A C G U = 240
> intloop22 A G U U A C U U = 200
> intloop22 A G U U A G A U = 190
> intloop22 A G U U A G C U = 200
> intloop22 A G U U A G G U = 160
> intloop22 A G U U A G U U = -10
> intloop22 A G U U A U A U = 220
> intloop22 A G U U A U C U = 200
> intloop22 A G U U A U G U = 200
> intloop22 A G U U A U U U = 200
> intloop22 A U A U A A A U = 200
> intloop22 A U A U A A C U = 310
> intloop22 A U A U A A G U = 130
> intloop22 A U A U A A U U = 270
> intloop22 A U A U A C A U = 200
> intloop22 A U A U A C C U = 220
> intloop22 A U A U A C G U = 10
> intloop22 A U A U A C U U = 140
> intloop22 A U A U A G A U = 200
> intloop22 A U A U A G C U = 220
> intloop22 A U A U A G G U = 90
> intloop22 A U A U A G U U = 220
> intloop22 A U A U A U A U = 200
> intloop22 A U A U A U C U = 200
> intloop22 A U A U A U G U = 200
> intloop22 A U A U A U U U = 200
> intloop22 A U C U A A A U = 200
> intloop22 A U C U A A C U = 230
> intloop22 A U C U A A G U = 20
> intloop22 A U C U A A U U = 150
> intloop22 A U C U A C A U = 200
> intloop22 A U C U A C C U = 220
> intloop22 A U C U A C G U = 100
> intloop22 A U C U A C U U = 140
> intloop22 A U C U A G A U = 200
> intloop22 A U C U A G C U = 200
> intloop22 A U C U A G G U = 200
> intloop22 A U C U A G U U = 200
> intloop22 A U C U A U A U = 200
> intloop22 A U C U A U C U = 220
> intloop22 A U C U A U G U = 100
> intloop22 A U C U A U U U = 140
> intloop22 A U G U A A A U = 200
> intloop22 A U G U A A C U = 220
> intloop22 A U G U A A G U = 90
> intloop22 A U G U A A U U = 220
> intloop22 A U G U A C A U = 200
> intloop22 A U G U A C C U = 200
> intloop22 A U G U A C G U = 200
> intloop22 A U G U A C U U = 200
> intloop22 A U G U A G A U = 200
> intloop22 A U G U A G C U = 240
> intloop22 A U G U A G G U = 70
> intloop22 A U G U A G U U = 200
> intloop22 A U G U A U A U = 200
> intloop22 A U G U A U C U = 100
> intloop22 A U G U A U G U = -210
> intloop22 A U G U A U U U = 110
> intloop22 A U U U A A A U = 200
> intloop22 A U U U A A C U = 200
> intloop22 A U U U A A G U = 200
> intloop22 A U U U A A U U = 200
> intloop22 A U U U A C A U = 200
> intloop22 A U U U A C C U = 220
> intloop22 A U U U A C G U = 100
> intloop22 A U U U A C U U = 140
> intloop22 A U U U A G A U = 200
> intloop22 A U U U A G C U = 200
> intloop22 A U U U A G G U = -110
> intloop22 A U U U A G U U = 200
> intloop22 A U U U A U A U = 200
> intloop22 A U U U A U C U = 140
> intloop22 A U U U A U G U = 110
> intloop22 A U U U A U U U = 60
> intloop22 A A A A U A A U = 280
> intloop22 A A A A U A C U = 260
> intloop22 A A A A U A G U = 150
> intloop22 A A A A U A U U = 200
> intloop22 A A A A U C A U = 230
> intloop22 A A A A U C C U = 220
> intloop22 A A A A U C G U = 110
> intloop22 A A A A U C U U = 200
> intloop22 A A A A U G A U = 170
> intloop22 A A A A U G C U = 160
> intloop22 A A A A U G G U = 50
> intloop22 A A A A U G U U = 200
> intloop22 A A A A U U A U = 200
> intloop22 A A A A U U C U = 200
> intloop22 A A A A U U G U = 200
> intloop22 A A A A U U U U = 200
> intloop22 A A C A U A A U = 280
> intloop22 A A C A U A C U = 260
> intloop22 A A C A U A G U = 150
> intloop22 A A C A U A U U = 200
> intloop22 A A C A U C A U = 340
> intloop22 A A C A U C C U = 260
> intloop22 A A C A U C G U = 250
> intloop22 A A C A U C U U = 200
> intloop22 A A C A U G A U = 200
> intloop22 A A C A U G C U = 200
> intloop22 A A C A U G G U = 200
> intloop22 A A C A U G U U = 200
> intloop22 A A C A U U A U = 340
> intloop22 A A C A U U C U = 260
> intloop22 A A C A U U G U = 250
> intloop22 A A C A U U U U = 200
> intloop22 A A G A U A A U = 170
> intloop22 A A G A U A C U = 160
> intloop22 A A G A U A G U = 50
> intloop22 A A G A U A U U = 200
> intloop22 A A G A U C A U = 200
> intloop22 A A G A U C C U = 200
> intloop22 A A G A U C G U = 200
> intloop22 A A G A U C U U = 200
> intloop22 A A G A U G A U = 210
> intloop22 A A G A U G C U = 200
> intloop22 A A G A U G G U = 90
> intloop22 A A G A U G U U = 200
> intloop22 A A G A U U A U = 100
> intloop22 A A G A U U C U = -20
> intloop22 A A G A U U G U = 50
> intloop22 A A G A U U U U = 200
> intloop22 A A U A U A A U = 200
> intloop22 A A U A U A C U = 200
> intloop22 A A U A U A G U = 200
> intloop22 A A U A U A U U = 200
> intloop22 A A U A U C A U = 310
> intloop22 A A U A U C C U = 230
> intloop22 A A U A U C G U = 220
> intloop22 A A U A U C U U = 200
> intloop22 A A U A U G A U = 220
> intloop22 A A U A U G C U = 110
> intloop22 A A U A U G G U = 180
> intloop22 A A U A U G U U = 200
> intloop22 A A U A U U A U = 290
> intloop22 A A U A U U C U = 180
> intloop22 A A U A U U G U = 250
> intloop22 A A U A U U U U = 200
> intloop22 A C A A U A A U = 250
> intloop22 A C A A U A C U = 310
> intloop22 A C A A U A G U = 200
> intloop22 A C A A U A U U = 310
> intloop22 A C A A U C A U = 210
> intloop22 A C A A U C C U = 200
> intloop22 A C A A U C G U = 200
> intloop22 A C A A U C U U = 200
> intloop22 A C A A U G A U = 150
> intloop22 A C A A U G C U = 240
> intloop22 A C A A U G G U = 200
> intloop22 A C A A U G U U = 240
> intloop22 A C A A U U A U = 200
> intloop22 A C A A U U C U = 200
> intloop22 A C A A U U G U = 200
> intloop22 A C A A U U U U = 200
> intloop22 A C C A U A A U = 250
> intloop22 A C C A U A C U = 250
> intloop22 A C C A U A G U = 200
> intloop22 A C C A U A U U = 250
> intloop22 A C C A U C A U = 250
> intloop22 A C C A U C C U = 250
> intloop22 A C C A U C G U = 200
> intloop22 A C C A U C U U = 250
> intloop22 A C C A U G A U = 200
> intloop22 A C C A U G C U = 200
> intloop22 A C C A U G G U = 200
> intloop22 A C C A U G U U = 200
> intloop22 A C C A U U A U = 250
> intloop22 A C C A U U C U = 250
> intloop22 A C C A U U G U = 200
> intloop22 A C C A U U U U = 250
> intloop22 A C G A U A A U = 150
> intloop22 A C G A U A C U = 240
> intloop22 A C G A U A G U = 200
> intloop22 A C G A U A U U = 240
> intloop22 A C G A U C A U = 200
> intloop22 A C G A U C C U = 200
> intloop22 A C G A U C G U = 200
> intloop22 A C G A U C U U = 200
> intloop22 A C G A U G A U = 190
> intloop22 A C G A U G C U = 240
> intloop22 A C G A U G G U = 200
> intloop22 A C G A U G U U = 240
> intloop22 A C G A U U A U = -30
> intloop22 A C G A U U C U = 70
> intloop22 A C G A U U G U = 200
> intloop22 A C G A U U U U = 70
> intloop22 A C U A U A A U = 200
> intloop22 A C U A U A C U = 200
> intloop22 A C U A U A G U = 200
> intloop22 A C U A U A U U = 200
> intloop22 A C U A U C A U = 220
> intloop22 A C U A U C C U = 220
> intloop22 A C U A U C G U = 200
> intloop22 A C U A U C U U = 220
> intloop22 A C U A U G A U = 100
> intloop22 A C U A U G C U = 190
> intloop22 A C U A U G G U = 200
> intloop22 A C U A U G U U = 190
> intloop22 A C U A U U A U = 170
> intloop22 A C U A U U C U = 160
> intloop22 A C U A U U G U = 200
> intloop22 A C U A U U U U = 160
> intloop22 A G A A U A A U = 150
> intloop22 A G A A U A C U = 200
> intloop22 A G A A U A G U = 210
> intloop22 A G A A U A U U = 230
> intloop22 A G A A U C A U = 110
> intloop22 A G A A U C C U = 200
> intloop22 A G A A U C G U = 160
> intloop22 A G A A U C U U = 90
> intloop22 A G A A U G A U = 50
> intloop22 A G A A U G C U = 200
> intloop22 A G A A U G G U = 100
> intloop22 A G A A U G U U = 210
> intloop22 A G A A U U A U = 200
> intloop22 A G A A U U C U = 200
> intloop22 A G A A U U G U = 200
> intloop22 A G A A U U U U = 200
> intloop22 A G C A U A A U = 150
> intloop22 A G C A U A C U = 200
> intloop22 A G C A U A G U = 210
> intloop22 A G C A U A U U = 130
> intloop22 A G C A U C A U = 250
> intloop22 A G C A U C C U = 200
> intloop22 A G C A U C G U = 270
> intloop22 A G C A U C U U = 230
> intloop22 A G C A U G A U = 200
> intloop22 A G C A U G C U = 200
> intloop22 A G C A U G G U = 200
> intloop22 A G C A U G U U = 200
> intloop22 A G C A U U A U = 250
> intloop22 A G C A U U C U = 200
> intloop22 A G C A U U G U = 270
> intloop22 A G C A U U U U = 230
> intloop22 A G G A U A A U = 50
> intloop22 A G G A U A C U = 200
> intloop22 A G G A U A G U = 100
> intloop22 A G G A U A U U = 210
> intloop22 A G G A U C A U = 200
> intloop22 A G G A U C C U = 200
> intloop22 A G G A U C G U = 200
> intloop22 A G G A U C U U = 200
> intloop22 A G G A U G A U = 90
> intloop22 A G G A U G C U = 200
> intloop22 A G G A U G G U = 140
> intloop22 A G G A U G U U = 170
> intloop22 A G G A U U A U = 50
> intloop22 A G G A U U C U = 200
> intloop22 A G G A U U G U = 30
> intloop22 A G G A U U U U = -150
> intloop22 A G U A U A A U = 200
> intloop22 A G U A U A C U = 200
> intloop22 A G U A U A G U = 200
> intloop22 A G U A U A U U = 200
> intloop22 A G U A U C A U = 220
> intloop22 A G U A U C C U = 200
> intloop22 A G U A U C G U = 240
> intloop22 A G U A U C U U = 200
> intloop22 A G U A U G A U = 180
> intloop22 A G U A U G C U = 200
> intloop22 A G U A U G G U = 150
> intloop22 A G U A U G U U = -20
> intloop22 A G U A U U A U = 250
> intloop22 A G U A U U C U = 200
> intloop22 A G U A U U G U = 220
> intloop22 A G U A U U U U = 230
> intloop22 A U A A U A A U = 200
> intloop22 A U A A U A C U = 310
> intloop22 A U A A U A G U = 130
> intloop22 A U A A U A U U = 270
> intloop22 A U A A U C A U = 200
> intloop22 A U A A U C C U = 200
> intloop22 A U A A U C G U = -10
> intloop22 A U A A U C U U = 120
> intloop22 A U A A U G A U = 200
> intloop22 A U A A U G C U = 240
> intloop22 A U A A U G G U = 110
> intloop22 A U A A U G U U = 240
> intloop22 A U A A U U A U = 200
> intloop22 A U A A U U C U = 200
> intloop22 A U A A U U G U = 200
> intloop22 A U A A U U U U = 200
> intloop22 A U C A U A A U = 200
> intloop22 A U C A U A C U = 250
> intloop22 A U C A U A G U = 30
> intloop22 A U C A U A U U = 170
> intloop22 A U C A U C A U = 200
> intloop22 A U C A U C C U = 250
> intloop22 A U C A U C G U = 130
> intloop22 A U C A U C U U = 170
> intloop22 A U C A U G A U = 200
> intloop22 A U C A U G C U = 200
> intloop22 A U C A U G G U = 200
> intloop22 A U C A U G U U = 200
> intloop22 A U C A U U A U = 200
> intloop22 A U C A U U C U = 250
> intloop22 A U C A U U G U = 130
> intloop22 A U C A U U U U = 170
> intloop22 A U G A U A A U = 200
> intloop22 A U G A U A C U = 240
> intloop22 A U G A U A G U = 110
> intloop22 A U G A U A U U = 240
> intloop22 A U G A U C A U = 200
> intloop22 A U G A U C C U = 200
> intloop22 A U G A U C G U = 200
> intloop22 A U G A U C U U = 200
> intloop22 A U G A U G A U = 200
> intloop22 A U G A U G C U = 240
> intloop22 A U G A U G G U = 70
> intloop22 A U G A U G U U = 200
> intloop22 A U G A U U A U = 200
> intloop22 A U G A U U C U = 70
> intloop22 A U G A U U G U = -250
> intloop22 A U G A U U U U = 70
> intloop22 A U U A U A A U = 200
> intloop22 A U U A U A C U = 200
> intloop22 A U U A U A G U = 200
> intloop22 A U U A U A U U = 200
> intloop22 A U U A U C A U = 200
> intloop22 A U U A U C C U = 220
> intloop22 A U U A U C G U = 100
> intloop22 A U U A U C U U = 140
> intloop22 A U U A U G A U = 200
> intloop22 A U U A U G C U = 190
> intloop22 A U U A U G G U = -120
> intloop22 A U U A U G U U = 190
> intloop22 A U U A U U A U = 200
> intloop22 A U U A U U C U = 160
> intloop22 A U U A U U G U = 130
> intloop22 A U U A U U U U = 80
> intloop22 U A A G C A A A = 200
> intloop22 U A A G C A C A = 200
> intloop22 U A A G C A G A = 100
> intloop22 U A A G C A U A = 200
> intloop22 U A A G C C A A = 190
> intloop22 U A A G C C C A = 190
> intloop22 U A A G C C G A = 90
> intloop22 U A A G C C U A = 200
> intloop22 U A A G C G A A = 100
> intloop22 U A A G C G C A = 100
> intloop22 U A A G C G G A = 0
> intloop22 U A A G C G U A = 200
> intloop22 U A A G C U A A = 200
> intloop22 U A A G C U C A = 200
> intloop22 U A A G C U G A = 200
> intloop22 U A A G C U U A = 200
> intloop22 U A C G C A A A = 240
> intloop22 U A C G C A C A = 240
> intloop22 U A C G C A G A = 130
> intloop22 U A C G C A U A = 200
> intloop22 U A C G C C A A = 280
> intloop22 U A C G C C C A = 220
> intloop22 U A C G C C G A = 220
> intloop22 U A C G C C U A = 200
> intloop22 U A C G C G A A = 200
> intloop22 U A C G C G C A = 200
> intloop22 U A C G C G G A = 200
> intloop22 U A C G C G U A = 200
> intloop22 U A C G C U A A = 270
> intloop22 U A C G C U C A = 210
> intloop22 U A C G C U G A = 200
> intloop22 U A C G C U U A = 200
> intloop22 U A G G C A A A = 100
> intloop22 U A G G C A C A = 100
> intloop22 U A G G C A G A = 0
> intloop22 U A G G C A U A = 200
> intloop22 U A G G C C A A = 200
> intloop22 U A G G C C C A = 200
> intloop22 U A G G C C G A = 200
> intloop22 U A G G C C U A = 200
> intloop22 U A G G C G A A = 180
> intloop22 U A G G C G C A = 180
> intloop22 U A G G C G G A = 70
> intloop22 U A G G C G U A = 200
> intloop22 U A G G C U A A = 30
> intloop22 U A G G C U C A = -70
> intloop22 U A G G C U G A = 10
> intloop22 U A G G C U U A = 200
> intloop22 U A U G C A A A = 200
> intloop22 U A U G C A C A = 200
> intloop22 U A U G C A G A = 200
> intloop22 U A U G C A U A = 200
> intloop22 U A U G C C A A = 270
> intloop22 U A U G C C C A = 210
> intloop22 U A U G C C G A = 200
> intloop22 U A U G C C U A = 200
> intloop22 U A U G C G A A = 180
> intloop22 U A U G C G C A = 80
> intloop22 U A U G C G G A = 160
> intloop22 U A U G C G U A = 200
> intloop22 U A U G C U A A = 220
> intloop22 U A U G C U C A = 120
> intloop22 U A U G C U G A = 190
> intloop22 U A U G C U U A = 200
> intloop22 U C A G C A A A = 160
> intloop22 U C A G C A C A = 260
> intloop22 U C A G C A G A = 200
> intloop22 U C A G C A U A = 230
> intloop22 U C A G C C A A = 150
> intloop22 U C A G C C C A = 190
> intloop22 U C A G C C G A = 200
> intloop22 U C A G C C U A = 160
> intloop22 U C A G C G A A = 60
> intloop22 U C A G C G C A = 200
> intloop22 U C A G C G G A = 200
> intloop22 U C A G C G U A = 170
> intloop22 U C A G C U A A = 200
> intloop22 U C A G C U C A = 200
> intloop22 U C A G C U G A = 200
> intloop22 U C A G C U U A = 200
> intloop22 U C C G C A A A = 190
> intloop22 U C C G C A C A = 240
> intloop22 U C C G C A G A = 200
> intloop22 U C C G C A U A = 210
> intloop22 U C C G C C A A = 180
> intloop22 U C C G C C C A = 220
> intloop22 U C C G C C G A = 200
> intloop22 U C C G C C U A = 190
> intloop22 U C C G C G A A = 200
> intloop22 U C C G C G C A = 200
> intloop22 U C C G C G G A = 200
> intloop22 U C C G C G U A = 200
> intloop22 U C C G C U A A = 160
> intloop22 U C C G C U C A = 210
> intloop22 U C C G C U G A = 200
> intloop22 U C C G C U U A = 180
> intloop22 U C G G C A A A = 60
> intloop22 U C G G C A C A = 200
> intloop22 U C G G C A G A = 200
> intloop22 U C G G C A U A = 170
> intloop22 U C G G C C A A = 200
> intloop22 U C G G C C C A = 200
> intloop22 U C G G C C G A = 200
> intloop22 U C G G C C U A = 200
> intloop22 U C G G C G A A = 130
> intloop22 U C G G C G C A = 240
> intloop22 U C G G C G G A = 200
> intloop22 U C G G C G U A = 210
> intloop22 U C G G C U A A = -110
> intloop22 U C G G C U C A = 30
> intloop22 U C G G C U G A = 200
> intloop22 U C G G C U U A = 0
> intloop22 U C U G C A A A = 200
> intloop22 U C U G C A C A = 200
> intloop22 U C U G C A G A = 200
> intloop22 U C U G C A U A = 200
> intloop22 U C U G C C A A = 160
> intloop22 U C U G C C C A = 210
> intloop22 U C U G C C G A = 200
> intloop22 U C U G C C U A = 180
> intloop22 U C U G C G A A = 40
> intloop22 U C U G C G C A = 180
> intloop22 U C U G C G G A = 200
> intloop22 U C U G C G U A = 150
> intloop22 U C U G C U A A = 70
> intloop22 U C U G C U C A = 120
> intloop22 U C U G C U G A = 200
> intloop22 U C U G C U U A = 90
> intloop22 U G A G C A A A = 100
> intloop22 U G A G C A C A = 200
> intloop22 U G A G C A G A = 140
> intloop22 U G A G C A U A = 150
> intloop22 U G A G C C A A = 90
> intloop22 U G A G C C C A = 200
> intloop22 U G A G C C G A = 130
> intloop22 U G A G C C U A = 40
> intloop22 U G A G C G A A = 0
> intloop22 U G A G C G C A = 200
> intloop22 U G A G C G G A = 40
> intloop22 U G A G C G U A = 130
> intloop22 U G A G C U A A = 200
> intloop22 U G A G C U C A = 200
> intloop22 U G A G C U G A = 200
> intloop22 U G A G C U U A = 200
> intloop22 U G C G C A A A = 130
> intloop22 U G C G C A C A = 200
> intloop22 U G C G C A G A = 170
> intloop22 U G C G C A U A = 80
> intloop22 U G C G C C A A = 220
> intloop22 U G C G C C C A = 200
> intloop22 U G C G C C G A = 220
> intloop22 U G C G C C U A = 170
> intloop22 U G C G C G A A = 200
> intloop22 U G C G C G C A = 200
> intloop22 U G C G C G G A = 200
> intloop22 U G C G C G U A = 200
> intloop22 U G C G C U A A = 200
> intloop22 U G C G C U C A = 200
> intloop22 U G C G C U G A = 200
> intloop22 U G C G C U U A = 150
> intloop22 U G G G C A A A = 0
> intloop22 U G G G C A C A = 200
> intloop22 U G G G C A G A = 40
> intloop22 U G G G C A U A = 130
> intloop22 U G G G C C A A = 200
> intloop22 U G G G C C C A = 200
> intloop22 U G G G C C G A = 200
> intloop22 U G G G C C U A = 200
> intloop22 U G G G C G A A = 70
> intloop22 U G G G C G C A = 200
> intloop22 U G G G C G G A = 110
> intloop22 U G G G C G U A = 120
> intloop22 U G G G C U A A = 10
> intloop22 U G G G C U C A = 200
> intloop22 U G G G C U G A = -30
> intloop22 U G G G C U U A = -220
> intloop22 U G U G C A A A = 200
> intloop22 U G U G C A C A = 200
> intloop22 U G U G C A G A = 200
> intloop22 U G U G C A U A = 200
> intloop22 U G U G C C A A = 200
> intloop22 U G U G C C C A = 200
> intloop22 U G U G C C G A = 200
> intloop22 U G U G C C U A = 150
> intloop22 U G U G C G A A = 160
> intloop22 U G U G C G C A = 200
> intloop22 U G U G C G G A = 120
> intloop22 U G U G C G U A = -70
> intloop22 U G U G C U A A = 190
> intloop22 U G U G C U C A = 200
> intloop22 U G U G C U G A = 150
> intloop22 U G U G C U U A = 150
> intloop22 U U A G C A A A = 200
> intloop22 U U A G C A C A = 260
> intloop22 U U A G C A G A = 20
> intloop22 U U A G C A U A = 220
> intloop22 U U A G C C A A = 200
> intloop22 U U A G C C C A = 190
> intloop22 U U A G C C G A = -90
> intloop22 U U A G C C U A = 110
> intloop22 U U A G C G A A = 200
> intloop22 U U A G C G C A = 200
> intloop22 U U A G C G G A = 0
> intloop22 U U A G C G U A = 200
> intloop22 U U A G C U A A = 200
> intloop22 U U A G C U C A = 200
> intloop22 U U A G C U G A = 200
> intloop22 U U A G C U U A = 200
> intloop22 U U C G C A A A = 200
> intloop22 U U C G C A C A = 240
> intloop22 U U C G C A G A = -40
> intloop22 U U C G C A U A = 150
> intloop22 U U C G C C A A = 200
> intloop22 U U C G C C C A = 220
> intloop22 U U C G C C G A = 40
> intloop22 U U C G C C U A = 140
> intloop22 U U C G C G A A = 200
> intloop22 U U C G C G C A = 200
> intloop22 U U C G C G G A = 200
> intloop22 U U C G C G U A = 200
> intloop22 U U C G C U A A = 200
> intloop22 U U C G C U C A = 210
> intloop22 U U C G C U G A = 30
> intloop22 U U C G C U U A = 120
> intloop22 U U G G C A A A = 200
> intloop22 U U G G C A C A = 200
> intloop22 U U G G C A G A = 0
> intloop22 U U G G C A U A = 200
> intloop22 U U G G C C A A = 200
> intloop22 U U G G C C C A = 200
> intloop22 U U G G C C G A = 200
> intloop22 U U G G C C U A = 200
> intloop22 U U G G C G A A = 200
> intloop22 U U G G C G C A = 240
> intloop22 U U G G C G G A = 0
> intloop22 U U G G C G U A = 190
> intloop22 U U G G C U A A = 200
> intloop22 U U G G C U C A = 30
> intloop22 U U G G C U G A = -350
> intloop22 U U G G C U U A = 30
> intloop22 U U U G C A A A = 200
> intloop22 U U U G C A C A = 200
> intloop22 U U U G C A G A = 200
> intloop22 U U U G C A U A = 200
> intloop22 U U U G C C A A = 200
> intloop22 U U U G C C C A = 210
> intloop22 U U U G C C G A = 30
> intloop22 U U U G C C U A = 120
> intloop22 U U U G C G A A = 200
> intloop22 U U U G C G C A = 180
> intloop22 U U U G C G G A = -200
> intloop22 U U U G C G U A = 180
> intloop22 U U U G C U A A = 200
> intloop22 U U U G C U C A = 120
> intloop22 U U U G C U G A = 20
> intloop22 U U U G C U U A = 30
> intloop22 U A A C G A A A = 210
> intloop22 U A A C G A C A = 210
> intloop22 U A A C G A G A = 110
> intloop22 U A A C G A U A = 200
> intloop22 U A A C G C A A = 190
> intloop22 U A A C G C C A = 190
> intloop22 U A A C G C G A = 80
> intloop22 U A A C G C U A = 200
> intloop22 U A A C G G A A = 10
> intloop22 U A A C G G C A = 10
> intloop22 U A A C G G G A = -90
> intloop22 U A A C G G U A = 200
> intloop22 U A A C G U A A = 200
> intloop22 U A A C G U C A = 200
> intloop22 U A A C G U G A = 200
> intloop22 U A A C G U U A = 200
> intloop22 U A C C G A A A = 180
> intloop22 U A C C G A C A = 180
> intloop22 U A C C G A G A = 80
> intloop22 U A C C G A U A = 200
> intloop22 U A C C G C A A = 250
> intloop22 U A C C G C C A = 190
> intloop22 U A C C G C G A = 180
> intloop22 U A C C G C U A = 200
> intloop22 U A C C G G A A = 200
> intloop22 U A C C G G C A = 200
> intloop22 U A C C G G G A = 200
> intloop22 U A C C G G U A = 200
> intloop22 U A C C G U A A = 150
> intloop22 U A C C G U C A = 90
> intloop22 U A C C G U G A = 90
> intloop22 U A C C G U U A = 200
> intloop22 U A G C G A A A = 70
> intloop22 U A G C G A C A = 70
> intloop22 U A G C G A G A = -30
> intloop22 U A G C G A U A = 200
> intloop22 U A G C G C A A = 200
> intloop22 U A G C G C C A = 200
> intloop22 U A G C G C G A = 200
> intloop22 U A G C G C U A = 200
> intloop22 U A G C G G A A = 180
> intloop22 U A G C G G C A = 180
> intloop22 U A G C G G G A = 70
> intloop22 U A G C G G U A = 200
> intloop22 U A G C G U A A = 0
> intloop22 U A G C G U C A = -100
> intloop22 U A G C G U G A = -30
> intloop22 U A G C G U U A = 200
> intloop22 U A U C G A A A = 200
> intloop22 U A U C G A C A = 200
> intloop22 U A U C G A G A = 200
> intloop22 U A U C G A U A = 200
> intloop22 U A U C G C A A = 250
> intloop22 U A U C G C C A = 190
> intloop22 U A U C G C G A = 190
> intloop22 U A U C G C U A = 200
> intloop22 U A U C G G A A = 40
> intloop22 U A U C G G C A = -60
> intloop22 U A U C G G G A = 10
> intloop22 U A U C G G U A = 200
> intloop22 U A U C G U A A = 210
> intloop22 U A U C G U C A = 110
> intloop22 U A U C G U G A = 190
> intloop22 U A U C G U U A = 200
> intloop22 U C A C G A A A = 170
> intloop22 U C A C G A C A = 270
> intloop22 U C A C G A G A = 200
> intloop22 U C A C G A U A = 240
> intloop22 U C A C G C A A = 140
> intloop22 U C A C G C C A = 190
> intloop22 U C A C G C G A = 200
> intloop22 U C A C G C U A = 160
> intloop22 U C A C G G A A = -30
> intloop22 U C A C G G C A = 110
> intloop22 U C A C G G G A = 200
> intloop22 U C A C G G U A = 80
> intloop22 U C A C G U A A = 200
> intloop22 U C A C G U C A = 200
> intloop22 U C A C G U G A = 200
> intloop22 U C A C G U U A = 200
> intloop22 U C C C G A A A = 140
> intloop22 U C C C G A C A = 180
> intloop22 U C C C G A G A = 200
> intloop22 U C C C G A U A = 150
> intloop22 U C C C G C A A = 140
> intloop22 U C C C G C C A = 190
> intloop22 U C C C G C G A = 200
> intloop22 U C C C G C U A = 160
> intloop22 U C C C G G A A = 200
> intloop22 U C C C G G C A = 200
> intloop22 U C C C G G G A = 200
> intloop22 U C C C G G U A = 200
> intloop22 U C C C G U A A = 40
> intloop22 U C C C G U C A = 90
> intloop22 U C C C G U G A = 200
> intloop22 U C C C G U U A = 60
> intloop22 U C G C G A A A = 30
> intloop22 U C G C G A C A = 170
> intloop22 U C G C G A G A = 200
> intloop22 U C G C G A U A = 140
> intloop22 U C G C G C A A = 200
> intloop22 U C G C G C C A = 200
> intloop22 U C G C G C G A = 200
> intloop22 U C G C G C U A = 200
> intloop22 U C G C G G A A = 130
> intloop22 U C G C G G C A = 240
> intloop22 U C G C G G G A = 200
> intloop22 U C G C G G U A = 210
> intloop22 U C G C G U A A = -150
> intloop22 U C G C G U C A = 0
> intloop22 U C G C G U G A = 200
> intloop22 U C G C G U U A = -30
> intloop22 U C U C G A A A = 200
> intloop22 U C U C G A C A = 200
> intloop22 U C U C G A G A = 200
> intloop22 U C U C G A U A = 200
> intloop22 U C U C G C A A = 150
> intloop22 U C U C G C C A = 190
> intloop22 U C U C G C G A = 200
> intloop22 U C U C G C U A = 160
> intloop22 U C U C G G A A = -110
> intloop22 U C U C G G C A = 40
> intloop22 U C U C G G G A = 200
> intloop22 U C U C G G U A = 10
> intloop22 U C U C G U A A = 70
> intloop22 U C U C G U C A = 110
> intloop22 U C U C G U G A = 200
> intloop22 U C U C G U U A = 80
> intloop22 U G A C G A A A = 110
> intloop22 U G A C G A C A = 200
> intloop22 U G A C G A G A = 150
> intloop22 U G A C G A U A = 160
> intloop22 U G A C G C A A = 80
> intloop22 U G A C G C C A = 200
> intloop22 U G A C G C G A = 120
> intloop22 U G A C G C U A = 30
> intloop22 U G A C G G A A = -90
> intloop22 U G A C G G C A = 200
> intloop22 U G A C G G G A = -50
> intloop22 U G A C G G U A = 40
> intloop22 U G A C G U A A = 200
> intloop22 U G A C G U C A = 200
> intloop22 U G A C G U G A = 200
> intloop22 U G A C G U U A = 200
> intloop22 U G C C G A A A = 80
> intloop22 U G C C G A C A = 200
> intloop22 U G C C G A G A = 120
> intloop22 U G C C G A U A = 30
> intloop22 U G C C G C A A = 180
> intloop22 U G C C G C C A = 200
> intloop22 U G C C G C G A = 180
> intloop22 U G C C G C U A = 130
> intloop22 U G C C G G A A = 200
> intloop22 U G C C G G C A = 200
> intloop22 U G C C G G G A = 200
> intloop22 U G C C G G U A = 200
> intloop22 U G C C G U A A = 90
> intloop22 U G C C G U C A = 200
> intloop22 U G C C G U G A = 80
> intloop22 U G C C G U U A = 40
> intloop22 U G G C G A A A = -30
> intloop22 U G G C G A C A = 200
> intloop22 U G G C G A G A = 10
> intloop22 U G G C G A U A = 100
> intloop22 U G G C G C A A = 200
> intloop22 U G G C G C C A = 200
> intloop22 U G G C G C G A = 200
> intloop22 U G G C G C U A = 200
> intloop22 U G G C G G A A = 70
> intloop22 U G G C G G C A = 200
> intloop22 U G G C G G G A = 110
> intloop22 U G G C G G U A = 120
> intloop22 U G G C G U A A = -30
> intloop22 U G G C G U C A = 200
> intloop22 U G G C G U G A = -70
> intloop22 U G G C G U U A = -260
> intloop22 U G U C G A A A = 200
> intloop22 U G U C G A C A = 200
> intloop22 U G U C G A G A = 200
> intloop22 U G U C G A U A = 200
> intloop22 U G U C G C A A = 190
> intloop22 U G U C G C C A = 200
> intloop22 U G U C G C G A = 190
> intloop22 U G U C G C U A = 140
> intloop22 U G U C G G A A = 10
> intloop22 U G U C G G C A = 200
> intloop22 U G U C G G G A = -30
> intloop22 U G U C G G U A = -220
> intloop22 U G U C G U A A = 190
> intloop22 U G U C G U C A = 200
> intloop22 U G U C G U G A = 150
> intloop22 U G U C G U U A = 140
> intloop22 U U A C G A A A = 200
> intloop22 U U A C G A C A = 270
> intloop22 U U A C G A G A = 30
> intloop22 U U A C G A U A = 230
> intloop22 U U A C G C A A = 200
> intloop22 U U A C G C C A = 190
> intloop22 U U A C G C G A = -90
> intloop22 U U A C G C U A = 100
> intloop22 U U A C G G A A = 200
> intloop22 U U A C G G C A = 110
> intloop22 U U A C G G G A = -90
> intloop22 U U A C G G U A = 110
> intloop22 U U A C G U A A = 200
> intloop22 U U A C G U C A = 200
> intloop22 U U A C G U G A = 200
> intloop22 U U A C G U U A = 200
> intloop22 U U C C G A A A = 200
> intloop22 U U C C G A C A = 180
> intloop22 U U C C G A G A = -100
> intloop22 U U C C G A U A = 100
> intloop22 U U C C G C A A = 200
> intloop22 U U C C G C C A = 190
> intloop22 U U C C G C G A = 10
> intloop22 U U C C G C U A = 100
> intloop22 U U C C G G A A = 200
> intloop22 U U C C G G C A = 200
> intloop22 U U C C G G G A = 200
> intloop22 U U C C G G U A = 200
> intloop22 U U C C G U A A = 200
> intloop22 U U C C G U C A = 90
> intloop22 U U C C G U G A = -90
> intloop22 U U C C G U U A = 0
> intloop22 U U G C G A A A = 200
> intloop22 U U G C G A C A = 170
> intloop22 U U G C G A G A = -30
> intloop22 U U G C G A U A = 170
> intloop22 U U G C G C A A = 200
> intloop22 U U G C G C C A = 200
> intloop22 U U G C G C G A = 200
> intloop22 U U G C G C U A = 200
> intloop22 U U G C G G A A = 200
> intloop22 U U G C G G C A = 240
> intloop22 U U G C G G G A = 0
> intloop22 U U G C G G U A = 190
> intloop22 U U G C G U A A = 200
> intloop22 U U G C G U C A = 0
> intloop22 U U G C G U G A = -390
> intloop22 U U G C G U U A = -10
> intloop22 U U U C G A A A = 200
> intloop22 U U U C G A C A = 200
> intloop22 U U U C G A G A = 200
> intloop22 U U U C G A U A = 200
> intloop22 U U U C G C A A = 200
> intloop22 U U U C G C C A = 190
> intloop22 U U U C G C G A = 10
> intloop22 U U U C G C U A = 110
> intloop22 U U U C G G A A = 200
> intloop22 U U U C G G C A = 40
> intloop22 U U U C G G G A = -350
> intloop22 U U U C G G U A = 30
> intloop22 U U U C G U A A = 200
> intloop22 U U U C G U C A = 110
> intloop22 U U U C G U G A = 10
> intloop22 U U U C G U U A = 30
> intloop22 U A A U G A A A = 280
> intloop22 U A A U G A C A = 280
> intloop22 U A A U G A G A = 170
> intloop22 U A A U G A U A = 200
> intloop22 U A A U G C A A = 250
> intloop22 U A A U G C C A = 250
> intloop22 U A A U G C G A = 150
> intloop22 U A A U G C U A = 200
> intloop22 U A A U G G A A = 150
> intloop22 U A A U G G C A = 150
> intloop22 U A A U G G G A = 50
> intloop22 U A A U G G U A = 200
> intloop22 U A A U G U A A = 200
> intloop22 U A A U G U C A = 200
> intloop22 U A A U G U G A = 200
> intloop22 U A A U G U U A = 200
> intloop22 U A C U G A A A = 260
> intloop22 U A C U G A C A = 260
> intloop22 U A C U G A G A = 160
> intloop22 U A C U G A U A = 200
> intloop22 U A C U G C A A = 310
> intloop22 U A C U G C C A = 250
> intloop22 U A C U G C G A = 240
> intloop22 U A C U G C U A = 200
> intloop22 U A C U G G A A = 200
> intloop22 U A C U G G C A = 200
> intloop22 U A C U G G G A = 200
> intloop22 U A C U G G U A = 200
> intloop22 U A C U G U A A = 310
> intloop22 U A C U G U C A = 250
> intloop22 U A C U G U G A = 240
> intloop22 U A C U G U U A = 200
> intloop22 U A G U G A A A = 150
> intloop22 U A G U G A C A = 150
> intloop22 U A G U G A G A = 50
> intloop22 U A G U G A U A = 200
> intloop22 U A G U G C A A = 200
> intloop22 U A G U G C C A = 200
> intloop22 U A G U G C G A = 200
> intloop22 U A G U G C U A = 200
> intloop22 U A G U G G A A = 210
> intloop22 U A G U G G C A = 210
> intloop22 U A G U G G G A = 100
> intloop22 U A G U G G U A = 200
> intloop22 U A G U G U A A = 130
> intloop22 U A G U G U C A = 30
> intloop22 U A G U G U G A = 110
> intloop22 U A G U G U U A = 200
> intloop22 U A U U G A A A = 200
> intloop22 U A U U G A C A = 200
> intloop22 U A U U G A G A = 200
> intloop22 U A U U G A U A = 200
> intloop22 U A U U G C A A = 310
> intloop22 U A U U G C C A = 250
> intloop22 U A U U G C G A = 240
> intloop22 U A U U G C U A = 200
> intloop22 U A U U G G A A = 230
> intloop22 U A U U G G C A = 130
> intloop22 U A U U G G G A = 210
> intloop22 U A U U G G U A = 200
> intloop22 U A U U G U A A = 270
> intloop22 U A U U G U C A = 170
> intloop22 U A U U G U G A = 240
> intloop22 U A U U G U U A = 200
> intloop22 U C A U G A A A = 230
> intloop22 U C A U G A C A = 340
> intloop22 U C A U G A G A = 200
> intloop22 U C A U G A U A = 310
> intloop22 U C A U G C A A = 210
> intloop22 U C A U G C C A = 250
> intloop22 U C A U G C G A = 200
> intloop22 U C A U G C U A = 220
> intloop22 U C A U G G A A = 110
> intloop22 U C A U G G C A = 250
> intloop22 U C A U G G G A = 200
> intloop22 U C A U G G U A = 220
> intloop22 U C A U G U A A = 200
> intloop22 U C A U G U C A = 200
> intloop22 U C A U G U G A = 200
> intloop22 U C A U G U U A = 200
> intloop22 U C C U G A A A = 220
> intloop22 U C C U G A C A = 260
> intloop22 U C C U G A G A = 200
> intloop22 U C C U G A U A = 230
> intloop22 U C C U G C A A = 200
> intloop22 U C C U G C C A = 250
> intloop22 U C C U G C G A = 200
> intloop22 U C C U G C U A = 220
> intloop22 U C C U G G A A = 200
> intloop22 U C C U G G C A = 200
> intloop22 U C C U G G G A = 200
> intloop22 U C C U G G U A = 200
> intloop22 U C C U G U A A = 200
> intloop22 U C C U G U C A = 250
> intloop22 U C C U G U G A = 200
> intloop22 U C C U G U U A = 220
> intloop22 U C G U G A A A = 110
> intloop22 U C G U G A C A = 250
> intloop22 U C G U G A G A = 200
> intloop22 U C G U G A U A = 220
> intloop22 U C G U G C A A = 200
> intloop22 U C G U G C C A = 200
> intloop22 U C G U G C G A = 200
> intloop22 U C G U G C U A = 200
> intloop22 U C G U G G A A = 160
> intloop22 U C G U G G C A = 270
> intloop22 U C G U G G G A = 200
> intloop22 U C G U G G U A = 240
> intloop22 U C G U G U A A = -10
> intloop22 U C G U G U C A = 130
> intloop22 U C G U G U G A = 200
> intloop22 U C G U G U U A = 100
> intloop22 U C U U G A A A = 200
> intloop22 U C U U G A C A = 200
> intloop22 U C U U G A G A = 200
> intloop22 U C U U G A U A = 200
> intloop22 U C U U G C A A = 200
> intloop22 U C U U G C C A = 250
> intloop22 U C U U G C G A = 200
> intloop22 U C U U G C U A = 220
> intloop22 U C U U G G A A = 90
> intloop22 U C U U G G C A = 230
> intloop22 U C U U G G G A = 200
> intloop22 U C U U G G U A = 200
> intloop22 U C U U G U A A = 120
> intloop22 U C U U G U C A = 170
> intloop22 U C U U G U G A = 200
> intloop22 U C U U G U U A = 140
> intloop22 U G A U G A A A = 170
> intloop22 U G A U G A C A = 200
> intloop22 U G A U G A G A = 210
> intloop22 U G A U G A U A = 220
> intloop22 U G A U G C A A = 150
> intloop22 U G A U G C C A = 200
> intloop22 U G A U G C G A = 190
> intloop22 U G A U G C U A = 100
> intloop22 U G A U G G A A = 50
> intloop22 U G A U G G C A = 200
> intloop22 U G A U G G G A = 90
> intloop22 U G A U G G U A = 180
> intloop22 U G A U G U A A = 200
> intloop22 U G A U G U C A = 200
> intloop22 U G A U G U G A = 200
> intloop22 U G A U G U U A = 200
> intloop22 U G C U G A A A = 160
> intloop22 U G C U G A C A = 200
> intloop22 U G C U G A G A = 200
> intloop22 U G C U G A U A = 110
> intloop22 U G C U G C A A = 240
> intloop22 U G C U G C C A = 200
> intloop22 U G C U G C G A = 240
> intloop22 U G C U G C U A = 190
> intloop22 U G C U G G A A = 200
> intloop22 U G C U G G C A = 200
> intloop22 U G C U G G G A = 200
> intloop22 U G C U G G U A = 200
> intloop22 U G C U G U A A = 240
> intloop22 U G C U G U C A = 200
> intloop22 U G C U G U G A = 240
> intloop22 U G C U G U U A = 190
> intloop22 U G G U G A A A = 50
> intloop22 U G G U G A C A = 200
> intloop22 U G G U G A G A = 90
> intloop22 U G G U G A U A = 180
> intloop22 U G G U G C A A = 200
> intloop22 U G G U G C C A = 200
> intloop22 U G G U G C G A = 200
> intloop22 U G G U G C U A = 200
> intloop22 U G G U G G A A = 100
> intloop22 U G G U G G C A = 200
> intloop22 U G G U G G G A = 140
> intloop22 U G G U G G U A = 150
> intloop22 U G G U G U A A = 110
> intloop22 U G G U G U C A = 200
> intloop22 U G G U G U G A = 70
> intloop22 U G G U G U U A = -120
> intloop22 U G U U G A A A = 200
> intloop22 U G U U G A C A = 200
> intloop22 U G U U G A G A = 200
> intloop22 U G U U G A U A = 200
> intloop22 U G U U G C A A = 240
> intloop22 U G U U G C C A = 200
> intloop22 U G U U G C G A = 240
> intloop22 U G U U G C U A = 190
> intloop22 U G U U G G A A = 210
> intloop22 U G U U G G C A = 200
> intloop22 U G U U G G G A = 170
> intloop22 U G U U G G U A = -20
> intloop22 U G U U G U A A = 240
> intloop22 U G U U G U C A = 200
> intloop22 U G U U G U G A = 200
> intloop22 U G U U G U U A = 190
> intloop22 U U A U G A A A = 200
> intloop22 U U A U G A C A = 340
> intloop22 U U A U G A G A = 100
> intloop22 U U A U G A U A = 290
> intloop22 U U A U G C A A = 200
> intloop22 U U A U G C C A = 250
> intloop22 U U A U G C G A = -30
> intloop22 U U A U G C U A = 170
> intloop22 U U A U G G A A = 200
> intloop22 U U A U G G C A = 250
> intloop22 U U A U G G G A = 50
> intloop22 U U A U G G U A = 250
> intloop22 U U A U G U A A = 200
> intloop22 U U A U G U C A = 200
> intloop22 U U A U G U G A = 200
> intloop22 U U A U G U U A = 200
> intloop22 U U C U G A A A = 200
> intloop22 U U C U G A C A = 260
> intloop22 U U C U G A G A = -20
> intloop22 U U C U G A U A = 180
> intloop22 U U C U G C A A = 200
> intloop22 U U C U G C C A = 250
> intloop22 U U C U G C G A = 70
> intloop22 U U C U G C U A = 160
> intloop22 U U C U G G A A = 200
> intloop22 U U C U G G C A = 200
> intloop22 U U C U G G G A = 200
> intloop22 U U C U G G U A = 200
> intloop22 U U C U G U A A = 200
> intloop22 U U C U G U C A = 250
> intloop22 U U C U G U G A = 70
> intloop22 U U C U G U U A = 160
> intloop22 U U G U G A A A = 200
> intloop22 U U G U G A C A = 250
> intloop22 U U G U G A G A = 50
> intloop22 U U G U G A U A = 250
> intloop22 U U G U G C A A = 200
> intloop22 U U G U G C C A = 200
> intloop22 U U G U G C G A = 200
> intloop22 U U G U G C U A = 200
> intloop22 U U G U G G A A = 200
> intloop22 U U G U G G C A = 270
> intloop22 U U G U G G G A = 30
> intloop22 U U G U G G U A = 220
> intloop22 U U G U G U A A = 200
> intloop22 U U G U G U C A = 130
> intloop22 U U G U G U G A = -250
> intloop22 U U G U G U U A = 130
> intloop22 U U U U G A A A = 200
> intloop22 U U U U G A C A = 200
> intloop22 U U U U G A G A = 200
> intloop22 U U U U G A U A = 200
> intloop22 U U U U G C A A = 200
> intloop22 U U U U G C C A = 250
> intloop22 U U U U G C G A = 70
> intloop22 U U U U G C U A = 160
> intloop22 U U U U G G A A = 200
> intloop22 U U U U G G C A = 230
> intloop22 U U U U G G G A = -150
> intloop22 U U U U G G U A = 230
> intloop22 U U U U G U A A = 200
> intloop22 U U U U G U C A = 170
> intloop22 U U U U G U G A = 70
> intloop22 U U U U G U U A = 80
> intloop22 U A A G U A A A = 280
> intloop22 U A A G U A C A = 280
> intloop22 U A A G U A G A = 170
> intloop22 U A A G U A U A = 200
> intloop22 U A A G U C A A = 230
> intloop22 U A A G U C C A = 230
> intloop22 U A A G U C G A = 130
> intloop22 U A A G U C U A = 200
> intloop22 U A A G U G A A = 170
> intloop22 U A A G U G C A = 170
> intloop22 U A A G U G G A = 70
> intloop22 U A A G U G U A = 200
> intloop22 U A A G U U A A = 200
> intloop22 U A A G U U C A = 200
> intloop22 U A A G U U G A = 200
> intloop22 U A A G U U U A = 200
> intloop22 U A C G U A A A = 280
> intloop22 U A C G U A C A = 280
> intloop22 U A C G U A G A = 170
> intloop22 U A C G U A U A = 200
> intloop22 U A C G U C A A = 340
> intloop22 U A C G U C C A = 280
> intloop22 U A C G U C G A = 270
> intloop22 U A C G U C U A = 200
> intloop22 U A C G U G A A = 200
> intloop22 U A C G U G C A = 200
> intloop22 U A C G U G G A = 200
> intloop22 U A C G U G U A = 200
> intloop22 U A C G U U A A = 340
> intloop22 U A C G U U C A = 280
> intloop22 U A C G U U G A = 270
> intloop22 U A C G U U U A = 200
> intloop22 U A G G U A A A = 170
> intloop22 U A G G U A C A = 170
> intloop22 U A G G U A G A = 70
> intloop22 U A G G U A U A = 200
> intloop22 U A G G U C A A = 200
> intloop22 U A G G U C C A = 200
> intloop22 U A G G U C G A = 200
> intloop22 U A G G U C U A = 200
> intloop22 U A G G U G A A = 210
> intloop22 U A G G U G C A = 210
> intloop22 U A G G U G G A = 110
> intloop22 U A G G U G U A = 200
> intloop22 U A G G U U A A = 100
> intloop22 U A G G U U C A = 0
> intloop22 U A G G U U G A = 70
> intloop22 U A G G U U U A = 200
> intloop22 U A U G U A A A = 200
> intloop22 U A U G U A C A = 200
> intloop22 U A U G U A G A = 200
> intloop22 U A U G U A U A = 200
> intloop22 U A U G U C A A = 310
> intloop22 U A U G U C C A = 250
> intloop22 U A U G U C G A = 240
> intloop22 U A U G U C U A = 200
> intloop22 U A U G U G A A = 220
> intloop22 U A U G U G C A = 120
> intloop22 U A U G U G G A = 200
> intloop22 U A U G U G U A = 200
> intloop22 U A U G U U A A = 290
> intloop22 U A U G U U C A = 190
> intloop22 U A U G U U G A = 270
> intloop22 U A U G U U U A = 200
> intloop22 U C A G U A A A = 230
> intloop22 U C A G U A C A = 340
> intloop22 U C A G U A G A = 200
> intloop22 U C A G U A U A = 310
> intloop22 U C A G U C A A = 190
> intloop22 U C A G U C C A = 230
> intloop22 U C A G U C G A = 200
> intloop22 U C A G U C U A = 200
> intloop22 U C A G U G A A = 130
> intloop22 U C A G U G C A = 270
> intloop22 U C A G U G G A = 200
> intloop22 U C A G U G U A = 240
> intloop22 U C A G U U A A = 200
> intloop22 U C A G U U C A = 200
> intloop22 U C A G U U G A = 200
> intloop22 U C A G U U U A = 200
> intloop22 U C C G U A A A = 230
> intloop22 U C C G U A C A = 280
> intloop22 U C C G U A G A = 200
> intloop22 U C C G U A U A = 250
> intloop22 U C C G U C A A = 230
> intloop22 U C C G U C C A = 280
> intloop22 U C C G U C G A = 200
> intloop22 U C C G U C U A = 250
> intloop22 U C C G U G A A = 200
> intloop22 U C C G U G C A = 200
> intloop22 U C C G U G G A = 200
> intloop22 U C C G U G U A = 200
> intloop22 U C C G U U A A = 230
> intloop22 U C C G U U C A = 280
> intloop22 U C C G U U G A = 200
> intloop22 U C C G U U U A = 250
> intloop22 U C G G U A A A = 130
> intloop22 U C G G U A C A = 270
> intloop22 U C G G U A G A = 200
> intloop22 U C G G U A U A = 240
> intloop22 U C G G U C A A = 200
> intloop22 U C G G U C C A = 200
> intloop22 U C G G U C G A = 200
> intloop22 U C G G U C U A = 200
> intloop22 U C G G U G A A = 170
> intloop22 U C G G U G C A = 270
> intloop22 U C G G U G G A = 200
> intloop22 U C G G U G U A = 240
> intloop22 U C G G U U A A = -50
> intloop22 U C G G U U C A = 100
> intloop22 U C G G U U G A = 200
> intloop22 U C G G U U U A = 70
> intloop22 U C U G U A A A = 200
> intloop22 U C U G U A C A = 200
> intloop22 U C U G U A G A = 200
> intloop22 U C U G U A U A = 200
> intloop22 U C U G U C A A = 200
> intloop22 U C U G U C C A = 250
> intloop22 U C U G U C G A = 200
> intloop22 U C U G U C U A = 220
> intloop22 U C U G U G A A = 80
> intloop22 U C U G U G C A = 220
> intloop22 U C U G U G G A = 200
> intloop22 U C U G U G U A = 190
> intloop22 U C U G U U A A = 150
> intloop22 U C U G U U C A = 190
> intloop22 U C U G U U G A = 200
> intloop22 U C U G U U U A = 160
> intloop22 U G A G U A A A = 170
> intloop22 U G A G U A C A = 200
> intloop22 U G A G U A G A = 210
> intloop22 U G A G U A U A = 220
> intloop22 U G A G U C A A = 130
> intloop22 U G A G U C C A = 200
> intloop22 U G A G U C G A = 170
> intloop22 U G A G U C U A = 80
> intloop22 U G A G U G A A = 70
> intloop22 U G A G U G C A = 200
> intloop22 U G A G U G G A = 110
> intloop22 U G A G U G U A = 200
> intloop22 U G A G U U A A = 200
> intloop22 U G A G U U C A = 200
> intloop22 U G A G U U G A = 200
> intloop22 U G A G U U U A = 200
> intloop22 U G C G U A A A = 170
> intloop22 U G C G U A C A = 200
> intloop22 U G C G U A G A = 210
> intloop22 U G C G U A U A = 120
> intloop22 U G C G U C A A = 270
> intloop22 U G C G U C C A = 200
> intloop22 U G C G U C G A = 270
> intloop22 U G C G U C U A = 220
> intloop22 U G C G U G A A = 200
> intloop22 U G C G U G C A = 200
> intloop22 U G C G U G G A = 200
> intloop22 U G C G U G U A = 200
> intloop22 U G C G U U A A = 270
> intloop22 U G C G U U C A = 200
> intloop22 U G C G U U G A = 270
> intloop22 U G C G U U U A = 220
> intloop22 U G G G U A A A = 70
> intloop22 U G G G U A C A = 200
> intloop22 U G G G U A G A = 110
> intloop22 U G G G U A U A = 200
> intloop22 U G G G U C A A = 200
> intloop22 U G G G U C C A = 200
> intloop22 U G G G U C G A = 200
> intloop22 U G G G U C U A = 200
> intloop22 U G G G U G A A = 110
> intloop22 U G G G U G C A = 200
> intloop22 U G G G U G G A = 150
> intloop22 U G G G U G U A = 160
> intloop22 U G G G U U A A = 70
> intloop22 U G G G U U C A = 200
> intloop22 U G G G U U G A = 30
> intloop22 U G G G U U U A = -160
> intloop22 U G U G U A A A = 200
> intloop22 U G U G U A C A = 200
> intloop22 U G U G U A G A = 200
> intloop22 U G U G U A U A = 200
> intloop22 U G U G U C A A = 240
> intloop22 U G U G U C C A = 200
> intloop22 U G U G U C G A = 240
> intloop22 U G U G U C U A = 190
> intloop22 U G U G U G A A = 200
> intloop22 U G U G U G C A = 200
> intloop22 U G U G U G G A = 160
> intloop22 U G U G U G U A = -30
> intloop22 U G U G U U A A = 270
> intloop22 U G U G U U C A = 200
> intloop22 U G U G U U G A = 230
> intloop22 U G U G U U U A = 220
> intloop22 U U A G U A A A = 200
> intloop22 U U A G U A C A = 340
> intloop22 U U A G U A G A = 100
> intloop22 U U A G U A U A = 290
> intloop22 U U A G U C A A = 200
> intloop22 U U A G U C C A = 230
> intloop22 U U A G U C G A = -50
> intloop22 U U A G U C U A = 150
> intloop22 U U A G U G A A = 200
> intloop22 U U A G U G C A = 270
> intloop22 U U A G U G G A = 70
> intloop22 U U A G U G U A = 270
> intloop22 U U A G U U A A = 200
> intloop22 U U A G U U C A = 200
> intloop22 U U A G U U G A = 200
> intloop22 U U A G U U U A = 200
> intloop22 U U C G U A A A = 200
> intloop22 U U C G U A C A = 280
> intloop22 U U C G U A G A = 0
> intloop22 U U C G U A U A = 190
> intloop22 U U C G U C A A = 200
> intloop22 U U C G U C C A = 280
> intloop22 U U C G U C G A = 100
> intloop22 U U C G U C U A = 190
> intloop22 U U C G U G A A = 200
> intloop22 U U C G U G C A = 200
> intloop22 U U C G U G G A = 200
> intloop22 U U C G U G U A = 200
> intloop22 U U C G U U A A = 200
> intloop22 U U C G U U C A = 280
> intloop22 U U C G U U G A = 100
> intloop22 U U C G U U U A = 190
> intloop22 U U G G U A A A = 200
> intloop22 U U G G U A C A = 270
> intloop22 U U G G U A G A = 70
> intloop22 U U G G U A U A = 270
> intloop22 U U G G U C A A = 200
> intloop22 U U G G U C C A = 200
> intloop22 U U G G U C G A = 200
> intloop22 U U G G U C U A = 200
> intloop22 U U G G U G A A = 200
> intloop22 U U G G U G C A = 270
> intloop22 U U G G U G G A = 30
> intloop22 U U G G U G U A = 230
> intloop22 U U G G U U A A = 200
> intloop22 U U G G U U C A = 100
> intloop22 U U G G U U G A = -290
> intloop22 U U G G U U U A = 90
> intloop22 U U U G U A A A = 200
> intloop22 U U U G U A C A = 200
> intloop22 U U U G U A G A = 200
> intloop22 U U U G U A U A = 200
> intloop22 U U U G U C A A = 200
> intloop22 U U U G U C C A = 250
> intloop22 U U U G U C G A = 70
> intloop22 U U U G U C U A = 160
> intloop22 U U U G U G A A = 200
> intloop22 U U U G U G C A = 220
> intloop22 U U U G U G G A = -160
> intloop22 U U U G U G U A = 220
> intloop22 U U U G U U A A = 200
> intloop22 U U U G U U C A = 190
> intloop22 U U U G U U G A = 90
> intloop22 U U U G U U U A = 110
> intloop22 U A A U A A A A = 280
> intloop22 U A A U A A C A = 280
> intloop22 U A A U A A G A = 170
> intloop22 U A A U A A U A = 200
> intloop22 U A A U A C A A = 250
> intloop22 U A A U A C C A = 250
> intloop22 U A A U A C G A = 150
> intloop22 U A A U A C U A = 200
> intloop22 U A A U A G A A = 150
> intloop22 U A A U A G C A = 150
> intloop22 U A A U A G G A = 50
> intloop22 U A A U A G U A = 200
> intloop22 U A A U A U A A = 200
> intloop22 U A A U A U C A = 200
> intloop22 U A A U A U G A = 200
> intloop22 U A A U A U U A = 200
> intloop22 U A C U A A A A = 260
> intloop22 U A C U A A C A = 260
> intloop22 U A C U A A G A = 160
> intloop22 U A C U A A U A = 200
> intloop22 U A C U A C A A = 310
> intloop22 U A C U A C C A = 250
> intloop22 U A C U A C G A = 240
> intloop22 U A C U A C U A = 200
> intloop22 U A C U A G A A = 200
> intloop22 U A C U A G C A = 200
> intloop22 U A C U A G G A = 200
> intloop22 U A C U A G U A = 200
> intloop22 U A C U A U A A = 310
> intloop22 U A C U A U C A = 250
> intloop22 U A C U A U G A = 240
> intloop22 U A C U A U U A = 200
> intloop22 U A G U A A A A = 150
> intloop22 U A G U A A C A = 150
> intloop22 U A G U A A G A = 50
> intloop22 U A G U A A U A = 200
> intloop22 U A G U A C A A = 200
> intloop22 U A G U A C C A = 200
> intloop22 U A G U A C G A = 200
> intloop22 U A G U A C U A = 200
> intloop22 U A G U A G A A = 210
> intloop22 U A G U A G C A = 210
> intloop22 U A G U A G G A = 100
> intloop22 U A G U A G U A = 200
> intloop22 U A G U A U A A = 130
> intloop22 U A G U A U C A = 30
> intloop22 U A G U A U G A = 110
> intloop22 U A G U A U U A = 200
> intloop22 U A U U A A A A = 200
> intloop22 U A U U A A C A = 200
> intloop22 U A U U A A G A = 200
> intloop22 U A U U A A U A = 200
> intloop22 U A U U A C A A = 310
> intloop22 U A U U A C C A = 250
> intloop22 U A U U A C G A = 240
> intloop22 U A U U A C U A = 200
> intloop22 U A U U A G A A = 230
> intloop22 U A U U A G C A = 130
> intloop22 U A U U A G G A = 210
> intloop22 U A U U A G U A = 200
> intloop22 U A U U A U A A = 270
> intloop22 U A U U A U C A = 170
> intloop22 U A U U A U G A = 240
> intloop22 U A U U A U U A = 200
> intloop22 U C A U A A A A = 230
> intloop22 U C A U A A C A = 340
> intloop22 U C A U A A G A = 200
> intloop22 U C A U A A U A = 310
> intloop22 U C A U A C A A = 210
> intloop22 U C A U A C C A = 250
> intloop22 U C A U A C G A = 200
> intloop22 U C A U A C U A = 220
> intloop22 U C A U A G A A = 110
> intloop22 U C A U A G C A = 250
> intloop22 U C A U A G G A = 200
> intloop22 U C A U A G U A = 220
> intloop22 U C A U A U A A = 200
> intloop22 U C A U A U C A = 200
> intloop22 U C A U A U G A = 200
> intloop22 U C A U A U U A = 200
> intloop22 U C C U A A A A = 220
> intloop22 U C C U A A C A = 260
> intloop22 U C C U A A G A = 200
> intloop22 U C C U A A U A = 230
> intloop22 U C C U A C A A = 200
> intloop22 U C C U A C C A = 250
> intloop22 U C C U A C G A = 200
> intloop22 U C C U A C U A = 220
> intloop22 U C C U A G A A = 200
> intloop22 U C C U A G C A = 200
> intloop22 U C C U A G G A = 200
> intloop22 U C C U A G U A = 200
> intloop22 U C C U A U A A = 200
> intloop22 U C C U A U C A = 250
> intloop22 U C C U A U G A = 200
> intloop22 U C C U A U U A = 220
> intloop22 U C G U A A A A = 110
> intloop22 U C G U A A C A = 250
> intloop22 U C G U A A G A = 200
> intloop22 U C G U A A U A = 220
> intloop22 U C G U A C A A = 200
> intloop22 U C G U A C C A = 200
> intloop22 U C G U A C G A = 200
> intloop22 U C G U A C U A = 200
> intloop22 U C G U A G A A = 160
> intloop22 U C G U A G C A = 270
> intloop22 U C G U A G G A = 200
> intloop22 U C G U A G U A = 240
> intloop22 U C G U A U A A = -10
> intloop22 U C G U A U C A = 130
> intloop22 U C G U A U G A = 200
> intloop22 U C G U A U U A = 100
> intloop22 U C U U A A A A = 200
> intloop22 U C U U A A C A = 200
> intloop22 U C U U A A G A = 200
> intloop22 U C U U A A U A = 200
> intloop22 U C U U A C A A = 200
> intloop22 U C U U A C C A = 250
> intloop22 U C U U A C G A = 200
> intloop22 U C U U A C U A = 220
> intloop22 U C U U A G A A = 90
> intloop22 U C U U A G C A = 230
> intloop22 U C U U A G G A = 200
> intloop22 U C U U A G U A = 200
> intloop22 U C U U A U A A = 120
> intloop22 U C U U A U C A = 170
> intloop22 U C U U A U G A = 200
> intloop22 U C U U A U U A = 140
> intloop22 U G A U A A A A = 170
> intloop22 U G A U A A C A = 200
> intloop22 U G A U A A G A = 210
> intloop22 U G A U A A U A = 220
> intloop22 U G A U A C A A = 150
> intloop22 U G A U A C C A = 200
> intloop22 U G A U A C G A = 190
> intloop22 U G A U A C U A = 100
> intloop22 U G A U A G A A = 50
> intloop22 U G A U A G C A = 200
> intloop22 U G A U A G G A = 90
> intloop22 U G A U A G U A = 180
> intloop22 U G A U A U A A = 200
> intloop22 U G A U A U C A = 200
> intloop22 U G A U A U G A = 200
> intloop22 U G A U A U U A = 200
> intloop22 U G C U A A A A = 160
> intloop22 U G C U A A C A = 200
> intloop22 U G C U A A G A = 200
> intloop22 U G C U A A U A = 110
> intloop22 U G C U A C A A = 240
> intloop22 U G C U A C C A = 200
> intloop22 U G C U A C G A = 240
> intloop22 U G C U A C U A = 190
> intloop22 U G C U A G A A = 200
> intloop22 U G C U A G C A = 200
> intloop22 U G C U A G G A = 200
> intloop22 U G C U A G U A = 200
> intloop22 U G C U A U A A = 240
> intloop22 U G C U A U C A = 200
> intloop22 U G C U A U G A = 240
> intloop22 U G C U A U U A = 190
> intloop22 U G G U A A A A = 50
> intloop22 U G G U A A C A = 200
> intloop22 U G G U A A G A = 90
> intloop22 U G G U A A U A = 180
> intloop22 U G G U A C A A = 200
> intloop22 U G G U A C C A = 200
> intloop22 U G G U A C G A = 200
> intloop22 U G G U A C U A = 200
> intloop22 U G G U A G A A = 100
> intloop22 U G G U A G C A = 200
> intloop22 U G G U A G G A = 140
> intloop22 U G G U A G U A = 150
> intloop22 U G G U A U A A = 110
> intloop22 U G G U A U C A = 200
> intloop22 U G G U A U G A = 70
> intloop22 U G G U A U U A = -120
> intloop22 U G U U A A A A = 200
> intloop22 U G U U A A C A = 200
> intloop22 U G U U A A G A = 200
> intloop22 U G U U A A U A = 200
> intloop22 U G U U A C A A = 240
> intloop22 U G U U A C C A = 200
> intloop22 U G U U A C G A = 240
> intloop22 U G U U A C U A = 190
> intloop22 U G U U A G A A = 210
> intloop22 U G U U A G C A = 200
> intloop22 U G U U A G G A = 170
> intloop22 U G U U A G U A = -20
> intloop22 U G U U A U A A = 240
> intloop22 U G U U A U C A = 200
> intloop22 U G U U A U G A = 200
> intloop22 U G U U A U U A = 190
> intloop22 U U A U A A A A = 200
> intloop22 U U A U A A C A = 340
> intloop22 U U A U A A G A = 100
> intloop22 U U A U A A U A = 290
> intloop22 U U A U A C A A = 200
> intloop22 U U A U A C C A = 250
> intloop22 U U A U A C G A = -30
> intloop22 U U A U A C U A = 170
> intloop22 U U A U A G A A = 200
> intloop22 U U A U A G C A = 250
> intloop22 U U A U A G G A = 50
> intloop22 U U A U A G U A = 250
> intloop22 U U A U A U A A = 200
> intloop22 U U A U A U C A = 200
> intloop22 U U A U A U G A = 200
> intloop22 U U A U A U U A = 200
> intloop22 U U C U A A A A = 200
> intloop22 U U C U A A C A = 260
> intloop22 U U C U A A G A = -20
> intloop22 U U C U A A U A = 180
> intloop22 U U C U A C A A = 200
> intloop22 U U C U A C C A = 250
> intloop22 U U C U A C G A = 70
> intloop22 U U C U A C U A = 160
> intloop22 U U C U A G A A = 200
> intloop22 U U C U A G C A = 200
> intloop22 U U C U A G G A = 200
> intloop22 U U C U A G U A = 200
> intloop22 U U C U A U A A = 200
> intloop22 U U C U A U C A = 250
> intloop22 U U C U A U G A = 70
> intloop22 U U C U A U U A = 160
> intloop22 U U G U A A A A = 200
> intloop22 U U G U A A C A = 250
> intloop22 U U G U A A G A = 50
> intloop22 U U G U A A U A = 250
> intloop22 U U G U A C A A = 200
> intloop22 U U G U A C C A = 200
> intloop22 U U G U A C G A = 200
> intloop22 U U G U A C U A = 200
> intloop22 U U G U A G A A = 200
> intloop22 U U G U A G C A = 270
> intloop22 U U G U A G G A = 30
> intloop22 U U G U A G U A = 220
> intloop22 U U G U A U A A = 200
> intloop22 U U G U A U C A = 130
> intloop22 U U G U A U G A = -250
> intloop22 U U G U A U U A = 130
> intloop22 U U U U A A A A = 200
> intloop22 U U U U A A C A = 200
> intloop22 U U U U A A G A = 200
> intloop22 U U U U A A U A = 200
> intloop22 U U U U A C A A = 200
> intloop22 U U U U A C C A = 250
> intloop22 U U U U A C G A = 70
> intloop22 U U U U A C U A = 160
> intloop22 U U U U A G A A = 200
> intloop22 U U U U A G C A = 230
> intloop22 U U U U A G G A = -150
> intloop22 U U U U A G U A = 230
> intloop22 U U U U A U A A = 200
> intloop22 U U U U A U C A = 170
> intloop22 U U U U A U G A = 70
> intloop22 U U U U A U U A = 80
> intloop22 U A A A U A A A = 280
> intloop22 U A A A U A C A = 280
> intloop22 U A A A U A G A = 170
> intloop22 U A A A U A U A = 200
> intloop22 U A A A U C A A = 230
> intloop22 U A A A U C C A = 230
> intloop22 U A A A U C G A = 130
> intloop22 U A A A U C U A = 200
> intloop22 U A A A U G A A = 170
> intloop22 U A A A U G C A = 170
> intloop22 U A A A U G G A = 70
> intloop22 U A A A U G U A = 200
> intloop22 U A A A U U A A = 200
> intloop22 U A A A U U C A = 200
> intloop22 U A A A U U G A = 200
> intloop22 U A A A U U U A = 200
> intloop22 U A C A U A A A = 280
> intloop22 U A C A U A C A = 280
> intloop22 U A C A U A G A = 170
> intloop22 U A C A U A U A = 200
> intloop22 U A C A U C A A = 340
> intloop22 U A C A U C C A = 280
> intloop22 U A C A U C G A = 270
> intloop22 U A C A U C U A = 200
> intloop22 U A C A U G A A = 200
> intloop22 U A C A U G C A = 200
> intloop22 U A C A U G G A = 200
> intloop22 U A C A U G U A = 200
> intloop22 U A C A U U A A = 340
> intloop22 U A C A U U C A = 280
> intloop22 U A C A U U G A = 270
> intloop22 U A C A U U U A = 200
> intloop22 U A G A U A A A = 170
> intloop22 U A G A U A C A = 170
> intloop22 U A G A U A G A = 70
> intloop22 U A G A U A U A = 200
> intloop22 U A G A U C A A = 200
> intloop22 U A G A U C C A = 200
> intloop22 U A G A U C G A = 200
> intloop22 U A G A U C U A = 200
> intloop22 U A G A U G A A = 210
> intloop22 U A G A U G C A = 210
> intloop22 U A G A U G G A = 110
> intloop22 U A G A U G U A = 200
> intloop22 U A G A U U A A = 100
> intloop22 U A G A U U C A = 0
> intloop22 U A G A U U G A = 70
> intloop22 U A G A U U U A = 200
> intloop22 U A U A U A A A = 200
> intloop22 U A U A U A C A = 200
> intloop22 U A U A U A G A = 200
> intloop22 U A U A U A U A = 200
> intloop22 U A U A U C A A = 310
> intloop22 U A U A U C C A = 250
> intloop22 U A U A U C G A = 240
> intloop22 U A U A U C U A = 200
> intloop22 U A U A U G A A = 220
> intloop22 U A U A U G C A = 120
> intloop22 U A U A U G G A = 200
> intloop22 U A U A U G U A = 200
> intloop22 U A U A U U A A = 290
> intloop22 U A U A U U C A = 190
> intloop22 U A U A U U G A = 270
> intloop22 U A U A U U U A = 200
> intloop22 U C A A U A A A = 230
> intloop22 U C A A U A C A = 340
> intloop22 U C A A U A G A = 200
> intloop22 U C A A U A U A = 310
> intloop22 U C A A U C A A = 190
> intloop22 U C A A U C C A = 230
> intloop22 U C A A U C G A = 200
> intloop22 U C A A U C U A = 200
> intloop22 U C A A U G A A = 130
> intloop22 U C A A U G C A = 270
> intloop22 U C A A U G G A = 200
> intloop22 U C A A U G U A = 240
> intloop22 U C A A U U A A = 200
> intloop22 U C A A U U C A = 200
> intloop22 U C A A U U G A = 200
> intloop22 U C A A U U U A = 200
> intloop22 U C C A U A A A = 230
> intloop22 U C C A U A C A = 280
> intloop22 U C C A U A G A = 200
> intloop22 U C C A U A U A = 250
> intloop22 U C C A U C A A = 230
> intloop22 U C C A U C C A = 280
> intloop22 U C C A U C G A = 200
> intloop22 U C C A U C U A = 250
> intloop22 U C C A U G A A = 200
> intloop22 U C C A U G C A = 200
> intloop22 U C C A U G G A = 200
> intloop22 U C C A U G U A = 200
> intloop22 U C C A U U A A = 230
> intloop22 U C C A U U C A = 280
> intloop22 U C C A U U G A = 200
> intloop22 U C C A U U U A = 250
> intloop22 U C G A U A A A = 130
> intloop22 U C G A U A C A = 270
> intloop22 U C G A U A G A = 200
> intloop22 U C G A U A U A = 240
> intloop22 U C G A U C A A = 200
> intloop22 U C G A U C C A = 200
> intloop22 U C G A U C G A = 200
> intloop22 U C G A U C U A = 200
> intloop22 U C G A U G A A = 170
> intloop22 U C G A U G C A = 270
> intloop22 U C G A U G G A = 200
> intloop22 U C G A U G U A = 240
> intloop22 U C G A U U A A = -50
> intloop22 U C G A U U C A = 100
> intloop22 U C G A U U G A = 200
> intloop22 U C G A U U U A = 70
> intloop22 U C U A U A A A = 200
> intloop22 U C U A U A C A = 200
> intloop22 U C U A U A G A = 200
> intloop22 U C U A U A U A = 200
> intloop22 U C U A U C A A = 200
> intloop22 U C U A U C C A = 250
> intloop22 U C U A U C G A = 200
> intloop22 U C U A U C U A = 220
> intloop22 U C U A U G A A = 80
> intloop22 U C U A U G C A = 220
> intloop22 U C U A U G G A = 200
> intloop22 U C U A U G U A = 190
> intloop22 U C U A U U A A = 150
> intloop22 U C U A U U C A = 190
> intloop22 U C U A U U G A = 200
> intloop22 U C U A U U U A = 160
> intloop22 U G A A U A A A = 170
> intloop22 U G A A U A C A = 200
> intloop22 U G A A U A G A = 210
> intloop22 U G A A U A U A = 220
> intloop22 U G A A U C A A = 130
> intloop22 U G A A U C C A = 200
> intloop22 U G A A U C G A = 170
> intloop22 U G A A U C U A = 80
> intloop22 U G A A U G A A = 70
> intloop22 U G A A U G C A = 200
> intloop22 U G A A U G G A = 110
> intloop22 U G A A U G U A = 200
> intloop22 U G A A U U A A = 200
> intloop22 U G A A U U C A = 200
> intloop22 U G A A U U G A = 200
> intloop22 U G A A U U U A = 200
> intloop22 U G C A U A A A = 170
> intloop22 U G C A U A C A = 200
> intloop22 U G C A U A G A = 210
> intloop22 U G C A U A U A = 120
> intloop22 U G C A U C A A = 270
> intloop22 U G C A U C C A = 200
> intloop22 U G C A U C G A = 270
> intloop22 U G C A U C U A = 220
> intloop22 U G C A U G A A = 200
> intloop22 U G C A U G C A = 200
> intloop22 U G C A U G G A = 200
> intloop22 U G C A U G U A = 200
> intloop22 U G C A U U A A = 270
> intloop22 U G C A U U C A = 200
> intloop22 U G C A U U G A = 270
> intloop22 U G C A U U U A = 220
> intloop22 U G G A U A A A = 70
> intloop22 U G G A U A C A = 200
> intloop22 U G G A U A G A = 110
> intloop22 U G G A U A U A = 200
> intloop22 U G G A U C A A = 200
> intloop22 U G G A U C C A = 200
> intloop22 U G G A U C G A = 200
> intloop22 U G G A U C U A = 200
> intloop22 U G G A U G A A = 110
> intloop22 U G G A U G C A = 200
> intloop22 U G G A U G G A = 150
> intloop22 U G G A U G U A = 160
> intloop22 U G G A U U A A = 70
> intloop22 U G G A U U C A = 200
> intloop22 U G G A U U G A = 30
> intloop22 U G G A U U U A = -160
> intloop22 U G U A U A A A = 200
> intloop22 U G U A U A C A = 200
> intloop22 U G U A U A G A = 200
> intloop22 U G U A U A U A = 200
> intloop22 U G U A U C A A = 240
> intloop22 U G U A U C C A = 200
> intloop22 U G U A U C G A = 240
> intloop22 U G U A U C U A = 190
> intloop22 U G U A U G A A = 200
> intloop22 U G U A U G C A = 200
> intloop22 U G U A U G G A = 160
> intloop22 U G U A U G U A = -30
> intloop22 U G U A U U A A = 270
> intloop22 U G U A U U C A = 200
> intloop22 U G U A U U G A = 230
> intloop22 U G U A U U U A = 220
> intloop22 U U A A U A A A = 200
> intloop22 U U A A U A C A = 340
> intloop22 U U A A U A G A = 100
> intloop22 U U A A U A U A = 290
> intloop22 U U A A U C A A = 200
> intloop22 U U A A U C C A = 230
> intloop22 U U A A U C G A = -50
> intloop22 U U A A U C U A = 150
> intloop22 U U A A U G A A = 200
> intloop22 U U A A U G C A = 270
> intloop22 U U A A U G G A = 70
> intloop22 U U A A U G U A = 270
> intloop22 U U A A U U A A = 200
> intloop22 U U A A U U C A = 200
> intloop22 U U A A U U G A = 200
> intloop22 U U A A U U U A = 200
> intloop22 U U C A U A A A = 200
> intloop22 U U C A U A C A = 280
> intloop22 U U C A U A G A = 0
> intloop22 U U C A U A U A = 190
> intloop22 U U C A U C A A = 200
> intloop22 U U C A U C C A = 280
> intloop22 U U C A U C G A = 100
> intloop22 U U C A U C U A = 190
> intloop22 U U C A U G A A = 200
> intloop22 U U C A U G C A = 200
> intloop22 U U C A U G G A = 200
> intloop22 U U C A U G U A = 200
> intloop22 U U C A U U A A = 200
> intloop22 U U C A U U C A = 280
> intloop22 U U C A U U G A = 100
> intloop22 U U C A U U U A = 190
> intloop22 U U G A U A A A = 200
> intloop22 U U G A U A C A = 270
> intloop22 U U G A U A G A = 70
> intloop22 U U G A U A U A = 270
> intloop22 U U G A U C A A = 200
> intloop22 U U G A U C C A = 200
> intloop22 U U G A U C G A = 200
> intloop22 U U G A U C U A = 200
> intloop22 U U G A U G A A = 200
> intloop22 U U G A U G C A = 270
> intloop22 U U G A U G G A = 30
> intloop22 U U G A U G U A = 230
> intloop22 U U G A U U A A = 200
> intloop22 U U G A U U C A = 100
> intloop22 U U G A U U G A = -290
> intloop22 U U G A U U U A = 90
> intloop22 U U U A U A A A = 200
> intloop22 U U U A U A C A = 200
> intloop22 U U U A U A G A = 200
> intloop22 U U U A U A U A = 200
> intloop22 U U U A U C A A = 200
> intloop22 U U U A U C C A = 250
> intloop22 U U U A U C G A = 70
> intloop22 U U U A U C U A = 160
> intloop22 U U U A U G A A = 200
> intloop22 U U U A U G C A = 220
> intloop22 U U U A U G G A = -160
> intloop22 U U U A U G U A = 220
> intloop22 U U U A U U A A = 200
> intloop22 U U U A U U C A = 190
> intloop22 U U U A U U G A = 90
> intloop22 U U U A U U U A = 110
> intloop22 _ _ _ _ _ _ _ _ = 340
